@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,6 +22,7 @@
 
 #include <glib.h>
 #include <libxml/tree.h>
+#include <libxml/xpath.h>               // xmlXPathObject, etc.
 
 #include <crm/crm.h>
 #include <crm/cib.h>
@@ -301,11 +302,45 @@ cib_common_callback_worker(uint32_t id, uint32_t flags, xmlNode * op_request,
 int32_t
 cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean privileged)
 {
+    int rc = pcmk_rc_ok;
     uint32_t id = 0;
     uint32_t flags = 0;
     uint32_t call_options = cib_none;
     pcmk__client_t *cib_client = pcmk__find_client(c);
-    xmlNode *op_request = pcmk__client_data2xml(cib_client, data, &id, &flags);
+    xmlNode *op_request = NULL;
+
+    if (cib_client == NULL) {
+        crm_trace("Invalid client %p", c);
+        return 0;
+    }
+
+    rc = pcmk__ipc_msg_append(&cib_client->buffer, data);
+
+    if (rc == pcmk_rc_ipc_more) {
+        /* We haven't read the complete message yet, so just return. */
+        return 0;
+
+    } else if (rc == pcmk_rc_ok) {
+        /* We've read the complete message and there's already a header on
+         * the front.  Pass it off for processing.
+         */
+        op_request = pcmk__client_data2xml(cib_client, &id, &flags);
+        g_byte_array_free(cib_client->buffer, TRUE);
+        cib_client->buffer = NULL;
+
+    } else {
+        /* Some sort of error occurred reassembling the message.  All we can
+         * do is clean up, log an error and return.
+         */
+        crm_err("Error when reading IPC message: %s", pcmk_rc_str(rc));
+
+        if (cib_client->buffer != NULL) {
+            g_byte_array_free(cib_client->buffer, TRUE);
+            cib_client->buffer = NULL;
+        }
+
+        return 0;
+    }
 
     if (op_request) {
         int rc = pcmk_rc_ok;
@@ -323,10 +358,6 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
         pcmk__ipc_send_ack(cib_client, id, flags, PCMK__XE_NACK, NULL,
                            CRM_EX_PROTOCOL);
         return 0;
-
-    } else if(cib_client == NULL) {
-        crm_trace("Invalid client %p", c);
-        return 0;
     }
 
     if (pcmk_is_set(call_options, cib_sync_call)) {
@@ -472,7 +503,7 @@ process_ping_reply(xmlNode *reply)
 
             if(remote_cib && remote_cib->children) {
                 // Additional debug
-                xml_calculate_changes(the_cib, remote_cib);
+                pcmk__xml_mark_changes(the_cib, remote_cib);
                 pcmk__log_xml_changes(LOG_INFO, remote_cib);
                 crm_trace("End of differences");
             }
@@ -861,9 +892,8 @@ cib_process_request(xmlNode *request, gboolean privileged,
 
     if (cib_status != pcmk_ok) {
         rc = cib_status;
-        crm_err("Operation ignored, cluster configuration is invalid."
-                " Please repair and restart: %s", pcmk_strerror(cib_status));
-
+        crm_err("Ignoring request because cluster configuration is invalid "
+                "(please repair and restart): %s", pcmk_strerror(rc));
         op_reply = create_cib_reply(op, call_id, client_id, call_options, rc,
                                     the_cib);
 
@@ -1021,12 +1051,13 @@ contains_config_change(xmlNode *diff)
     bool changed = false;
 
     if (diff) {
-        xmlXPathObject *xpathObj = xpath_search(diff, XPATH_CONFIG_CHANGE);
+        xmlXPathObject *xpathObj = pcmk__xpath_search(diff->doc,
+                                                      XPATH_CONFIG_CHANGE);
 
-        if (numXpathResults(xpathObj) > 0) {
+        if (pcmk__xpath_num_results(xpathObj) > 0) {
             changed = true;
         }
-        freeXpathObject(xpathObj);
+        xmlXPathFreeObject(xpathObj);
     }
     return changed;
 }
@@ -1242,8 +1273,9 @@ cib_peer_callback(xmlNode * msg, void *private_data)
 static gboolean
 cib_force_exit(gpointer data)
 {
-    crm_notice("Forcing exit!");
-    terminate_cib(__func__, CRM_EX_ERROR);
+    crm_notice("Exiting immediately after %s without shutdown acknowledgment",
+               pcmk__readable_interval(EXIT_ESCALATION_MS));
+    terminate_cib(CRM_EX_ERROR);
     return FALSE;
 }
 
@@ -1264,7 +1296,8 @@ initiate_exit(void)
 
     active = pcmk__cluster_num_active_nodes();
     if (active < 2) { // This is the last active node
-        terminate_cib(__func__, 0);
+        crm_info("Exiting without sending shutdown request (no active peers)");
+        terminate_cib(CRM_EX_OK);
         return;
     }
 
@@ -1353,15 +1386,12 @@ extern int remote_tls_fd;
  * \internal
  * \brief Close remote sockets, free the global CIB and quit
  *
- * \param[in] caller           Name of calling function (for log message)
- * \param[in] fast             If -1, skip disconnect; if positive, exit that
+ * \param[in] exit_status  What exit status to use (if -1, use CRM_EX_OK, but
+ *                         skip disconnecting from the cluster layer)
  */
 void
-terminate_cib(const char *caller, int fast)
+terminate_cib(int exit_status)
 {
-    crm_info("%s: Exiting%s...", caller,
-             (fast > 0)? " fast" : mainloop ? " from mainloop" : "");
-
     if (remote_fd > 0) {
         close(remote_fd);
         remote_fd = 0;
@@ -1373,27 +1403,30 @@ terminate_cib(const char *caller, int fast)
 
     uninitializeCib();
 
-    if (fast > 0) {
-        /* Quit fast on error */
+    // Exit immediately on error
+    if (exit_status > CRM_EX_OK) {
         pcmk__stop_based_ipc(ipcs_ro, ipcs_rw, ipcs_shm);
-        crm_exit(fast);
+        crm_exit(exit_status);
+        return;
+    }
 
-    } else if ((mainloop != NULL) && g_main_loop_is_running(mainloop)) {
-        /* Quit via returning from the main loop. If fast == -1, we skip the
-         * disconnect here, and it will be done when the main loop returns
-         * (this allows the peer status callback to avoid messing with the
-         * peer caches).
+    if ((mainloop != NULL) && g_main_loop_is_running(mainloop)) {
+        /* Quit via returning from the main loop. If exit_status has the special
+         * value -1, we skip the disconnect here, and it will be done when the
+         * main loop returns (this allows the peer status callback to avoid
+         * messing with the peer caches).
          */
-        if (fast == 0) {
+        if (exit_status == CRM_EX_OK) {
             pcmk_cluster_disconnect(crm_cluster);
         }
         g_main_loop_quit(mainloop);
-
-    } else {
-        /* Quit via clean exit. Even the peer status callback can disconnect
-         * here, because we're not returning control to the caller. */
-        pcmk_cluster_disconnect(crm_cluster);
-        pcmk__stop_based_ipc(ipcs_ro, ipcs_rw, ipcs_shm);
-        crm_exit(CRM_EX_OK);
+        return;
     }
+
+    /* Exit cleanly. Even the peer status callback can disconnect here, because
+     * we're not returning control to the caller.
+     */
+    pcmk_cluster_disconnect(crm_cluster);
+    pcmk__stop_based_ipc(ipcs_ro, ipcs_rw, ipcs_shm);
+    crm_exit(CRM_EX_OK);
 }
