@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,6 +9,7 @@
 
 #include <crm_internal.h>
 
+#include <inttypes.h>               // PRIx64
 #include <sys/param.h>
 #include <stdio.h>
 #include <stdint.h>                 // uint64_t
@@ -27,60 +28,12 @@
 //! Triggers an FSA invocation
 static crm_trigger_t *fsa_trigger = NULL;
 
-#define DOT_PREFIX "actions:trace: "
-#define do_dot_log(fmt, args...)     crm_trace( fmt, ##args)
-
 static void do_state_transition(enum crmd_fsa_state cur_state,
                                 enum crmd_fsa_state next_state,
                                 fsa_data_t *msg_data);
 
 void s_crmd_fsa_actions(fsa_data_t * fsa_data);
 void log_fsa_input(fsa_data_t * stored_msg);
-void init_dotfile(void);
-
-void
-init_dotfile(void)
-{
-    do_dot_log(DOT_PREFIX "digraph \"g\" {");
-    do_dot_log(DOT_PREFIX "	size = \"30,30\"");
-    do_dot_log(DOT_PREFIX "	graph [");
-    do_dot_log(DOT_PREFIX "		fontsize = \"12\"");
-    do_dot_log(DOT_PREFIX "		fontname = \"Times-Roman\"");
-    do_dot_log(DOT_PREFIX "		fontcolor = \"black\"");
-    do_dot_log(DOT_PREFIX "		bb = \"0,0,398.922306,478.927856\"");
-    do_dot_log(DOT_PREFIX "		color = \"black\"");
-    do_dot_log(DOT_PREFIX "	]");
-    do_dot_log(DOT_PREFIX "	node [");
-    do_dot_log(DOT_PREFIX "		fontsize = \"12\"");
-    do_dot_log(DOT_PREFIX "		fontname = \"Times-Roman\"");
-    do_dot_log(DOT_PREFIX "		fontcolor = \"black\"");
-    do_dot_log(DOT_PREFIX "		shape = \"ellipse\"");
-    do_dot_log(DOT_PREFIX "		color = \"black\"");
-    do_dot_log(DOT_PREFIX "	]");
-    do_dot_log(DOT_PREFIX "	edge [");
-    do_dot_log(DOT_PREFIX "		fontsize = \"12\"");
-    do_dot_log(DOT_PREFIX "		fontname = \"Times-Roman\"");
-    do_dot_log(DOT_PREFIX "		fontcolor = \"black\"");
-    do_dot_log(DOT_PREFIX "		color = \"black\"");
-    do_dot_log(DOT_PREFIX "	]");
-    do_dot_log(DOT_PREFIX "// special nodes");
-    do_dot_log(DOT_PREFIX "	\"S_PENDING\" ");
-    do_dot_log(DOT_PREFIX "	[");
-    do_dot_log(DOT_PREFIX "	 color = \"blue\"");
-    do_dot_log(DOT_PREFIX "	 fontcolor = \"blue\"");
-    do_dot_log(DOT_PREFIX "	 ]");
-    do_dot_log(DOT_PREFIX "	\"S_TERMINATE\" ");
-    do_dot_log(DOT_PREFIX "	[");
-    do_dot_log(DOT_PREFIX "	 color = \"red\"");
-    do_dot_log(DOT_PREFIX "	 fontcolor = \"red\"");
-    do_dot_log(DOT_PREFIX "	 ]");
-    do_dot_log(DOT_PREFIX "// DC only nodes");
-    do_dot_log(DOT_PREFIX "	\"S_INTEGRATION\" [ fontcolor = \"green\" ]");
-    do_dot_log(DOT_PREFIX "	\"S_POLICY_ENGINE\" [ fontcolor = \"green\" ]");
-    do_dot_log(DOT_PREFIX "	\"S_TRANSITION_ENGINE\" [ fontcolor = \"green\" ]");
-    do_dot_log(DOT_PREFIX "	\"S_RELEASE_DC\" [ fontcolor = \"green\" ]");
-    do_dot_log(DOT_PREFIX "	\"S_IDLE\" [ fontcolor = \"green\" ]");
-}
 
 static void
 do_fsa_action(fsa_data_t * fsa_data, long long an_action,
@@ -90,7 +43,6 @@ do_fsa_action(fsa_data_t * fsa_data, long long an_action,
                                 enum crmd_fsa_input cur_input, fsa_data_t * msg_data))
 {
     controld_clear_fsa_action_flags(an_action);
-    crm_trace(DOT_PREFIX "\t// %s", fsa_action2string(an_action));
     function(an_action, fsa_data->fsa_cause, controld_globals.fsa_state,
              fsa_data->fsa_input, fsa_data);
 }
@@ -262,12 +214,6 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
          */
         if (last_state != globals->fsa_state) {
             do_state_transition(last_state, globals->fsa_state, fsa_data);
-        } else {
-            do_dot_log(DOT_PREFIX "\t// FSA input: State=%s \tCause=%s"
-                       " \tInput=%s \tOrigin=%s() \tid=%d",
-                       fsa_state2string(globals->fsa_state),
-                       fsa_cause2string(fsa_data->fsa_cause),
-                       fsa_input2string(fsa_data->fsa_input), fsa_data->origin, fsa_data->id);
         }
 
         /* start doing things... */
@@ -279,9 +225,10 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
         || (controld_globals.fsa_actions != A_NOTHING)
         || pcmk_is_set(controld_globals.flags, controld_fsa_is_stalled)) {
 
-        crm_debug("Exiting the FSA: queue=%d, fsa_actions=%#llx, stalled=%s",
+        crm_debug("Exiting the FSA: queue=%d, fsa_actions=%" PRIx64
+                  ", stalled=%s",
                   g_list_length(controld_globals.fsa_message_queue),
-                  (unsigned long long) controld_globals.fsa_actions,
+                  controld_globals.fsa_actions,
                   pcmk__flag_text(controld_globals.flags,
                                   controld_fsa_is_stalled));
     } else {
@@ -406,9 +353,6 @@ s_crmd_fsa_actions(fsa_data_t * fsa_data)
                                A_ELECTION_COUNT)) {
             do_fsa_action(fsa_data, A_ELECTION_COUNT, do_election_count_vote);
 
-        } else if (pcmk_is_set(controld_globals.fsa_actions, A_LRM_EVENT)) {
-            do_fsa_action(fsa_data, A_LRM_EVENT, do_lrm_event);
-
             /*
              * High priority actions
              */
@@ -505,9 +449,9 @@ s_crmd_fsa_actions(fsa_data_t * fsa_data)
 
             /* Error checking and reporting */
         } else {
-            crm_err("Action %s not supported " QB_XS " %#llx",
+            crm_err("Action %s not supported " QB_XS " %" PRIx64,
                     fsa_action2string(controld_globals.fsa_actions),
-                    (unsigned long long) controld_globals.fsa_actions);
+                    controld_globals.fsa_actions);
             register_fsa_error_adv(C_FSA_INTERNAL, I_ERROR, fsa_data, NULL,
                                    __func__);
         }
@@ -594,9 +538,6 @@ do_state_transition(enum crmd_fsa_state cur_state,
 
     CRM_LOG_ASSERT(cur_state != next_state);
 
-    do_dot_log(DOT_PREFIX "\t%s -> %s [ label=%s cause=%s origin=%s ]",
-               state_from, state_to, input, fsa_cause2string(cause), msg_data->origin);
-
     if (cur_state == S_IDLE || next_state == S_IDLE) {
         level = LOG_NOTICE;
     } else if (cur_state == S_NOT_DC || next_state == S_NOT_DC) {
@@ -637,7 +578,8 @@ do_state_transition(enum crmd_fsa_state cur_state,
     }
 
     if (cur_state == S_FINALIZE_JOIN && next_state == S_POLICY_ENGINE) {
-        populate_cib_nodes(node_update_quick|node_update_all, __func__);
+        populate_cib_nodes(controld_node_update_quick|controld_node_update_all,
+                           __func__);
     }
 
     switch (next_state) {
