@@ -34,8 +34,9 @@ update_dc_expected(const xmlNode *msg)
 {
     if ((controld_globals.dc_name != NULL)
         && pcmk__xe_attr_is_true(msg, PCMK__XA_DC_LEAVING)) {
-        crm_node_t *dc_node = pcmk__get_node(0, controld_globals.dc_name, NULL,
-                                             pcmk__node_search_cluster_member);
+        pcmk__node_status_t *dc_node =
+            pcmk__get_node(0, controld_globals.dc_name, NULL,
+                           pcmk__node_search_cluster_member);
 
         pcmk__update_peer_expected(__func__, dc_node, CRMD_JOINSTATE_DOWN);
     }
@@ -49,14 +50,14 @@ do_cl_join_query(long long action,
                  enum crmd_fsa_state cur_state,
                  enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
-    xmlNode *req = create_request(CRM_OP_JOIN_ANNOUNCE, NULL, NULL,
-                                  CRM_SYSTEM_DC, CRM_SYSTEM_CRMD, NULL);
+    xmlNode *req = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD, NULL,
+                                     CRM_SYSTEM_DC, CRM_OP_JOIN_ANNOUNCE, NULL);
 
     sleep(1);                   // Give the cluster layer time to propagate to the DC
     update_dc(NULL);            /* Unset any existing value so that the result is not discarded */
     crm_debug("Querying for a DC");
-    pcmk__cluster_send_message(NULL, crm_msg_crmd, req);
-    free_xml(req);
+    pcmk__cluster_send_message(NULL, pcmk_ipc_controld, req);
+    pcmk__xml_free(req);
 }
 
 /*	 A_CL_JOIN_ANNOUNCE	*/
@@ -79,13 +80,14 @@ do_cl_join_announce(long long action,
 
     if (!pcmk_is_set(controld_globals.fsa_input_register, R_STARTING)) {
         /* send as a broadcast */
-        xmlNode *req = create_request(CRM_OP_JOIN_ANNOUNCE, NULL, NULL,
-                                      CRM_SYSTEM_DC, CRM_SYSTEM_CRMD, NULL);
+        xmlNode *req = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD,
+                                         NULL, CRM_SYSTEM_DC,
+                                         CRM_OP_JOIN_ANNOUNCE, NULL);
 
         crm_debug("Announcing availability");
         update_dc(NULL);
-        pcmk__cluster_send_message(NULL, crm_msg_crmd, req);
-        free_xml(req);
+        pcmk__cluster_send_message(NULL, pcmk_ipc_controld, req);
+        pcmk__xml_free(req);
 
     } else {
         /* Delay announce until we have finished local startup */
@@ -114,7 +116,7 @@ do_cl_join_offer_respond(long long action,
 
     welcome_from = crm_element_value(input->msg, PCMK__XA_SRC);
     join_id = crm_element_value(input->msg, PCMK__XA_JOIN_ID);
-    crm_trace("Accepting cluster join offer from node %s "CRM_XS" join-%s",
+    crm_trace("Accepting cluster join offer from node %s " QB_XS " join-%s",
               welcome_from, crm_element_value(input->msg, PCMK__XA_JOIN_ID));
 
     /* we only ever want the last one */
@@ -133,7 +135,7 @@ do_cl_join_offer_respond(long long action,
     update_dc_expected(input->msg);
 
     query_call_id = cib_conn->cmds->query(cib_conn, NULL, NULL,
-                                          cib_scope_local|cib_no_children);
+                                          cib_no_children);
     fsa_register_cib_callback(query_call_id, pcmk__str_copy(join_id),
                               join_query_callback);
     crm_trace("Registered join query callback: %d", query_call_id);
@@ -165,8 +167,8 @@ join_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *
         crm_debug("Membership is in flux, not continuing join-%s", join_id);
 
     } else {
-        xmlNode *reply = NULL;
-        const crm_node_t *dc_node =
+        xmlNode *join_request = NULL;
+        const pcmk__node_status_t *dc_node =
             pcmk__get_node(0, controld_globals.dc_name, NULL,
                            pcmk__node_search_cluster_member);
 
@@ -174,18 +176,19 @@ join_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *
                   join_id, controld_globals.dc_name);
         pcmk__xe_copy_attrs(generation, output, pcmk__xaf_none);
 
-        reply = create_request(CRM_OP_JOIN_REQUEST, generation,
-                               controld_globals.dc_name, CRM_SYSTEM_DC,
-                               CRM_SYSTEM_CRMD, NULL);
+        join_request = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD,
+                                         controld_globals.dc_name,
+                                         CRM_SYSTEM_DC, CRM_OP_JOIN_REQUEST,
+                                         generation);
 
-        crm_xml_add(reply, PCMK__XA_JOIN_ID, join_id);
-        crm_xml_add(reply, PCMK_XA_CRM_FEATURE_SET, CRM_FEATURE_SET);
-        pcmk__cluster_send_message(dc_node, crm_msg_crmd, reply);
-        free_xml(reply);
+        crm_xml_add(join_request, PCMK__XA_JOIN_ID, join_id);
+        crm_xml_add(join_request, PCMK_XA_CRM_FEATURE_SET, CRM_FEATURE_SET);
+        pcmk__cluster_send_message(dc_node, pcmk_ipc_controld, join_request);
+        pcmk__xml_free(join_request);
     }
 
   done:
-    free_xml(generation);
+    pcmk__xml_free(generation);
 }
 
 void
@@ -229,7 +232,8 @@ update_conn_host_cache(xmlNode *node, void *userdata)
     const char *conn_host = crm_element_value(node, PCMK__XA_CONNECTION_HOST);
     const char *state = crm_element_value(node, PCMK__XA_NODE_STATE);
 
-    crm_node_t *remote_peer = pcmk__cluster_lookup_remote_node(remote);
+    pcmk__node_status_t *remote_peer =
+        pcmk__cluster_lookup_remote_node(remote);
 
     if (remote_peer == NULL) {
         return pcmk_rc_ok;
@@ -278,15 +282,13 @@ do_cl_join_finalize_respond(long long action,
 
     if (was_nack) {
         crm_err("Shutting down because cluster join with leader %s failed "
-                CRM_XS" join-%d NACK'd", welcome_from, join_id);
+                QB_XS " join-%d NACK'd", welcome_from, join_id);
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
         controld_set_fsa_input_flags(R_STAYDOWN);
         return;
     }
 
-    if (!AM_I_DC
-        && pcmk__str_eq(welcome_from, controld_globals.our_nodename,
-                        pcmk__str_casei)) {
+    if (!AM_I_DC && controld_is_local_node(welcome_from)) {
         crm_warn("Discarding our own welcome - we're no longer the DC");
         return;
     }
@@ -300,21 +302,23 @@ do_cl_join_finalize_respond(long long action,
     update_dc_expected(input->msg);
 
     /* record the node's feature set as a transient attribute */
-    update_attrd(controld_globals.our_nodename, CRM_ATTR_FEATURE_SET,
-                 CRM_FEATURE_SET, NULL, FALSE);
+    update_attrd(controld_globals.cluster->priv->node_name,
+                 CRM_ATTR_FEATURE_SET, CRM_FEATURE_SET, NULL, FALSE);
 
     /* send our status section to the DC */
     tmp1 = controld_query_executor_state();
     if (tmp1 != NULL) {
         xmlNode *remotes = NULL;
-        xmlNode *reply = create_request(CRM_OP_JOIN_CONFIRM, tmp1,
-                                        controld_globals.dc_name, CRM_SYSTEM_DC,
-                                        CRM_SYSTEM_CRMD, NULL);
-        const crm_node_t *dc_node =
+        xmlNode *join_confirm = pcmk__new_request(pcmk_ipc_controld,
+                                                  CRM_SYSTEM_CRMD,
+                                                  controld_globals.dc_name,
+                                                  CRM_SYSTEM_DC,
+                                                  CRM_OP_JOIN_CONFIRM, tmp1);
+        const pcmk__node_status_t *dc_node =
             pcmk__get_node(0, controld_globals.dc_name, NULL,
                            pcmk__node_search_cluster_member);
 
-        crm_xml_add_int(reply, PCMK__XA_JOIN_ID, join_id);
+        crm_xml_add_int(join_confirm, PCMK__XA_JOIN_ID, join_id);
 
         crm_debug("Confirming join-%d: sending local operation history to %s",
                   join_id, controld_globals.dc_name);
@@ -336,20 +340,21 @@ do_cl_join_finalize_respond(long long action,
 
             first_join = FALSE;
             if (start_state) {
-                set_join_state(start_state, controld_globals.our_nodename,
+                set_join_state(start_state,
+                               controld_globals.cluster->priv->node_name,
                                controld_globals.our_uuid, false);
             }
         }
 
-        pcmk__cluster_send_message(dc_node, crm_msg_crmd, reply);
-        free_xml(reply);
+        pcmk__cluster_send_message(dc_node, pcmk_ipc_controld, join_confirm);
+        pcmk__xml_free(join_confirm);
 
         if (AM_I_DC == FALSE) {
             register_fsa_input_adv(cause, I_NOT_DC, NULL, A_NOTHING, TRUE,
                                    __func__);
         }
 
-        free_xml(tmp1);
+        pcmk__xml_free(tmp1);
 
         /* Update the remote node cache with information about which node
          * is hosting the connection.
