@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -19,6 +19,8 @@
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_schedulerd.h>
 
+#include <libxml/xpath.h>               // xmlXPathObject, etc.
+
 #include <pacemaker-controld.h>
 
 static void handle_disconnect(void);
@@ -196,7 +198,7 @@ new_schedulerd_ipc_connection(void)
 
     pcmk_register_ipc_callback(schedulerd_api, scheduler_event_callback, NULL);
 
-    rc = pcmk__connect_ipc(schedulerd_api, pcmk_ipc_dispatch_main, 3);
+    rc = pcmk__connect_ipc_retry_conrefused(schedulerd_api, pcmk_ipc_dispatch_main, 3);
     if (rc != pcmk_rc_ok) {
         crm_err("Error connecting to %s: %s",
                 pcmk_ipc_name(schedulerd_api, true), pcmk_rc_str(rc));
@@ -375,7 +377,7 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
     int lpc = 0;
     const char *xpath_base = NULL;
     char *xpath_string = NULL;
-    xmlXPathObjectPtr xpathObj = NULL;
+    xmlXPathObject *xpathObj = NULL;
 
     xpath_base = pcmk_cib_xpath_for(PCMK_XE_CRM_CONFIG);
     if (xpath_base == NULL) {
@@ -386,12 +388,16 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
     xpath_string = crm_strdup_printf("%s//%s//nvpair[@name='%s']",
                                      xpath_base, PCMK_XE_CLUSTER_PROPERTY_SET,
                                      attr_name);
-    xpathObj = xpath_search(xml, xpath_string);
-    max = numXpathResults(xpathObj);
+    xpathObj = pcmk__xpath_search(xml->doc, xpath_string);
+    max = pcmk__xpath_num_results(xpathObj);
     free(xpath_string);
 
     for (lpc = 0; lpc < max; lpc++) {
-        xmlNode *match = getXpathResult(xpathObj, lpc);
+        xmlNode *match = pcmk__xpath_result(xpathObj, lpc);
+
+        if (match == NULL) {
+            continue;
+        }
         crm_trace("Forcing %s/%s = %s",
                   pcmk__xe_id(match), attr_name, attr_value);
         crm_xml_add(match, PCMK_XA_VALUE, attr_value);
@@ -435,7 +441,7 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
         crm_xml_add(xml, PCMK_XA_NAME, attr_name);
         crm_xml_add(xml, PCMK_XA_VALUE, attr_value);
     }
-    freeXpathObject(xpathObj);
+    xmlXPathFreeObject(xpathObj);
 }
 
 static void
