@@ -161,7 +161,7 @@ abort_after_delay(int abort_priority, enum pcmk__graph_next abort_action,
     abort_timer.priority = abort_priority;
     abort_timer.action = abort_action;
     abort_timer.text = abort_text;
-    abort_timer.id = g_timeout_add(delay_ms, abort_timer_popped, &abort_timer);
+    abort_timer.id = pcmk__create_timer(delay_ms, abort_timer_popped, &abort_timer);
 }
 
 static void
@@ -205,12 +205,12 @@ node_pending_timer_popped(gpointer key)
 }
 
 static void
-init_node_pending_timer(const crm_node_t *node, guint timeout)
+init_node_pending_timer(const pcmk__node_status_t *node, guint timeout)
 {
     struct abort_timer_s *node_pending_timer = NULL;
     char *key = NULL;
 
-    if (node->uuid == NULL) {
+    if (node->xml_id == NULL) {
         return;
     }
 
@@ -219,16 +219,16 @@ init_node_pending_timer(const crm_node_t *node, guint timeout)
                                                   free_node_pending_timer);
 
     // The timer is somehow already existing
-    } else if (g_hash_table_lookup(node_pending_timers, node->uuid) != NULL) {
+    } else if (g_hash_table_lookup(node_pending_timers, node->xml_id) != NULL) {
         return;
     }
 
     crm_notice("Waiting for pending %s with " PCMK_XA_ID " '%s' "
                "to join the process group (timeout=%us)",
-               node->uname ? node->uname : "node", node->uuid,
+               pcmk__s(node->name, "node"), node->xml_id,
                controld_globals.node_pending_timeout);
 
-    key = pcmk__str_copy(node->uuid);
+    key = pcmk__str_copy(node->xml_id);
     node_pending_timer = pcmk__assert_alloc(1, sizeof(struct abort_timer_s));
 
     node_pending_timer->aborted = FALSE;
@@ -238,10 +238,10 @@ init_node_pending_timer(const crm_node_t *node, guint timeout)
 
     g_hash_table_replace(node_pending_timers, key, node_pending_timer);
 
-    node_pending_timer->id = g_timeout_add_seconds(timeout,
-                                                   node_pending_timer_popped,
-                                                   key);
-    CRM_ASSERT(node_pending_timer->id != 0);
+    node_pending_timer->id = pcmk__create_timer(timeout * 1000,
+                                                node_pending_timer_popped,
+                                                key);
+    pcmk__assert(node_pending_timer->id != 0);
 }
 
 static void
@@ -255,7 +255,7 @@ remove_node_pending_timer(const char *node_uuid)
 }
 
 void
-controld_node_pending_timer(const crm_node_t *node)
+controld_node_pending_timer(const pcmk__node_status_t *node)
 {
     long long remaining_timeout = 0;
 
@@ -263,10 +263,11 @@ controld_node_pending_timer(const crm_node_t *node)
      * already part of CPG, or PCMK_OPT_NODE_PENDING_TIMEOUT is disabled, free
      * any node pending timer for it.
      */
-    if (pcmk_is_set(node->flags, crm_remote_node)
+    if (pcmk_is_set(node->flags, pcmk__node_status_remote)
         || (node->when_member <= 1) || (node->when_online > 0)
         || (controld_globals.node_pending_timeout == 0)) {
-        remove_node_pending_timer(node->uuid);
+
+        remove_node_pending_timer(node->xml_id);
         return;
     }
 
@@ -279,7 +280,7 @@ controld_node_pending_timer(const crm_node_t *node)
      * Free any node pending timer of it.
      */
     if (remaining_timeout <= 0) {
-        remove_node_pending_timer(node->uuid);
+        remove_node_pending_timer(node->xml_id);
         return;
     }
 
@@ -401,17 +402,17 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
 
     if (reason == NULL) {
         do_crm_log(level,
-                   "Transition %d aborted: %s " CRM_XS " source=%s:%d "
+                   "Transition %d aborted: %s " QB_XS " source=%s:%d "
                    "complete=%s", controld_globals.transition_graph->id,
                    abort_text, fn, line,
                    pcmk__btoa(controld_globals.transition_graph->complete));
 
     } else if(change == NULL) {
         GString *local_path = pcmk__element_xpath(reason);
-        CRM_ASSERT(local_path != NULL);
+        pcmk__assert(local_path != NULL);
 
         do_crm_log(level, "Transition %d aborted by %s.%s: %s "
-                   CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
+                   QB_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                    controld_globals.transition_graph->id, reason->name,
                    pcmk__xe_id(reason), abort_text, add[0], add[1], add[2], fn,
                    line, (const char *) local_path->str,
@@ -440,7 +441,7 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
             const char *shortpath = strrchr(path, '/');
 
             do_crm_log(level, "Transition %d aborted by deletion of %s: %s "
-                       CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
+                       QB_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                        controld_globals.transition_graph->id,
                        (shortpath? (shortpath + 1) : path), abort_text,
                        add[0], add[1], add[2], fn, line, path,
@@ -448,7 +449,7 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
 
         } else if (pcmk__xe_is(reason, PCMK_XE_NVPAIR)) {
             do_crm_log(level, "Transition %d aborted by %s doing %s %s=%s: %s "
-                       CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
+                       QB_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                        controld_globals.transition_graph->id,
                        crm_element_value(reason, PCMK_XA_ID), op,
                        crm_element_value(reason, PCMK_XA_NAME),
@@ -461,7 +462,7 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
                                                   PCMK__XA_TRANSITION_MAGIC);
 
             do_crm_log(level, "Transition %d aborted by operation %s '%s' on %s: %s "
-                       CRM_XS " magic=%s cib=%d.%d.%d source=%s:%d complete=%s",
+                       QB_XS " magic=%s cib=%d.%d.%d source=%s:%d complete=%s",
                        controld_globals.transition_graph->id,
                        crm_element_value(reason, PCMK__XA_OPERATION_KEY), op,
                        crm_element_value(reason, PCMK__META_ON_NODE),
@@ -474,7 +475,7 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
             const char *uname = pcmk__node_name_from_uuid(pcmk__xe_id(reason));
 
             do_crm_log(level, "Transition %d aborted by %s '%s' on %s: %s "
-                       CRM_XS " cib=%d.%d.%d source=%s:%d complete=%s",
+                       QB_XS " cib=%d.%d.%d source=%s:%d complete=%s",
                        controld_globals.transition_graph->id,
                        reason->name, op, pcmk__s(uname, pcmk__xe_id(reason)),
                        abort_text, add[0], add[1], add[2], fn, line,
@@ -484,7 +485,7 @@ abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
             const char *id = pcmk__xe_id(reason);
 
             do_crm_log(level, "Transition %d aborted by %s.%s '%s': %s "
-                       CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
+                       QB_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                        controld_globals.transition_graph->id,
                        reason->name, pcmk__s(id, ""), pcmk__s(op, "change"),
                        abort_text, add[0], add[1], add[2], fn, line, path,
