@@ -11,6 +11,7 @@
 
 #include <time.h>
 #include <stdlib.h>
+#include <glib.h>
 
 #include <crm/crm.h>
 #include <crm/common/xml.h>
@@ -117,8 +118,8 @@ static void
 controld_start_timer(fsa_timer_t *timer)
 {
     if (timer->source_id == 0 && timer->period_ms > 0) {
-        timer->source_id = g_timeout_add(timer->period_ms, timer->callback, (void *)timer);
-        CRM_ASSERT(timer->source_id != 0);
+        timer->source_id = pcmk__create_timer(timer->period_ms, timer->callback, timer);
+        pcmk__assert(timer->source_id != 0);
         crm_debug("Started %s (inject %s if pops after %ums, source=%d)",
                   get_timer_desc(timer), fsa_input2string(timer->fsa_input),
                   timer->period_ms, timer->source_id);
@@ -173,12 +174,12 @@ crm_timer_popped(gpointer data)
     fsa_timer_t *timer = (fsa_timer_t *) data;
 
     if (timer->log_error) {
-        crm_err("%s just popped in state %s! " CRM_XS " input=%s time=%ums",
+        crm_err("%s just popped in state %s! " QB_XS " input=%s time=%ums",
                 get_timer_desc(timer),
                 fsa_state2string(controld_globals.fsa_state),
                 fsa_input2string(timer->fsa_input), timer->period_ms);
     } else {
-        crm_info("%s just popped " CRM_XS " input=%s time=%ums",
+        crm_info("%s just popped " QB_XS " input=%s time=%ums",
                  get_timer_desc(timer), fsa_input2string(timer->fsa_input),
                  timer->period_ms);
         timer->counter++;
@@ -194,9 +195,9 @@ crm_timer_popped(gpointer data)
 
     if (timer->fsa_input == I_INTEGRATED) {
         crm_info("Welcomed: %d, Integrated: %d",
-                 crmd_join_phase_count(crm_join_welcomed),
-                 crmd_join_phase_count(crm_join_integrated));
-        if (crmd_join_phase_count(crm_join_welcomed) == 0) {
+                 crmd_join_phase_count(controld_join_welcomed),
+                 crmd_join_phase_count(controld_join_integrated));
+        if (crmd_join_phase_count(controld_join_welcomed) == 0) {
             // If we don't even have ourselves, start again
             register_fsa_error_adv(C_FSA_INTERNAL, I_ELECTION, NULL, NULL,
                                    __func__);
@@ -383,7 +384,7 @@ controld_start_recheck_timer(void)
             // We're already past the desired time
             period_ms = 500;
         } else {
-            period_ms = (guint) diff_seconds * 1000;
+            period_ms = (guint) QB_MIN(G_MAXUINT, diff_seconds * 1000LL);
         }
 
         // Use "recheck by" only if it's sooner than interval from CIB
@@ -476,7 +477,7 @@ controld_shutdown_start_countdown(guint default_period_ms)
         shutdown_escalation_timer->period_ms = default_period_ms;
     }
 
-    crm_notice("Initiating controller shutdown sequence " CRM_XS " limit=%ums",
+    crm_notice("Initiating controller shutdown sequence " QB_XS " limit=%ums",
                shutdown_escalation_timer->period_ms);
     controld_start_timer(shutdown_escalation_timer);
 }
