@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2024 the Pacemaker project contributors
+ * Copyright 2009-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -19,6 +19,7 @@
 #include <pacemaker-internal.h>
 #include <pacemaker-fenced.h>
 
+// fenced_scheduler_run() assumes it's the only place scheduler->input gets set
 static pcmk_scheduler_t *scheduler = NULL;
 
 /*!
@@ -37,7 +38,7 @@ fenced_scheduler_init(void)
         return rc;
     }
 
-    scheduler = pe_new_working_set();
+    scheduler = pcmk_new_scheduler();
     if (scheduler == NULL) {
         pcmk__output_free(logger);
         return ENOMEM;
@@ -95,7 +96,7 @@ fenced_scheduler_cleanup(void)
             pcmk__output_free(logger);
             scheduler->priv->out = NULL;
         }
-        pe_free_working_set(scheduler);
+        pcmk_free_scheduler(scheduler);
         scheduler = NULL;
     }
 }
@@ -219,13 +220,13 @@ register_if_fencing_device(gpointer data, gpointer user_data)
         if ((name == NULL) || (value == NULL)) {
             continue;
         }
-        params = stonith_key_value_add(params, name, value);
+        params = stonith__key_value_add(params, name, value);
     }
 
     xml = create_device_registration_xml(rsc_id, st_namespace_any, agent,
                                          params, rsc_provides);
-    stonith_key_value_freeall(params, 1, 1);
-    pcmk__assert(stonith_device_register(xml, TRUE) == pcmk_ok);
+    stonith__key_value_freeall(params, true, true);
+    pcmk__assert(fenced_device_register(xml, true) == pcmk_rc_ok);
     pcmk__xml_free(xml);
 }
 
@@ -233,22 +234,25 @@ register_if_fencing_device(gpointer data, gpointer user_data)
  * \internal
  * \brief Run the scheduler for fencer purposes
  *
- * \param[in] cib  Cluster's current CIB
+ * \param[in] cib  CIB to use as scheduler input
+ *
+ * \note Scheduler object is reset before returning, but \p cib is not freed.
  */
 void
 fenced_scheduler_run(xmlNode *cib)
 {
-    CRM_CHECK((cib != NULL) && (scheduler != NULL), return);
+    CRM_CHECK((cib != NULL) && (scheduler != NULL)
+              && (scheduler->input == NULL), return);
 
-    if (scheduler->priv->now != NULL) {
-        crm_time_free(scheduler->priv->now);
-        scheduler->priv->now = NULL;
-    }
-    pcmk__schedule_actions(cib, pcmk__sched_location_only
-                                |pcmk__sched_no_counts, scheduler);
+    pcmk_reset_scheduler(scheduler);
+
+    scheduler->input = cib;
+    pcmk__set_scheduler_flags(scheduler,
+                              pcmk__sched_location_only|pcmk__sched_no_counts);
+    pcmk__schedule_actions(scheduler);
     g_list_foreach(scheduler->priv->resources, register_if_fencing_device,
                    NULL);
 
     scheduler->input = NULL; // Wasn't a copy, so don't let API free it
-    pe_reset_working_set(scheduler);
+    pcmk_reset_scheduler(scheduler);
 }
