@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the Pacemaker project contributors
+ * Copyright 2015-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,7 +13,7 @@
 #include <string.h>         // strcmp()
 #include <libxml/tree.h>    // xmlNode
 
-#include <crm/common/xml_internal.h>    // enum xml_private_flags
+#include <crm/common/xml_internal.h>    // enum pcmk__xml_flags
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,8 +33,8 @@ pcmk__is_privileged(const char *user)
 
 void pcmk__enable_acl(xmlNode *acl_source, xmlNode *target, const char *user);
 
-bool pcmk__check_acl(xmlNode *xml, const char *name,
-                     enum xml_private_flags mode);
+bool pcmk__check_acl(xmlNode *xml, const char *attr_name,
+                     enum pcmk__xml_flags mode);
 
 #ifdef __cplusplus
 }
