@@ -20,6 +20,8 @@
 #include <crm/common/scheduler_types.h>     // pcmk_resource_t, pcmk_node_t
 #include <crm/common/strings_internal.h>    // pcmk__str_eq()
 
+#include <crm/common/strings_internal.h>    // pcmk__str_any_of()
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -232,6 +234,7 @@ struct pcmk__action {
     GList *actions_after;
 };
 
+void pcmk__free_action(gpointer user_data);
 char *pcmk__op_key(const char *rsc_id, const char *op_type, guint interval_ms);
 char *pcmk__notify_key(const char *rsc_id, const char *notify_type,
                        const char *op_type);
@@ -262,6 +265,36 @@ pcmk__readable_action(const char *action_name, guint interval_ms) {
     return action_name;
 }
 
+/*!
+ * \internal
+ * \brief Check whether an action raises a resource's role
+ *
+ * \param[in] action  Action to check
+ *
+ * \return \c true if \p action is "start", "promote", or "migrate_from"
+ */
+static inline bool
+pcmk__is_up_action(const char *action)
+{
+    return pcmk__str_any_of(action, PCMK_ACTION_START, PCMK_ACTION_PROMOTE,
+                            PCMK_ACTION_MIGRATE_FROM, NULL);
+}
+
+/*!
+ * \internal
+ * \brief Check whether an action lowers a resource's role
+ *
+ * \param[in] action  Action to check
+ *
+ * \return \c true if \p action is "stop", "demote", or "migrate_to"
+ */
+static inline bool
+pcmk__is_down_action(const char *action)
+{
+    return pcmk__str_any_of(action, PCMK_ACTION_STOP, PCMK_ACTION_DEMOTE,
+                            PCMK_ACTION_MIGRATE_TO, NULL);
+}
+
 #ifdef __cplusplus
 }
 #endif
