@@ -23,17 +23,13 @@ extern "C" {
 #include <stdint.h>       // uint32_t
 #include <stdbool.h>
 
-// Known resource classes
+// Known (not necessarily supported) resource classes
 #define PCMK_RESOURCE_CLASS_OCF     "ocf"
 #define PCMK_RESOURCE_CLASS_SERVICE "service"
 #define PCMK_RESOURCE_CLASS_LSB     "lsb"
 #define PCMK_RESOURCE_CLASS_SYSTEMD "systemd"
 #define PCMK_RESOURCE_CLASS_STONITH "stonith"
 #define PCMK_RESOURCE_CLASS_ALERT   "alert"
-//! \deprecated Do not use
-#define PCMK_RESOURCE_CLASS_NAGIOS  "nagios"
-//! \deprecated Do not use
-#define PCMK_RESOURCE_CLASS_UPSTART "upstart"
 
 /* Special stonith-class agent parameters interpreted directly by Pacemaker
  * (not including the pcmk_ACTION_{action,retries,timeout} parameters)
@@ -55,14 +51,15 @@ extern "C" {
 
 // Capabilities supported by a resource agent standard
 enum pcmk_ra_caps {
-    pcmk_ra_cap_none         = 0,
-    pcmk_ra_cap_provider     = (1 << 0), // Requires provider
-    pcmk_ra_cap_status       = (1 << 1), // Supports status instead of monitor
-    pcmk_ra_cap_params       = (1 << 2), // Supports parameters
-    pcmk_ra_cap_unique       = (1 << 3), // Supports unique clones
-    pcmk_ra_cap_promotable   = (1 << 4), // Supports promotable clones
-    pcmk_ra_cap_stdin        = (1 << 5), // Reads from standard input
-    pcmk_ra_cap_fence_params = (1 << 6), // Supports pcmk_monitor_timeout, etc.
+    pcmk_ra_cap_none         = 0U,
+    pcmk_ra_cap_provider     = (1U << 0), // Requires provider
+    pcmk_ra_cap_status       = (1U << 1), // Supports status instead of monitor
+    pcmk_ra_cap_params       = (1U << 2), // Supports parameters
+    pcmk_ra_cap_unique       = (1U << 3), // Supports unique clones
+    pcmk_ra_cap_promotable   = (1U << 4), // Supports promotable clones
+    pcmk_ra_cap_stdin        = (1U << 5), // Reads from standard input
+    pcmk_ra_cap_fence_params = (1U << 6), // Supports pcmk_monitor_timeout, etc.
+    pcmk_ra_cap_cli_exec     = (1U << 7), // Supports execution by crm_resource
 };
 
 uint32_t pcmk_get_ra_caps(const char *standard);
@@ -72,10 +69,6 @@ int crm_parse_agent_spec(const char *spec, char **standard, char **provider,
                          char **type);
 bool pcmk_stonith_param(const char *param);
 
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-#include <crm/common/agents_compat.h>
-#endif
-
 #ifdef __cplusplus
 }
 #endif
