@@ -18,20 +18,9 @@ extern "C" {
 enum pcmk__fc_flags {
     pcmk__fc_default   = (1 << 0),
     pcmk__fc_effective = (1 << 1),  // Don't count expired failures
-    pcmk__fc_fillers   = (1 << 2),  // If container, include filler failures
-};
 
-/*!
- * \internal
- * \enum pcmk__rsc_node
- * \brief Type of resource location lookup to perform
- */
-enum pcmk__rsc_node {
-    pcmk__rsc_node_assigned = 0,  //!< Where resource is assigned
-    pcmk__rsc_node_current  = 1,  //!< Where resource is running
-
-    // @COMPAT: Use in native_location() at a compatibility break
-    pcmk__rsc_node_pending  = 2,  //!< Where resource is pending
+    // If resource is a launcher, include failures of launched resources
+    pcmk__fc_launched  = (1 << 2),
 };
 
 #ifdef __cplusplus
