@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 the Pacemaker project contributors
+ * Copyright 2022-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -10,6 +10,10 @@
 #ifndef PCMK__CRM_COMMON_HEALTH_INTERNAL__H
 #define PCMK__CRM_COMMON_HEALTH_INTERNAL__H
 
+#include <stdbool.h>                    // bool
+
+#include <crm/common/scheduler_types.h> // pcmk_scheduler_t
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -33,6 +37,8 @@ bool pcmk__validate_health_strategy(const char *value);
 
 enum pcmk__health_strategy pcmk__parse_health_strategy(const char *value);
 
+int pcmk__health_score(const char *option, const pcmk_scheduler_t *scheduler);
+
 #ifdef __cplusplus
 }
 #endif
