@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-2022 the Pacemaker project contributors
+ * Copyright 2020-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,15 +7,15 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__LISTS_INTERNAL__H
-#define PCMK__LISTS_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_LISTS_INTERNAL__H
+#define PCMK__CRM_COMMON_LISTS_INTERNAL__H
+
+#include <glib.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include <glib.h>
-
 /*!
  * \internal
  * \brief Return the list that is \p from - \p items
@@ -33,4 +33,4 @@ GList *pcmk__subtract_lists(GList *from, const GList *items,
 }
 #endif
 
-#endif
+#endif // PCMK__CRM_COMMON_LISTS_INTERNAL__H
