@@ -48,24 +48,6 @@ extern "C" {
 #define LOG_NEVER   255
 #endif
 
-/* "Extended information" logging support */
-#ifdef QB_XS
-#define CRM_XS QB_XS
-#define crm_extended_logging(t, e) qb_log_ctl((t), QB_LOG_CONF_EXTENDED, (e))
-#else
-#define CRM_XS "|"
-
-/* A caller might want to check the return value, so we can't define this as a
- * no-op, and we can't simply define it to be 0 because gcc will then complain
- * when the value isn't checked.
- */
-static inline int
-crm_extended_logging(int t, int e)
-{
-    return 0;
-}
-#endif
-
 // @COMPAT Make internal when we can break API backward compatibility
 //! \deprecated Do not use
 extern unsigned int crm_log_level;
@@ -74,20 +56,6 @@ extern unsigned int crm_log_level;
 //! \deprecated Do not use
 extern unsigned int crm_trace_nonlog;
 
-/*! \deprecated Pacemaker library functions set this when a configuration
- *              error is found, which turns on extra messages at the end of
- *              processing. It should not be used directly and will be removed
- *              from the public C API in a future release.
- */
-extern gboolean crm_config_error;
-
-/*! \deprecated Pacemaker library functions set this when a configuration
- *              warning is found, which turns on extra messages at the end of
- *              processing. It should not be used directly and will be removed
- *              from the public C API in a future release.
- */
-extern gboolean crm_config_warning;
-
 void crm_enable_blackbox(int nsig);
 void crm_disable_blackbox(int nsig);
 void crm_write_blackbox(int nsig, const struct qb_log_callsite *callsite);
@@ -326,7 +294,7 @@ pcmk__clip_log_level(int level)
  * \note This is a macro, and \p level may be evaluated more than once.
  * \note Because crm_perror() adds the system error message and error number
  *       onto the end of fmt, that information will become extended information
- *       if CRM_XS is used inside fmt and will not show up in syslog.
+ *       if QB_XS is used inside fmt and will not show up in syslog.
  */
 #define crm_perror(level, fmt, args...) do {                                \
         uint8_t _level = pcmk__clip_log_level(level);                       \
@@ -421,12 +389,12 @@ pcmk__clip_log_level(int level)
         }                                                       \
     } while(0)
 
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-#include <crm/common/logging_compat.h>
-#endif
-
 #ifdef __cplusplus
 }
 #endif
 
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/common/logging_compat.h>
+#endif
+
 #endif
