@@ -7,8 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__OUTPUT_INTERNAL__H
-#define PCMK__OUTPUT_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_OUTPUT_INTERNAL__H
+#define PCMK__CRM_COMMON_OUTPUT_INTERNAL__H
 
 #include <stdbool.h>
 #include <stdint.h>
@@ -143,7 +143,6 @@ typedef struct pcmk__supported_format_s {
  */
 
 extern GOptionEntry pcmk__html_output_entries[];
-extern GOptionEntry pcmk__text_output_entries[];
 
 pcmk__output_t *pcmk__mk_html_output(char **argv);
 pcmk__output_t *pcmk__mk_log_output(char **argv);
@@ -154,7 +153,7 @@ pcmk__output_t *pcmk__mk_xml_output(char **argv);
 #define PCMK__SUPPORTED_FORMAT_HTML { "html", pcmk__mk_html_output, pcmk__html_output_entries }
 #define PCMK__SUPPORTED_FORMAT_LOG  { "log", pcmk__mk_log_output, NULL }
 #define PCMK__SUPPORTED_FORMAT_NONE { PCMK_VALUE_NONE, pcmk__mk_none_output, NULL }
-#define PCMK__SUPPORTED_FORMAT_TEXT { "text", pcmk__mk_text_output, pcmk__text_output_entries }
+#define PCMK__SUPPORTED_FORMAT_TEXT { "text", pcmk__mk_text_output, NULL }
 #define PCMK__SUPPORTED_FORMAT_XML  { "xml", pcmk__mk_xml_output, NULL }
 
 /*!
@@ -866,7 +865,7 @@ pcmk__output_create_html_node(pcmk__output_t *out, const char *element_name, con
                               const char *class_name, const char *text);
 
 xmlNode *pcmk__html_create(xmlNode *parent, const char *name, const char *id,
-                           const char *class);
+                           const char *class_name);
 
 /*!
  * \internal
@@ -1016,4 +1015,4 @@ GHashTable *pcmk__output_formatters(void);
 }
 #endif
 
-#endif
+#endif // PCMK__CRM_COMMON_OUTPUT_INTERNAL__H
