@@ -7,15 +7,32 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__COMMON_RESULTS_INTERNAL__H
-#define PCMK__COMMON_RESULTS_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_RESULTS_INTERNAL__H
+#define PCMK__CRM_COMMON_RESULTS_INTERNAL__H
+
+#include <stdbool.h>
 
 #include <glib.h>               // GQuark
 
+#include <crm/common/results.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern const size_t pcmk__n_rc;
 
 int pcmk__result_bounds(enum pcmk_result_type, int *lower, int *upper);
 
+_Noreturn void pcmk__abort_as(const char *file, const char *function, int line,
+                              const char *assert_condition);
+
+#define pcmk__assert(expr) do {                                     \
+        if (!(expr)) {                                              \
+            pcmk__abort_as(__FILE__, __func__, __LINE__, #expr);    \
+        }                                                           \
+    } while(0)
+
 /*!
  * \internal
  * \brief Abort without dumping core if a pointer is \c NULL
@@ -101,4 +118,8 @@ pcmk__result_ok(const pcmk__action_result_t *result)
             && (result->execution_status == PCMK_EXEC_DONE);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif // PCMK__COMMON_RESULTS_INTERNAL__H
