@@ -23,6 +23,7 @@
 #include <crm/common/agents.h>
 #include <crm/common/results.h>
 #include <crm/common/scores.h>
+#include <crm/common/strings.h>
 #include <crm/common/nvpair.h>
 
 #ifdef __cplusplus
@@ -35,33 +36,14 @@ extern "C" {
  * \ingroup core
  */
 
-/* public node attribute functions (from attrd_client.c) */
+/* public node attribute functions (from attrs.c) */
 char *pcmk_promotion_score_name(const char *rsc_id);
 
 /* public Pacemaker Remote functions (from remote.c) */
 int crm_default_remote_port(void);
 
-/* public string functions (from strings.c) */
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-gboolean crm_is_true(const char *s);
-
-int crm_str_to_boolean(const char *s, int *ret);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-long long crm_get_msec(const char *input);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-char *crm_strdup_printf(char const *format, ...) G_GNUC_PRINTF(1, 2);
-
-int pcmk_parse_interval_spec(const char *input, guint *result_ms);
-
 int compare_version(const char *version1, const char *version2);
 
-/* coverity[+kill] */
-void crm_abort(const char *file, const char *function, int line,
-               const char *condition, gboolean do_core, gboolean do_fork);
-
 /*!
  * \brief Check whether any of specified flags are set in a flag group
  *
@@ -97,29 +79,18 @@ pcmk_all_flags_set(uint64_t flag_group, uint64_t flags_to_check)
  */
 #define pcmk_is_set(g, f)   pcmk_all_flags_set((g), (f))
 
+void pcmk_common_cleanup(void);
 char *crm_md5sum(const char *buffer);
-
 char *crm_generate_uuid(void);
-
-// This belongs in ipc.h but is here for backward compatibility
-bool crm_is_daemon_name(const char *name);
-
 int crm_user_lookup(const char *name, uid_t * uid, gid_t * gid);
 int pcmk_daemon_user(uid_t *uid, gid_t *gid);
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-void crm_gnutls_global_init(void);
-#endif
-
-bool pcmk_str_is_infinity(const char *s);
-bool pcmk_str_is_minus_infinity(const char *s);
-
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-#include <crm/common/util_compat.h>
-#endif
-
 #ifdef __cplusplus
 }
 #endif
 
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/common/util_compat.h>
+#endif
+
 #endif
