@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2024 the Pacemaker project contributors
+ * Copyright 2017-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,12 +22,13 @@
 #include <crm/common/output_internal.h>
 #include <crm/common/xml_names.h>             // PCMK_XA_ID, PCMK_XE_CLONE
 
-// This file is a wrapper for other xml_*_internal.h headers
+// This file is a wrapper for other {xml_*,xpath}_internal.h headers
 #include <crm/common/xml_comment_internal.h>
 #include <crm/common/xml_element_internal.h>
 #include <crm/common/xml_idref_internal.h>
 #include <crm/common/xml_io_internal.h>
 #include <crm/common/xml_names_internal.h>
+#include <crm/common/xpath_internal.h>
 
 #include <libxml/relaxng.h>
 
@@ -137,8 +138,8 @@ do {
     }                                                                           \
 } while (0)
 
-/*
- * \enum pcmk__xml_fmt_options
+/*!
+ * \internal
  * \brief Bit flags to control format in XML logs and dumps
  */
 enum pcmk__xml_fmt_options {
@@ -162,38 +163,10 @@ enum pcmk__xml_fmt_options {
     pcmk__xml_fmt_text       = (1 << 6),
 };
 
-void pcmk__xml_init(void);
-void pcmk__xml_cleanup(void);
-
 int pcmk__xml_show(pcmk__output_t *out, const char *prefix, const xmlNode *data,
                    int depth, uint32_t options);
 int pcmk__xml_show_changes(pcmk__output_t *out, const xmlNode *xml);
 
-/* XML search strings for guest, remote and pacemaker_remote nodes */
-
-/* search string to find CIB resources entries for cluster nodes */
-#define PCMK__XP_MEMBER_NODE_CONFIG                                 \
-    "//" PCMK_XE_CIB "/" PCMK_XE_CONFIGURATION "/" PCMK_XE_NODES    \
-    "/" PCMK_XE_NODE                                                \
-    "[not(@" PCMK_XA_TYPE ") or @" PCMK_XA_TYPE "='" PCMK_VALUE_MEMBER "']"
-
-/* search string to find CIB resources entries for guest nodes */
-#define PCMK__XP_GUEST_NODE_CONFIG \
-    "//" PCMK_XE_CIB "//" PCMK_XE_CONFIGURATION "//" PCMK_XE_PRIMITIVE  \
-    "//" PCMK_XE_META_ATTRIBUTES "//" PCMK_XE_NVPAIR                    \
-    "[@" PCMK_XA_NAME "='" PCMK_META_REMOTE_NODE "']"
-
-/* search string to find CIB resources entries for remote nodes */
-#define PCMK__XP_REMOTE_NODE_CONFIG                                     \
-    "//" PCMK_XE_CIB "//" PCMK_XE_CONFIGURATION "//" PCMK_XE_PRIMITIVE  \
-    "[@" PCMK_XA_TYPE "='" PCMK_VALUE_REMOTE "']"                       \
-    "[@" PCMK_XA_PROVIDER "='pacemaker']"
-
-/* search string to find CIB node status entries for pacemaker_remote nodes */
-#define PCMK__XP_REMOTE_NODE_STATUS                                 \
-    "//" PCMK_XE_CIB "//" PCMK_XE_STATUS "//" PCMK__XE_NODE_STATE   \
-    "[@" PCMK_XA_REMOTE_NODE "='" PCMK_VALUE_TRUE "']"
-
 enum pcmk__xml_artefact_ns {
     pcmk__xml_artefact_ns_legacy_rng = 1,
     pcmk__xml_artefact_ns_legacy_xslt,
@@ -203,11 +176,8 @@ enum pcmk__xml_artefact_ns {
 
 void pcmk__strip_xml_text(xmlNode *xml);
 
-GString *pcmk__element_xpath(const xmlNode *xml);
-
 /*!
  * \internal
- * \enum pcmk__xml_escape_type
  * \brief Indicators of which XML characters to escape
  *
  * XML allows the escaping of special characters by replacing them with entity
@@ -354,7 +324,6 @@ xmlNode *pcmk__xml_copy(xmlNode *parent, xmlNode *src);
 
 /*!
  * \internal
- * \enum pcmk__xa_flags
  * \brief Flags for operations affecting XML attributes
  */
 enum pcmk__xa_flags {
@@ -373,57 +342,86 @@ enum pcmk__xa_flags {
 
 void pcmk__xml_sanitize_id(char *id);
 
-/*!
- * \internal
- * \brief Extract the ID attribute from an XML element
- *
- * \param[in] xpath String to search
- * \param[in] node  Node to get the ID for
- *
- * \return ID attribute of \p node in xpath string \p xpath
- */
-char *
-pcmk__xpath_node_id(const char *xpath, const char *node);
-
-/*!
- * \internal
- * \brief Print an informational message if an xpath query returned multiple
- *        items with the same ID.
- *
- * \param[in,out] out       The output object
- * \param[in]     search    The xpath search result, most typically the result of
- *                          calling cib->cmds->query().
- * \param[in]     name      The name searched for
- */
-void
-pcmk__warn_multiple_name_matches(pcmk__output_t *out, xmlNode *search,
-                                 const char *name);
-
 /* internal XML-related utilities */
 
-enum xml_private_flags {
-     pcmk__xf_none        = 0x0000,
-     pcmk__xf_dirty       = 0x0001,
-     pcmk__xf_deleted     = 0x0002,
-     pcmk__xf_created     = 0x0004,
-     pcmk__xf_modified    = 0x0008,
+/*!
+ * \internal
+ * \brief Flags related to XML change tracking and ACLs
+ */
+enum pcmk__xml_flags {
+    //! This flag has no effect
+    pcmk__xf_none            = UINT32_C(0),
 
-     pcmk__xf_tracking    = 0x0010,
-     pcmk__xf_processed   = 0x0020,
-     pcmk__xf_skip        = 0x0040,
-     pcmk__xf_moved       = 0x0080,
+    /*!
+     * Node was created or modified, or one of its descendants was created,
+     * modified, moved, or deleted.
+     */
+    pcmk__xf_dirty           = (UINT32_C(1) << 0),
 
-     pcmk__xf_acl_enabled = 0x0100,
-     pcmk__xf_acl_read    = 0x0200,
-     pcmk__xf_acl_write   = 0x0400,
-     pcmk__xf_acl_deny    = 0x0800,
+    //! Node was deleted (set for attribute only)
+    pcmk__xf_deleted         = (UINT32_C(1) << 1),
 
-     pcmk__xf_acl_create  = 0x1000,
-     pcmk__xf_acl_denied  = 0x2000,
-     pcmk__xf_lazy        = 0x4000,
+    //! Node was created
+    pcmk__xf_created         = (UINT32_C(1) << 2),
+
+    //! Node was modified
+    pcmk__xf_modified        = (UINT32_C(1) << 3),
+
+    /*!
+     * \brief Tracking is enabled (set for document only)
+     *
+     * Call \c pcmk__xml_commit_changes() before setting this flag if a clean
+     * start for tracking is needed.
+     */
+    pcmk__xf_tracking        = (UINT32_C(1) << 4),
+
+    //! Skip counting this node when getting a node's position among siblings
+    pcmk__xf_skip            = (UINT32_C(1) << 6),
+
+    //! Node was moved
+    pcmk__xf_moved           = (UINT32_C(1) << 7),
+
+    //! ACLs are enabled (set for document only)
+    pcmk__xf_acl_enabled     = (UINT32_C(1) << 8),
+
+    /* @TODO Consider splitting the ACL permission flags (pcmk__xf_acl_read,
+     * pcmk__xf_acl_write, pcmk__xf_acl_write, and pcmk__xf_acl_create) into a
+     * separate enum and reserving this enum for tracking-related flags.
+     *
+     * The ACL permission flags have various meanings in different contexts (for
+     * example, what permission an ACL grants or denies; what permissions the
+     * current ACL user has for a given XML node; and possibly others). And
+     * for xml_acl_t objects, they're used in exclusive mode (exactly one is
+     * set), rather than as flags.
+     */
+
+    //! ACL read permission
+    pcmk__xf_acl_read        = (UINT32_C(1) << 9),
+
+    //! ACL write permission (implies read permission in most or all contexts)
+    pcmk__xf_acl_write       = (UINT32_C(1) << 10),
+
+    //! ACL deny permission (that is, no permission)
+    pcmk__xf_acl_deny        = (UINT32_C(1) << 11),
+
+    /*!
+     * ACL create permission for attributes (if attribute exists, this is mapped
+     * to \c pcmk__xf_acl_write)
+     */
+    pcmk__xf_acl_create      = (UINT32_C(1) << 12),
+
+    //! ACLs deny the user access (set for document only)
+    pcmk__xf_acl_denied      = (UINT32_C(1) << 13),
+
+    //! Ignore attribute moves within an element (set for document only)
+    pcmk__xf_ignore_attr_pos = (UINT32_C(1) << 14),
 };
 
-void pcmk__set_xml_doc_flag(xmlNode *xml, enum xml_private_flags flag);
+void pcmk__xml_doc_set_flags(xmlDoc *doc, uint32_t flags);
+bool pcmk__xml_doc_all_flags_set(const xmlDoc *xml, uint32_t flags);
+
+void pcmk__xml_commit_changes(xmlDoc *doc);
+void pcmk__xml_mark_changes(xmlNode *old_xml, xmlNode *new_xml);
 
 bool pcmk__xml_tree_foreach(xmlNode *xml, bool (*fn)(xmlNode *, void *),
                             void *user_data);
