@@ -92,6 +92,7 @@ extern "C" {
 #define PCMK_XE_CRM_MON                     "crm_mon"
 #define PCMK_XE_CRM_MON_DISCONNECTED        "crm-mon-disconnected"
 #define PCMK_XE_CURRENT_DC                  "current_dc"
+#define PCMK_XE_DATE                        "date"
 #define PCMK_XE_DATE_EXPRESSION             "date_expression"
 #define PCMK_XE_DATE_SPEC                   "date_spec"
 #define PCMK_XE_DC                          "dc"
@@ -101,6 +102,8 @@ extern "C" {
 #define PCMK_XE_DIGESTS                     "digests"
 #define PCMK_XE_DOCKER                      "docker"
 #define PCMK_XE_DURATION                    "duration"
+#define PCMK_XE_DURATION_ENDS               "duration_ends"
+#define PCMK_XE_END                         "end"
 #define PCMK_XE_ERROR                       "error"
 #define PCMK_XE_ERRORS                      "errors"
 #define PCMK_XE_EXPRESSION                  "expression"
@@ -154,6 +157,7 @@ extern "C" {
 #define PCMK_XE_PACEMAKERD                  "pacemakerd"
 #define PCMK_XE_PARAMETER                   "parameter"
 #define PCMK_XE_PARAMETERS                  "parameters"
+#define PCMK_XE_PERIOD                      "period"
 #define PCMK_XE_PODMAN                      "podman"
 #define PCMK_XE_PORT_MAPPING                "port-mapping"
 #define PCMK_XE_POSITION                    "position"
@@ -196,6 +200,7 @@ extern "C" {
 #define PCMK_XE_SOURCE                      "source"
 #define PCMK_XE_SPECIAL                     "special"
 #define PCMK_XE_STACK                       "stack"
+#define PCMK_XE_START                       "start"
 #define PCMK_XE_STATUS                      "status"
 #define PCMK_XE_STORAGE                     "storage"
 #define PCMK_XE_STORAGE_MAPPING             "storage-mapping"
