@@ -7,17 +7,17 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PE_REMOTE__H
-#  define PE_REMOTE__H
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+#ifndef PCMK__CRM_PENGINE_REMOTE_INTERNAL__H
+#define PCMK__CRM_PENGINE_REMOTE_INTERNAL__H
 
 #include <glib.h>                   // gboolean
 #include <libxml/tree.h>            // xmlNode
 #include <crm/pengine/status.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 bool xml_contains_remote_node(xmlNode *xml);
 pcmk_resource_t *pe__resource_contains_guest_node(const pcmk_scheduler_t *scheduler,
                                                   const pcmk_resource_t *rsc);
@@ -34,4 +34,4 @@ xmlNode *pe_create_remote_xml(xmlNode *parent, const char *uname,
 }
 #endif
 
-#endif
+#endif // PCMK__CRM_PENGINE_REMOTE_INTERNAL__H
