@@ -14,7 +14,6 @@
 #  include <stdbool.h>              // bool
 #  include <crm/common/util.h>      // pcmk_is_set()
 #  include <crm/common/iso8601.h>
-#  include <crm/pengine/common.h>
 #  include <crm/pengine/pe_types.h> // pcmk_node_t, pcmk_resource_t, etc.
 #  include <crm/pengine/complex.h>
 
@@ -33,18 +32,6 @@ const char *rsc_printable_id(const pcmk_resource_t *rsc);
 // NOTE: sbd (as of at least 1.5.2) uses this
 gboolean cluster_status(pcmk_scheduler_t *scheduler);
 
-// NOTE: sbd (as of at least 1.5.2) uses this
-pcmk_scheduler_t *pe_new_working_set(void);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-void pe_free_working_set(pcmk_scheduler_t *scheduler);
-
-void set_working_set_defaults(pcmk_scheduler_t *scheduler);
-void cleanup_calculations(pcmk_scheduler_t *scheduler);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-void pe_reset_working_set(pcmk_scheduler_t *scheduler);
-
 pcmk_resource_t *pe_find_resource(GList *rsc_list, const char *id);
 pcmk_resource_t *pe_find_resource_with_flags(GList *rsc_list, const char *id,
                                              enum pe_find flags);
@@ -62,6 +49,7 @@ int pe_bundle_replicas(const pcmk_resource_t *rsc);
 #endif
 
 #if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/pengine/common.h>
 #include <crm/pengine/status_compat.h>
 #endif
 
