@@ -75,29 +75,6 @@ enum lsb_status_exitcode {
     PCMK_LSB_STATUS_INSUFFICIENT_PRIV  = 151,
 };
 
-//!@{
-//! \deprecated Do not use
-
-enum nagios_exitcode {
-    NAGIOS_STATE_OK        = 0,
-    NAGIOS_STATE_WARNING   = 1,
-    NAGIOS_STATE_CRITICAL  = 2,
-    NAGIOS_STATE_UNKNOWN   = 3,
-
-    /* This is a custom Pacemaker value (not a nagios convention), used as an
-     * intermediate value between the services library and the executor, so the
-     * executor can map it to the corresponding OCF code.
-     */
-    NAGIOS_INSUFFICIENT_PRIV = 100,
-
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-    NAGIOS_STATE_DEPENDENT   = 4,
-    NAGIOS_NOT_INSTALLED     = 101,
-#endif
-};
-
-//!@}
-
 enum svc_action_flags {
     /* On timeout, only kill pid, do not kill entire pid group */
     SVC_ACTION_LEAVE_GROUP = 0x01,
@@ -380,58 +357,12 @@ gboolean services_alert_async(svc_action_t *action,
 enum ocf_exitcode services_result2ocf(const char *standard, const char *action,
                                       int exit_status);
 
-    static inline const char *services_ocf_exitcode_str(enum ocf_exitcode code) {
-        switch (code) {
-            case PCMK_OCF_OK:
-                return "ok";
-            case PCMK_OCF_UNKNOWN_ERROR:
-                return "error";
-            case PCMK_OCF_INVALID_PARAM:
-                return "invalid parameter";
-            case PCMK_OCF_UNIMPLEMENT_FEATURE:
-                return "unimplemented feature";
-            case PCMK_OCF_INSUFFICIENT_PRIV:
-                return "insufficient privileges";
-            case PCMK_OCF_NOT_INSTALLED:
-                return "not installed";
-            case PCMK_OCF_NOT_CONFIGURED:
-                return "not configured";
-            case PCMK_OCF_NOT_RUNNING:
-                return "not running";
-            case PCMK_OCF_RUNNING_PROMOTED:
-                return "promoted";
-            case PCMK_OCF_FAILED_PROMOTED:
-                return "promoted (failed)";
-            case PCMK_OCF_DEGRADED:
-                return "OCF_DEGRADED";
-            case PCMK_OCF_DEGRADED_PROMOTED:
-                return "promoted (degraded)";
-
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-            case PCMK_OCF_NOT_SUPPORTED:
-                return "not supported (DEPRECATED STATUS)";
-            case PCMK_OCF_CANCELLED:
-                return "cancelled (DEPRECATED STATUS)";
-            case PCMK_OCF_OTHER_ERROR:
-                return "other error (DEPRECATED STATUS)";
-            case PCMK_OCF_SIGNAL:
-                return "interrupted by signal (DEPRECATED STATUS)";
-            case PCMK_OCF_PENDING:
-                return "pending (DEPRECATED STATUS)";
-            case PCMK_OCF_TIMEOUT:
-                return "timeout (DEPRECATED STATUS)";
-#endif
-            default:
-                return "unknown";
-        }
-    }
+#  ifdef __cplusplus
+}
+#  endif
 
 #if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
 #include <crm/services_compat.h>
 #endif
 
-#  ifdef __cplusplus
-}
-#  endif
-
 #endif                          /* __PCMK_SERVICES__ */
