@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-2023 the Pacemaker project contributors
+ * Copyright 2020-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,7 +8,7 @@
  */
 
 #ifndef PCMK__PCMKI_PCMKI_CLUSTER_QUERIES__H
-#  define PCMK__PCMKI_PCMKI_CLUSTER_QUERIES__H
+#define PCMK__PCMKI_PCMKI_CLUSTER_QUERIES__H
 
 #include <stdbool.h>
 #include <stdint.h>
@@ -18,6 +18,10 @@
 #include <crm/common/ipc_controld.h>
 #include <crm/common/ipc_pacemakerd.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 // CIB queries
 int pcmk__list_nodes(pcmk__output_t *out, const char *node_types,
                      bool bash_export);
@@ -45,7 +49,7 @@ int pcmk__query_node_info(pcmk__output_t *out, uint32_t *node_id,
  * \param[out]    node_name           If not \p NULL, where to store the node
  *                                    name
  * \param[in]     message_timeout_ms  How long to wait for a reply from the
- *                                    \p pacemaker-controld API. If 0,
+ *                                    controller API. If 0,
  *                                    \p pcmk_ipc_dispatch_sync will be used.
  *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
  *                                    be used.
@@ -67,4 +71,8 @@ int pcmk__pacemakerd_status(pcmk__output_t *out, const char *ipc_name,
                             unsigned int message_timeout_ms, bool show_output,
                             enum pcmk_pacemakerd_state *state);
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__PCMKI_PCMKI_CLUSTER_QUERIES__H
