@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2023 the Pacemaker project contributors
+ * Copyright 2014-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -17,6 +17,10 @@
 #include <crm/lrmd_events.h>    // lrmd_event_data_t
 #include <crm/pengine/status.h> // pcmk_resource_t, pcmk_scheduler_t
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct {
     const char *id;
     const char *node_attribute;
@@ -32,8 +36,7 @@ typedef struct {
 
 void pcmk__unpack_constraints(pcmk_scheduler_t *scheduler);
 
-void pcmk__schedule_actions(xmlNode *cib, unsigned long long flags,
-                            pcmk_scheduler_t *scheduler);
+void pcmk__schedule_actions(pcmk_scheduler_t *scheduler);
 
 GList *pcmk__copy_node_list(const GList *list, bool reset);
 
@@ -41,4 +44,8 @@ xmlNode *pcmk__create_history_xml(xmlNode *parent, lrmd_event_data_t *event,
                                   const char *caller_version, int target_rc,
                                   const char *node, const char *origin);
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__PCMKI_PCMKI_SCHEDULER__H
