@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2024 the Pacemaker project contributors
+ * Copyright 2008-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -239,6 +239,7 @@ cib_remote_callback_dispatch(gpointer user_data)
             return -1;
     }
 
+    // coverity[tainted_data] This can't easily be changed right now
     msg = pcmk__remote_message_xml(&private->callback);
     if (msg == NULL) {
         private->start_time = 0;
@@ -323,16 +324,16 @@ cib_tls_close(cib_t *cib)
         tls = NULL;
     }
 
-    if (private->command.tcp_socket) {
+    if (private->command.tcp_socket >= 0) {
         shutdown(private->command.tcp_socket, SHUT_RDWR);       /* no more receptions */
         close(private->command.tcp_socket);
     }
-    if (private->callback.tcp_socket) {
+    if (private->callback.tcp_socket >= 0) {
         shutdown(private->callback.tcp_socket, SHUT_RDWR);      /* no more receptions */
         close(private->callback.tcp_socket);
     }
-    private->command.tcp_socket = 0;
-    private->callback.tcp_socket = 0;
+    private->command.tcp_socket = -1;
+    private->callback.tcp_socket = -1;
 
     free(private->command.buffer);
     free(private->callback.buffer);
@@ -469,7 +470,6 @@ cib_remote_signon(cib_t *cib, const char *name, enum cib_conn_type type)
 {
     int rc = pcmk_ok;
     cib_remote_opaque_t *private = cib->variant_opaque;
-    xmlNode *hello = NULL;
 
     if (name == NULL) {
         name = pcmk__s(crm_system_name, "client");
@@ -497,19 +497,6 @@ cib_remote_signon(cib_t *cib, const char *name, enum cib_conn_type type)
     }
 
     rc = cib_tls_signon(cib, &(private->callback), TRUE);
-    if (rc != pcmk_ok) {
-        goto done;
-    }
-
-    rc = cib__create_op(cib, CRM_OP_REGISTER, NULL, NULL, NULL, cib_none, NULL,
-                        name, &hello);
-    if (rc != pcmk_ok) {
-        goto done;
-    }
-
-    rc = pcmk__remote_send_xml(&private->command, hello);
-    rc = pcmk_rc2legacy(rc);
-    pcmk__xml_free(hello);
 
 done:
     if (rc == pcmk_ok) {
