@@ -1,6 +1,6 @@
 /*
  * Original copyright 2004 International Business Machines
- * Later changes copyright 2008-2024 the Pacemaker project contributors
+ * Later changes copyright 2008-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,7 +22,6 @@
 #include <crm/common/cib_internal.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
-#include <crm/pengine/rules.h>
 
 gboolean
 cib_version_details(xmlNode * cib, int *admin_epoch, int *epoch, int *updates)
@@ -234,6 +233,7 @@ cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
     xmlNode *local_diff = NULL;
 
     const char *user = crm_element_value(req, PCMK__XA_CIB_USER);
+    const bool enable_acl = cib_acl_enabled(*current_cib, user);
     bool with_digest = false;
 
     crm_trace("Begin %s%s%s op",
@@ -260,7 +260,7 @@ cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
         xmlNode *cib_ro = *current_cib;
         xmlNode *cib_filtered = NULL;
 
-        if (cib_acl_enabled(cib_ro, user)
+        if (enable_acl
             && xml_acl_filtered_copy(user, *current_cib, *current_cib,
                                      &cib_filtered)) {
 
@@ -308,11 +308,19 @@ cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
         pcmk__xe_copy_attrs(top, scratch, pcmk__xaf_none);
         patchset_cib = top;
 
-        xml_track_changes(scratch, user, NULL, cib_acl_enabled(scratch, user));
+        pcmk__xml_commit_changes(scratch->doc);
+        pcmk__xml_doc_set_flags(scratch->doc, pcmk__xf_tracking);
+        if (enable_acl) {
+            pcmk__enable_acl(*current_cib, scratch, user);
+        }
+
         rc = (*fn) (op, call_options, section, req, input, scratch, &scratch, output);
 
         /* If scratch points to a new object now (for example, after an erase
          * operation), then *current_cib should point to the same object.
+         *
+         * @TODO Enable tracking and ACLs and calculate changes? Change tracking
+         * and unpacked ACLs didn't carry over to new object.
          */
         *current_cib = scratch;
 
@@ -320,15 +328,25 @@ cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
         scratch = pcmk__xml_copy(NULL, *current_cib);
         patchset_cib = *current_cib;
 
-        xml_track_changes(scratch, user, NULL, cib_acl_enabled(scratch, user));
+        pcmk__xml_doc_set_flags(scratch->doc, pcmk__xf_tracking);
+        if (enable_acl) {
+            pcmk__enable_acl(*current_cib, scratch, user);
+        }
+
         rc = (*fn) (op, call_options, section, req, input, *current_cib,
                     &scratch, output);
 
-        if ((scratch != NULL) && !xml_tracking_changes(scratch)) {
+        /* @TODO This appears to be a hack to determine whether scratch points
+         * to a new object now, without saving the old pointer (which may be
+         * invalid now) for comparison. Confirm this, and check more clearly.
+         */
+        if (!pcmk__xml_doc_all_flags_set(scratch->doc, pcmk__xf_tracking)) {
             crm_trace("Inferring changes after %s op", op);
-            xml_track_changes(scratch, user, *current_cib,
-                              cib_acl_enabled(*current_cib, user));
-            xml_calculate_changes(*current_cib, scratch);
+            pcmk__xml_commit_changes(scratch->doc);
+            if (enable_acl) {
+                pcmk__enable_acl(*current_cib, scratch, user);
+            }
+            pcmk__xml_mark_changes(*current_cib, scratch);
         }
         CRM_CHECK(*current_cib != scratch, return -EINVAL);
     }
@@ -408,7 +426,7 @@ cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
                                      config_changed, manage_counters);
 
     pcmk__log_xml_changes(LOG_TRACE, scratch);
-    xml_accept_changes(scratch);
+    pcmk__xml_commit_changes(scratch->doc);
 
     if(local_diff) {
         patchset_process_digest(local_diff, patchset_cib, scratch, with_digest);
@@ -739,9 +757,13 @@ cib_read_config(GHashTable * options, xmlNode * current_cib)
 
     config = pcmk_find_cib_element(current_cib, PCMK_XE_CRM_CONFIG);
     if (config) {
-        pe_unpack_nvpairs(NULL, config, PCMK_XE_CLUSTER_PROPERTY_SET, NULL,
-                          options, PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, FALSE, now,
-                          NULL);
+        pcmk_rule_input_t rule_input = {
+            .now = now,
+        };
+
+        pcmk_unpack_nvpair_blocks(config, PCMK_XE_CLUSTER_PROPERTY_SET,
+                                  PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, &rule_input,
+                                  options, NULL);
     }
 
     pcmk__validate_cluster_options(options);
