@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -64,11 +60,11 @@ pcmk__pid_active(pid_t pid, const char *daemon)
             if (last_asked_pid != pid) {
                 if (rc == EACCES) {
                     crm_info("Could not get executable for PID %lld: %s "
-                             CRM_XS " rc=%d",
+                             QB_XS " rc=%d",
                              (long long) pid, pcmk_rc_str(rc), rc);
                 } else {
                     crm_err("Could not get executable for PID %lld: %s "
-                            CRM_XS " rc=%d",
+                            QB_XS " rc=%d",
                             (long long) pid, pcmk_rc_str(rc), rc);
                 }
                 last_asked_pid = pid;
