@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2024 the Pacemaker project contributors
+ * Copyright 2008-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -137,29 +137,26 @@ send_tls(gnutls_session_t session, struct iovec *iov)
         return EINVAL;
     }
 
-    crm_trace("Sending TLS message of %llu bytes",
-              (unsigned long long) unsent_len);
+    crm_trace("Sending TLS message of %zu bytes", unsent_len);
+
     while (true) {
         gnutls_rc = gnutls_record_send(session, unsent, unsent_len);
 
         if (gnutls_rc == GNUTLS_E_INTERRUPTED || gnutls_rc == GNUTLS_E_AGAIN) {
-            crm_trace("Retrying to send %llu bytes remaining",
-                      (unsigned long long) unsent_len);
+            crm_trace("Retrying to send %zu bytes remaining", unsent_len);
 
         } else if (gnutls_rc < 0) {
             // Caller can log as error if necessary
-            crm_info("TLS connection terminated: %s " QB_XS " rc=%lld",
-                     gnutls_strerror((int) gnutls_rc),
-                     (long long) gnutls_rc);
+            crm_info("TLS connection terminated: %s " QB_XS " rc=%zd",
+                     gnutls_strerror((int) gnutls_rc), gnutls_rc);
             return ECONNABORTED;
 
         } else if (gnutls_rc < unsent_len) {
-            crm_trace("Sent %lld of %llu bytes remaining",
-                      (long long) gnutls_rc, (unsigned long long) unsent_len);
+            crm_trace("Sent %zd of %zu bytes remaining", gnutls_rc, unsent_len);
             unsent_len -= gnutls_rc;
             unsent += gnutls_rc;
         } else {
-            crm_trace("Sent all %lld bytes remaining", (long long) gnutls_rc);
+            crm_trace("Sent all %zd bytes remaining", gnutls_rc);
             break;
         }
     }
@@ -172,22 +169,22 @@ send_plaintext(int sock, struct iovec *iov)
 {
     const char *unsent = iov->iov_base;
     size_t unsent_len = iov->iov_len;
-    ssize_t write_rc;
 
     if (unsent == NULL) {
         return EINVAL;
     }
 
-    crm_debug("Sending plaintext message of %llu bytes to socket %d",
-              (unsigned long long) unsent_len, sock);
+    crm_debug("Sending plaintext message of %zu bytes to socket %d",
+              unsent_len, sock);
     while (true) {
-        write_rc = write(sock, unsent, unsent_len);
+        ssize_t write_rc = write(sock, unsent, unsent_len);
+
         if (write_rc < 0) {
             int rc = errno;
 
-            if ((errno == EINTR) || (errno == EAGAIN)) {
-                crm_trace("Retrying to send %llu bytes remaining to socket %d",
-                          (unsigned long long) unsent_len, sock);
+            if ((rc == EINTR) || (rc == EAGAIN) || (rc == EWOULDBLOCK)) {
+                crm_trace("Retrying to send %zu bytes remaining to socket %d",
+                          unsent_len, sock);
                 continue;
             }
 
@@ -197,19 +194,16 @@ send_plaintext(int sock, struct iovec *iov)
             return rc;
 
         } else if (write_rc < unsent_len) {
-            crm_trace("Sent %lld of %llu bytes remaining",
-                      (long long) write_rc, (unsigned long long) unsent_len);
+            crm_trace("Sent %zd of %zu bytes remaining", write_rc, unsent_len);
             unsent += write_rc;
             unsent_len -= write_rc;
-            continue;
 
         } else {
-            crm_trace("Sent all %lld bytes remaining: %.100s",
-                      (long long) write_rc, (char *) (iov->iov_base));
-            break;
+            crm_trace("Sent all %zd bytes remaining: %.100s",
+                      write_rc, (char *) (iov->iov_base));
+            return pcmk_rc_ok;
         }
     }
-    return pcmk_rc_ok;
 }
 
 // \return Standard Pacemaker return code
@@ -223,7 +217,7 @@ remote_send_iovs(pcmk__remote_t *remote, struct iovec *iov, int iovs)
             rc = send_tls(remote->tls_session, &(iov[lpc]));
             continue;
         }
-        if (remote->tcp_socket) {
+        if (remote->tcp_socket >= 0) {
             rc = send_plaintext(remote->tcp_socket, &(iov[lpc]));
         } else {
             rc = ESOCKTNOSUPPORT;
@@ -363,16 +357,12 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
 static int
 get_remote_socket(const pcmk__remote_t *remote)
 {
-    if (remote->tls_session) {
-        void *sock_ptr = gnutls_transport_get_ptr(remote->tls_session);
-
-        return GPOINTER_TO_INT(sock_ptr);
+    if (remote->tls_session != NULL) {
+        return pcmk__tls_get_client_sock(remote);
     }
-
-    if (remote->tcp_socket) {
+    if (remote->tcp_socket >= 0) {
         return remote->tcp_socket;
     }
-
     crm_err("Remote connection type undetermined (bug?)");
     return -1;
 }
@@ -392,13 +382,13 @@ int
 pcmk__remote_ready(const pcmk__remote_t *remote, int timeout_ms)
 {
     struct pollfd fds = { 0, };
-    int sock = 0;
+    int sock = -1;
     int rc = 0;
     time_t start;
     int timeout = timeout_ms;
 
     sock = get_remote_socket(remote);
-    if (sock <= 0) {
+    if (sock < 0) {
         crm_trace("No longer connected");
         return ENOTCONN;
     }
@@ -456,8 +446,7 @@ pcmk__read_available_remote_data(pcmk__remote_t *remote)
     /* automatically grow the buffer when needed */
     if(remote->buffer_size < read_len) {
         remote->buffer_size = 2 * read_len;
-        crm_trace("Expanding buffer to %llu bytes",
-                  (unsigned long long) remote->buffer_size);
+        crm_trace("Expanding buffer to %zu bytes", remote->buffer_size);
         remote->buffer = pcmk__realloc(remote->buffer, remote->buffer_size + 1);
     }
 
@@ -470,11 +459,11 @@ pcmk__read_available_remote_data(pcmk__remote_t *remote)
         } else if (read_rc == GNUTLS_E_AGAIN) {
             rc = EAGAIN;
         } else if (read_rc < 0) {
-            crm_debug("TLS receive failed: %s (%lld)",
-                      gnutls_strerror(read_rc), (long long) read_rc);
+            crm_debug("TLS receive failed: %s (%zd)",
+                      gnutls_strerror((int) read_rc), read_rc);
             rc = EIO;
         }
-    } else if (remote->tcp_socket) {
+    } else if (remote->tcp_socket >= 0) {
         read_rc = read(remote->tcp_socket,
                        remote->buffer + remote->buffer_offset,
                        remote->buffer_size - remote->buffer_offset);
@@ -491,35 +480,32 @@ pcmk__read_available_remote_data(pcmk__remote_t *remote)
         remote->buffer_offset += read_rc;
         /* always null terminate buffer, the +1 to alloc always allows for this. */
         remote->buffer[remote->buffer_offset] = '\0';
-        crm_trace("Received %lld more bytes (%llu total)",
-                  (long long) read_rc,
-                  (unsigned long long) remote->buffer_offset);
+        crm_trace("Received %zd more bytes (%zu total)",
+                  read_rc, remote->buffer_offset);
 
-    } else if ((rc == EINTR) || (rc == EAGAIN)) {
+    } else if (read_rc == 0) {
+        crm_debug("End of remote data encountered after %zu bytes",
+                  remote->buffer_offset);
+        return ENOTCONN;
+
+    } else if ((rc == EINTR) || (rc == EAGAIN) || (rc == EWOULDBLOCK)) {
         crm_trace("No data available for non-blocking remote read: %s (%d)",
                   pcmk_rc_str(rc), rc);
 
-    } else if (read_rc == 0) {
-        crm_debug("End of remote data encountered after %llu bytes",
-                  (unsigned long long) remote->buffer_offset);
-        return ENOTCONN;
-
     } else {
-        crm_debug("Error receiving remote data after %llu bytes: %s (%d)",
-                  (unsigned long long) remote->buffer_offset,
-                  pcmk_rc_str(rc), rc);
+        crm_debug("Error receiving remote data after %zu bytes: %s (%d)",
+                  remote->buffer_offset, pcmk_rc_str(rc), rc);
         return ENOTCONN;
     }
 
     header = localized_remote_header(remote);
     if(header) {
         if(remote->buffer_offset < header->size_total) {
-            crm_trace("Read partial remote message (%llu of %u bytes)",
-                      (unsigned long long) remote->buffer_offset,
-                      header->size_total);
+            crm_trace("Read partial remote message (%zu of %" PRIu32 " bytes)",
+                      remote->buffer_offset, header->size_total);
         } else {
-            crm_trace("Read full remote message of %llu bytes",
-                      (unsigned long long) remote->buffer_offset);
+            crm_trace("Read full remote message of %zu bytes",
+                      remote->buffer_offset);
             return pcmk_rc_ok;
         }
     }
@@ -620,7 +606,7 @@ check_connect_finished(gpointer userdata)
 
     if (rc < 0) { // select() error
         rc = errno;
-        if ((rc == EINPROGRESS) || (rc == EAGAIN)) {
+        if ((rc == EINTR) || (rc == EAGAIN)) {
             if ((time(NULL) - cb_data->start) < pcmk__timeout_ms2s(cb_data->timeout_ms)) {
                 return TRUE; // There is time left, so reschedule timer
             } else {
@@ -716,11 +702,19 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
     }
 
     rc = connect(sock, addr, addrlen);
-    if (rc < 0 && (errno != EINPROGRESS) && (errno != EAGAIN)) {
+    if (rc < 0) {
         rc = errno;
-        crm_warn("Could not connect socket: %s " QB_XS " rc=%d",
-                 pcmk_rc_str(rc), rc);
-        return rc;
+        switch (rc) {
+            case EINTR:
+            case EINPROGRESS:
+            case EAGAIN:
+                break;
+
+            default:
+                crm_warn("Could not connect socket: %s " QB_XS " rc=%d",
+                         pcmk_rc_str(rc), rc);
+                return rc;
+        }
     }
 
     cb_data = pcmk__assert_alloc(1, sizeof(struct tcp_async_cb_data));
@@ -756,8 +750,6 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
         *timer_id = timer;
     }
 
-    // timer callback should be taking care of cb_data
-    // cppcheck-suppress memleak
     return pcmk_rc_ok;
 }
 
