@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -426,6 +426,14 @@ static const struct pcmk__rc_info {
       "Compression/decompression error",
       -pcmk_err_generic,
     },
+    { "pcmk_rc_no_dc",
+      "DC is not yet elected",
+      -pcmk_err_generic,
+    },
+    { "pcmk_rc_ipc_more",
+      "More IPC message fragments to send",
+      -pcmk_err_generic,
+    },
 };
 
 /*!
@@ -734,6 +742,7 @@ crm_exit_name(crm_exit_t exit_code)
         case CRM_EX_NOT_YET_IN_EFFECT: return "CRM_EX_NOT_YET_IN_EFFECT";
         case CRM_EX_INDETERMINATE: return "CRM_EX_INDETERMINATE";
         case CRM_EX_UNSATISFIED: return "CRM_EX_UNSATISFIED";
+        case CRM_EX_NO_DC: return "CRM_EX_NO_DC";
         case CRM_EX_OLD: return "CRM_EX_OLD";
         case CRM_EX_TIMEOUT: return "CRM_EX_TIMEOUT";
         case CRM_EX_DEGRADED: return "CRM_EX_DEGRADED";
@@ -786,6 +795,7 @@ crm_exit_str(crm_exit_t exit_code)
         case CRM_EX_NOT_YET_IN_EFFECT: return "Requested item is not yet in effect";
         case CRM_EX_INDETERMINATE: return "Could not determine status";
         case CRM_EX_UNSATISFIED: return "Not applicable under current conditions";
+        case CRM_EX_NO_DC: return "DC is not yet elected";
         case CRM_EX_OLD: return "Update was older than existing configuration";
         case CRM_EX_TIMEOUT: return "Timeout occurred";
         case CRM_EX_DEGRADED: return "Service is active but might fail soon";
@@ -820,6 +830,7 @@ pcmk_rc2exitc(int rc)
         case pcmk_rc_old_data:
             return CRM_EX_OLD;
 
+        case pcmk_rc_cib_corrupt:
         case pcmk_rc_schema_validation:
         case pcmk_rc_transform_failed:
         case pcmk_rc_unpack_error:
@@ -922,6 +933,9 @@ pcmk_rc2exitc(int rc)
         case pcmk_rc_bad_xml_patch:
             return CRM_EX_DATAERR;
 
+        case pcmk_rc_no_dc:
+            return CRM_EX_NO_DC;
+
         default:
             return CRM_EX_ERROR;
     }
