@@ -56,6 +56,6 @@ is_xml_acl_denied_with_node(void **state)
     assert_true(xml_acl_denied(test_xml));
 }
 
-PCMK__UNIT_TEST(pcmk__xml_test_setup_group, NULL,
+PCMK__UNIT_TEST(pcmk__xml_test_setup_group, pcmk__xml_test_teardown_group,
                 cmocka_unit_test(is_xml_acl_denied_without_node),
                 cmocka_unit_test(is_xml_acl_denied_with_node))
