@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 the Pacemaker project contributors
+ * Copyright 2024-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,14 +18,14 @@ static int
 setup(void **state)
 {
     setenv("PCMK_schema_directory", PCMK__TEST_SCHEMA_DIR, 1);
-    pcmk__schema_init();
+    pcmk__xml_test_setup_group(state);
     return 0;
 }
 
 static int
 teardown(void **state)
 {
-    pcmk__schema_cleanup();
+    pcmk__xml_test_teardown_group(state);
     unsetenv("PCMK_schema_directory");
     return 0;
 }
