@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 the Pacemaker project contributors
+ * Copyright 2024-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <libxml/parser.h>                  // xmlCleanupParser()
+
 // LCOV_EXCL_START
 
 void
@@ -84,8 +86,7 @@ pcmk__assert_validates(xmlNode *xml)
 int
 pcmk__xml_test_setup_group(void **state)
 {
-    // Load schemas and set libxml2 buffer allocation scheme
-    pcmk__xml_init();
+    pcmk__schema_init();
     return 0;
 }
 
@@ -103,8 +104,8 @@ pcmk__xml_test_setup_group(void **state)
 int
 pcmk__xml_test_teardown_group(void **state)
 {
-    // Clean up schemas and libxml2 global memory
-    pcmk__xml_cleanup();
+    pcmk__schema_cleanup();
+    xmlCleanupParser();
     return 0;
 }
 
