@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,9 +18,8 @@
 #include <sys/types.h>
 
 #include <glib.h>                       // gboolean, GString
-#include <libxml/parser.h>              // xmlCleanupParser()
 #include <libxml/tree.h>                // xmlNode, etc.
-#include <libxml/xmlstring.h>           // xmlGetUTF8Char()
+#include <libxml/xmlstring.h>           // xmlChar, xmlGetUTF8Char()
 
 #include <crm/crm.h>
 #include <crm/common/xml.h>
@@ -28,7 +27,48 @@
 #include "crmcommon_private.h"
 
 //! libxml2 supports only XML version 1.0, at least as of libxml2-2.12.5
-#define XML_VERSION ((pcmkXmlStr) "1.0")
+#define XML_VERSION ((const xmlChar *) "1.0")
+
+/*!
+ * \internal
+ * \brief Get a string representation of an XML element type for logging
+ *
+ * \param[in] type  XML element type
+ *
+ * \return String representation of \p type
+ */
+const char *
+pcmk__xml_element_type_text(xmlElementType type)
+{
+    static const char *const element_type_names[] = {
+        [XML_ELEMENT_NODE]       = "element",
+        [XML_ATTRIBUTE_NODE]     = "attribute",
+        [XML_TEXT_NODE]          = "text",
+        [XML_CDATA_SECTION_NODE] = "CDATA section",
+        [XML_ENTITY_REF_NODE]    = "entity reference",
+        [XML_ENTITY_NODE]        = "entity",
+        [XML_PI_NODE]            = "PI",
+        [XML_COMMENT_NODE]       = "comment",
+        [XML_DOCUMENT_NODE]      = "document",
+        [XML_DOCUMENT_TYPE_NODE] = "document type",
+        [XML_DOCUMENT_FRAG_NODE] = "document fragment",
+        [XML_NOTATION_NODE]      = "notation",
+        [XML_HTML_DOCUMENT_NODE] = "HTML document",
+        [XML_DTD_NODE]           = "DTD",
+        [XML_ELEMENT_DECL]       = "element declaration",
+        [XML_ATTRIBUTE_DECL]     = "attribute declaration",
+        [XML_ENTITY_DECL]        = "entity declaration",
+        [XML_NAMESPACE_DECL]     = "namespace declaration",
+        [XML_XINCLUDE_START]     = "XInclude start",
+        [XML_XINCLUDE_END]       = "XInclude end",
+    };
+
+    // Assumes the numeric values of the indices are in ascending order
+    if ((type < XML_ELEMENT_NODE) || (type > XML_XINCLUDE_END)) {
+        return "unrecognized type";
+    }
+    return element_type_names[type];
+}
 
 /*!
  * \internal
@@ -65,21 +105,6 @@ pcmk__xml_tree_foreach(xmlNode *xml, bool (*fn)(xmlNode *, void *),
     return true;
 }
 
-bool
-pcmk__tracking_xml_changes(xmlNode *xml, bool lazy)
-{
-    if(xml == NULL || xml->doc == NULL || xml->doc->_private == NULL) {
-        return FALSE;
-    } else if (!pcmk_is_set(((xml_doc_private_t *)xml->doc->_private)->flags,
-                            pcmk__xf_tracking)) {
-        return FALSE;
-    } else if (lazy && !pcmk_is_set(((xml_doc_private_t *)xml->doc->_private)->flags,
-                                    pcmk__xf_lazy)) {
-        return FALSE;
-    }
-    return TRUE;
-}
-
 void
 pcmk__xml_set_parent_flags(xmlNode *xml, uint64_t flags)
 {
@@ -92,21 +117,51 @@ pcmk__xml_set_parent_flags(xmlNode *xml, uint64_t flags)
     }
 }
 
+/*!
+ * \internal
+ * \brief Set flags for an XML document
+ *
+ * \param[in,out] doc    XML document
+ * \param[in]     flags  Group of <tt>enum pcmk__xml_flags</tt>
+ */
 void
-pcmk__set_xml_doc_flag(xmlNode *xml, enum xml_private_flags flag)
+pcmk__xml_doc_set_flags(xmlDoc *doc, uint32_t flags)
 {
-    if (xml != NULL) {
-        xml_doc_private_t *docpriv = xml->doc->_private;
+    if (doc != NULL) {
+        xml_doc_private_t *docpriv = doc->_private;
 
-        pcmk__set_xml_flags(docpriv, flag);
+        pcmk__set_xml_flags(docpriv, flags);
     }
 }
 
+/*!
+ * \internal
+ * \brief Check whether the given flags are set for an XML document
+ *
+ * \param[in] doc    XML document to check
+ * \param[in] flags  Group of <tt>enum pcmk__xml_flags</tt>
+ *
+ * \return \c true if all of \p flags are set for \p doc, or \c false otherwise
+ */
+bool
+pcmk__xml_doc_all_flags_set(const xmlDoc *doc, uint32_t flags)
+{
+    if (doc != NULL) {
+        xml_doc_private_t *docpriv = doc->_private;
+
+        return (docpriv != NULL) && pcmk_all_flags_set(docpriv->flags, flags);
+    }
+    return false;
+}
+
 // Mark document, element, and all element's parents as changed
 void
 pcmk__mark_xml_node_dirty(xmlNode *xml)
 {
-    pcmk__set_xml_doc_flag(xml, pcmk__xf_dirty);
+    if (xml == NULL) {
+        return;
+    }
+    pcmk__xml_doc_set_flags(xml->doc, pcmk__xf_dirty);
     pcmk__xml_set_parent_flags(xml, pcmk__xf_dirty);
 }
 
@@ -167,7 +222,7 @@ mark_xml_tree_dirty_created(xmlNode *xml)
 {
     pcmk__assert(xml != NULL);
 
-    if (!pcmk__tracking_xml_changes(xml, false)) {
+    if (!pcmk__xml_doc_all_flags_set(xml->doc, pcmk__xf_tracking)) {
         // Tracking is disabled for entire document
         return;
     }
@@ -197,8 +252,7 @@ reset_xml_private_data(xml_doc_private_t *docpriv)
     if (docpriv != NULL) {
         pcmk__assert(docpriv->check == PCMK__XML_DOC_PRIVATE_MAGIC);
 
-        free(docpriv->user);
-        docpriv->user = NULL;
+        pcmk__str_update(&(docpriv->acl_user), NULL);
 
         if (docpriv->acls != NULL) {
             pcmk__free_acls(docpriv->acls);
@@ -226,12 +280,16 @@ reset_xml_private_data(xml_doc_private_t *docpriv)
 static bool
 new_private_data(xmlNode *node, void *user_data)
 {
+    bool tracking = false;
+
     CRM_CHECK(node != NULL, return true);
 
     if (node->_private != NULL) {
         return true;
     }
 
+    tracking = pcmk__xml_doc_all_flags_set(node->doc, pcmk__xf_tracking);
+
     switch (node->type) {
         case XML_DOCUMENT_NODE:
             {
@@ -240,7 +298,6 @@ new_private_data(xmlNode *node, void *user_data)
 
                 docpriv->check = PCMK__XML_DOC_PRIVATE_MAGIC;
                 node->_private = docpriv;
-                pcmk__set_xml_flags(docpriv, pcmk__xf_dirty|pcmk__xf_created);
             }
             break;
 
@@ -253,7 +310,9 @@ new_private_data(xmlNode *node, void *user_data)
 
                 nodepriv->check = PCMK__XML_NODE_PRIVATE_MAGIC;
                 node->_private = nodepriv;
-                pcmk__set_xml_flags(nodepriv, pcmk__xf_dirty|pcmk__xf_created);
+                if (tracking) {
+                    pcmk__set_xml_flags(nodepriv, pcmk__xf_dirty|pcmk__xf_created);
+                }
 
                 for (xmlAttr *iter = pcmk__xe_first_attr(node); iter != NULL;
                      iter = iter->next) {
@@ -273,7 +332,7 @@ new_private_data(xmlNode *node, void *user_data)
             return true;
     }
 
-    if (pcmk__tracking_xml_changes(node, false)) {
+    if (tracking) {
         pcmk__mark_xml_node_dirty(node);
     }
     return true;
@@ -342,36 +401,6 @@ pcmk__xml_free_private_data(xmlNode *xml)
     pcmk__xml_tree_foreach(xml, free_private_data, NULL);
 }
 
-void
-xml_track_changes(xmlNode * xml, const char *user, xmlNode *acl_source, bool enforce_acls) 
-{
-    xml_accept_changes(xml);
-    crm_trace("Tracking changes%s to %p", enforce_acls?" with ACLs":"", xml);
-    pcmk__set_xml_doc_flag(xml, pcmk__xf_tracking);
-    if(enforce_acls) {
-        if(acl_source == NULL) {
-            acl_source = xml;
-        }
-        pcmk__set_xml_doc_flag(xml, pcmk__xf_acl_enabled);
-        pcmk__unpack_acl(acl_source, xml, user);
-        pcmk__apply_acl(xml);
-    }
-}
-
-bool xml_tracking_changes(xmlNode * xml)
-{
-    return (xml != NULL) && (xml->doc != NULL) && (xml->doc->_private != NULL)
-           && pcmk_is_set(((xml_doc_private_t *)(xml->doc->_private))->flags,
-                          pcmk__xf_tracking);
-}
-
-bool xml_document_dirty(xmlNode *xml) 
-{
-    return (xml != NULL) && (xml->doc != NULL) && (xml->doc->_private != NULL)
-           && pcmk_is_set(((xml_doc_private_t *)(xml->doc->_private))->flags,
-                          pcmk__xf_dirty);
-}
-
 /*!
  * \internal
  * \brief Return ordinal position of an XML node among its siblings
@@ -382,7 +411,7 @@ bool xml_document_dirty(xmlNode *xml)
  * \return Ordinal position of \p xml (starting with 0)
  */
 int
-pcmk__xml_position(const xmlNode *xml, enum xml_private_flags ignore_if_set)
+pcmk__xml_position(const xmlNode *xml, enum pcmk__xml_flags ignore_if_set)
 {
     int position = 0;
 
@@ -409,61 +438,52 @@ pcmk__xml_position(const xmlNode *xml, enum xml_private_flags ignore_if_set)
  * \note This is compatible with \c pcmk__xml_tree_foreach().
  */
 static bool
-accept_attr_deletions(xmlNode *xml, void *user_data)
+commit_attr_deletions(xmlNode *xml, void *user_data)
 {
     pcmk__xml_reset_node_flags(xml, NULL);
-    pcmk__xe_remove_matching_attrs(xml, pcmk__marked_as_deleted, NULL);
+    pcmk__xe_remove_matching_attrs(xml, true, pcmk__marked_as_deleted, NULL);
     return true;
 }
 
 /*!
  * \internal
- * \brief Find first child XML node matching another given XML node
+ * \brief Finalize all pending changes to an XML document and reset private data
  *
- * \param[in] haystack  XML whose children should be checked
- * \param[in] needle    XML to match (comment content or element name and ID)
- * \param[in] exact     If true and needle is a comment, position must match
+ * Clear the ACL user and all flags, unpacked ACLs, and deleted node records for
+ * the document; clear all flags on each node in the tree; and delete any
+ * attributes that are marked for deletion.
+ *
+ * \param[in,out] doc  XML document
+ *
+ * \note When change tracking is enabled, "deleting" an attribute simply marks
+ *       it for deletion (using \c pcmk__xf_deleted) until changes are
+ *       committed. Freeing a node (using \c pcmk__xml_free()) adds a deleted
+ *       node record (\c pcmk__deleted_xml_t) to the node's document before
+ *       freeing it.
+ * \note This function clears all flags, not just flags that indicate changes.
+ *       In particular, note that it clears the \c pcmk__xf_tracking flag, thus
+ *       disabling tracking.
  */
-xmlNode *
-pcmk__xml_match(const xmlNode *haystack, const xmlNode *needle, bool exact)
-{
-    CRM_CHECK(needle != NULL, return NULL);
-
-    if (needle->type == XML_COMMENT_NODE) {
-        return pcmk__xc_match(haystack, needle, exact);
-
-    } else {
-        const char *id = pcmk__xe_id(needle);
-        const char *attr = (id == NULL)? NULL : PCMK_XA_ID;
-
-        return pcmk__xe_first_child(haystack, (const char *) needle->name, attr,
-                                    id);
-    }
-}
-
 void
-xml_accept_changes(xmlNode * xml)
+pcmk__xml_commit_changes(xmlDoc *doc)
 {
-    xmlNode *top = NULL;
     xml_doc_private_t *docpriv = NULL;
 
-    if(xml == NULL) {
+    if (doc == NULL) {
         return;
     }
 
-    crm_trace("Accepting changes to %p", xml);
-    docpriv = xml->doc->_private;
-    top = xmlDocGetRootElement(xml->doc);
-
-    reset_xml_private_data(xml->doc->_private);
-
-    if (!pcmk_is_set(docpriv->flags, pcmk__xf_dirty)) {
-        docpriv->flags = pcmk__xf_none;
+    docpriv = doc->_private;
+    if (docpriv == NULL) {
         return;
     }
 
+    if (pcmk_is_set(docpriv->flags, pcmk__xf_dirty)) {
+        pcmk__xml_tree_foreach(xmlDocGetRootElement(doc), commit_attr_deletions,
+                               NULL);
+    }
+    reset_xml_private_data(docpriv);
     docpriv->flags = pcmk__xf_none;
-    pcmk__xml_tree_foreach(top, accept_attr_deletions, NULL);
 }
 
 /*!
@@ -540,7 +560,7 @@ pcmk__xml_is_name_start_char(const char *utf8, int *len)
     *len = 4;
 
     // Note: xmlGetUTF8Char() assumes a 32-bit int
-    c = xmlGetUTF8Char((pcmkXmlStr) utf8, len);
+    c = xmlGetUTF8Char((const xmlChar *) utf8, len);
     if (c < 0) {
         GString *buf = g_string_sized_new(32);
 
@@ -601,7 +621,7 @@ pcmk__xml_is_name_char(const char *utf8, int *len)
     *len = 4;
 
     // Note: xmlGetUTF8Char() assumes a 32-bit int
-    c = xmlGetUTF8Char((pcmkXmlStr) utf8, len);
+    c = xmlGetUTF8Char((const xmlChar *) utf8, len);
     if (c < 0) {
         GString *buf = g_string_sized_new(32);
 
@@ -710,15 +730,17 @@ pcmk__xml_free_node(xmlNode *xml)
  * \param[in]     position  Position of \p node among its siblings for change
  *                          tracking (negative to calculate automatically if
  *                          needed)
+ *
+ * \return Standard Pacemaker return code
  */
-static void
+static int
 free_xml_with_position(xmlNode *node, int position)
 {
     xmlDoc *doc = NULL;
     xml_node_private_t *nodepriv = NULL;
 
     if (node == NULL) {
-        return;
+        return pcmk_rc_ok;
     }
     doc = node->doc;
     nodepriv = node->_private;
@@ -728,22 +750,26 @@ free_xml_with_position(xmlNode *node, int position)
          * free the root element without write permission.
          */
         pcmk__xml_free_doc(doc);
-        return;
+        return pcmk_rc_ok;
     }
 
     if (!pcmk__check_acl(node, NULL, pcmk__xf_acl_write)) {
-        GString *xpath = NULL;
+        pcmk__if_tracing(
+            {
+                GString *xpath = pcmk__element_xpath(node);
 
-        pcmk__if_tracing({}, return);
-        xpath = pcmk__element_xpath(node);
-        qb_log_from_external_source(__func__, __FILE__,
-                                    "Cannot remove %s %x", LOG_TRACE,
-                                    __LINE__, 0, xpath->str, nodepriv->flags);
-        g_string_free(xpath, TRUE);
-        return;
+                qb_log_from_external_source(__func__, __FILE__,
+                                            "Cannot remove %s %x", LOG_TRACE,
+                                            __LINE__, 0, xpath->str,
+                                            nodepriv->flags);
+                g_string_free(xpath, TRUE);
+            },
+            {}
+        );
+        return EACCES;
     }
 
-    if ((doc != NULL) && pcmk__tracking_xml_changes(node, false)
+    if (pcmk__xml_doc_all_flags_set(node->doc, pcmk__xf_tracking)
         && !pcmk_is_set(nodepriv->flags, pcmk__xf_created)) {
 
         xml_doc_private_t *docpriv = doc->_private;
@@ -771,10 +797,11 @@ free_xml_with_position(xmlNode *node, int position)
 
             docpriv->deleted_objs = g_list_append(docpriv->deleted_objs,
                                                   deleted_obj);
-            pcmk__set_xml_doc_flag(node, pcmk__xf_dirty);
+            pcmk__xml_doc_set_flags(node->doc, pcmk__xf_dirty);
         }
     }
     pcmk__xml_free_node(node);
+    return pcmk_rc_ok;
 }
 
 /*!
@@ -1060,21 +1087,6 @@ pcmk__xml_escape(const char *text, enum pcmk__xml_escape_type type)
     return g_string_free(copy, FALSE);
 }
 
-/*!
- * \internal
- * \brief Set a flag on all attributes of an XML element
- *
- * \param[in,out] xml   XML node to set flags on
- * \param[in]     flag  XML private flag to set
- */
-static void
-set_attrs_flag(xmlNode *xml, enum xml_private_flags flag)
-{
-    for (xmlAttr *attr = pcmk__xe_first_attr(xml); attr; attr = attr->next) {
-        pcmk__set_xml_flags((xml_node_private_t *) (attr->_private), flag);
-    }
-}
-
 /*!
  * \internal
  * \brief Add an XML attribute to a node, marked as deleted
@@ -1105,7 +1117,7 @@ mark_attr_deleted(xmlNode *new_xml, const char *element, const char *attr_name,
     pcmk__set_xml_flags(docpriv, pcmk__xf_tracking);
 
     // Reset flags (so the attribute doesn't appear as newly created)
-    attr = xmlHasProp(new_xml, (pcmkXmlStr) attr_name);
+    attr = xmlHasProp(new_xml, (const xmlChar *) attr_name);
     nodepriv = attr->_private;
     nodepriv->flags = 0;
 
@@ -1209,7 +1221,13 @@ xml_diff_old_attrs(xmlNode *old_xml, xmlNode *new_xml)
                                   old_value);
 
             } else if ((old_pos != new_pos)
-                       && !pcmk__tracking_xml_changes(new_xml, TRUE)) {
+                       && !pcmk__xml_doc_all_flags_set(new_xml->doc,
+                                                       pcmk__xf_ignore_attr_pos
+                                                       |pcmk__xf_tracking)) {
+                /* pcmk__xf_tracking is always set by pcmk__xml_mark_changes()
+                 * before this function is called, so only the
+                 * pcmk__xf_ignore_attr_pos check is truly relevant.
+                 */
                 mark_attr_moved(new_xml, (const char *) old_xml->name,
                                 old_attr, new_attr, old_pos, new_pos);
             }
@@ -1266,197 +1284,351 @@ mark_created_attrs(xmlNode *new_xml)
 static void
 xml_diff_attrs(xmlNode *old_xml, xmlNode *new_xml)
 {
-    set_attrs_flag(new_xml, pcmk__xf_created); // cleared later if not really new
+    // Cleared later if attributes are not really new
+    for (xmlAttr *attr = pcmk__xe_first_attr(new_xml); attr != NULL;
+         attr = attr->next) {
+        xml_node_private_t *nodepriv = attr->_private;
+
+        pcmk__set_xml_flags(nodepriv, pcmk__xf_created);
+    }
+
     xml_diff_old_attrs(old_xml, new_xml);
     mark_created_attrs(new_xml);
 }
 
 /*!
  * \internal
- * \brief Add an XML child element to a node, marked as deleted
+ * \brief Add a deleted object record for an old XML child if ACLs allow
  *
- * When calculating XML changes, we need to know when a child element has been
- * deleted. Add the child back to the new XML, so that we can check the removal
- * against ACLs, and mark it as deleted for later removal after differences have
- * been calculated.
+ * This is intended to be called for a child of an old XML element that is not
+ * present as a child of a new XML element.
  *
- * \param[in,out] old_child    Child element from original XML
- * \param[in,out] new_parent   New XML to add marked copy to
+ * Add a temporary copy of the old child to the new XML. Then check whether ACLs
+ * would have allowed the deletion of that element. If so, add a deleted object
+ * record for it to the new XML's document, and set the \c pcmk__xf_skip flag on
+ * the old child.
+ *
+ * The temporary copy is removed before returning. The new XML and all of its
+ * ancestors will have the \c pcmk__xf_dirty flag set because of the creation,
+ * however.
+ *
+ * \param[in,out] old_child   Child of old XML
+ * \param[in,out] new_parent  New XML that does not contain \p old_child
  */
 static void
 mark_child_deleted(xmlNode *old_child, xmlNode *new_parent)
 {
+    int pos = pcmk__xml_position(old_child, pcmk__xf_skip);
+
     // Re-create the child element so we can check ACLs
     xmlNode *candidate = pcmk__xml_copy(new_parent, old_child);
 
     // Clear flags on new child and its children
     pcmk__xml_tree_foreach(candidate, pcmk__xml_reset_node_flags, NULL);
 
-    // Check whether ACLs allow the deletion
+    // free_xml_with_position() will check whether ACLs allow the deletion
     pcmk__apply_acl(xmlDocGetRootElement(candidate->doc));
 
-    // Remove the child again (which will track it in document's deleted_objs)
-    free_xml_with_position(candidate,
-                           pcmk__xml_position(old_child, pcmk__xf_skip));
-
-    if (pcmk__xml_match(new_parent, old_child, true) == NULL) {
-        pcmk__set_xml_flags((xml_node_private_t *) (old_child->_private),
-                            pcmk__xf_skip);
+    /* Try to remove the child again (which will track it in document's
+     * deleted_objs on success)
+     */
+    if (free_xml_with_position(candidate, pos) != pcmk_rc_ok) {
+        // ACLs denied deletion in free_xml_with_position. Free candidate here.
+        pcmk__xml_free_node(candidate);
     }
+
+    pcmk__set_xml_flags((xml_node_private_t *) old_child->_private,
+                        pcmk__xf_skip);
 }
 
+/*!
+ * \internal
+ * \brief Mark a new child as moved and set \c pcmk__xf_skip as appropriate
+ *
+ * \param[in,out] old_child  Child of old XML
+ * \param[in,out] new_child  Child of new XML that matches \p old_child
+ * \param[in]     old_pos    Position of \p old_child among its siblings
+ * \param[in]     new_pos    Position of \p new_child among its siblings
+ */
 static void
-mark_child_moved(xmlNode *old_child, xmlNode *new_parent, xmlNode *new_child,
-                 int p_old, int p_new)
+mark_child_moved(xmlNode *old_child, xmlNode *new_child, int old_pos,
+                 int new_pos)
 {
+    const char *id_s = pcmk__s(pcmk__xe_id(new_child), "<no id>");
+    xmlNode *new_parent = new_child->parent;
     xml_node_private_t *nodepriv = new_child->_private;
 
-    crm_trace("Child element %s with "
-              PCMK_XA_ID "='%s' moved from position %d to %d under %s",
-              new_child->name, pcmk__s(pcmk__xe_id(new_child), "<no id>"),
-              p_old, p_new, new_parent->name);
+    crm_trace("Child element %s with " PCMK_XA_ID "='%s' moved from position "
+              "%d to %d under %s",
+              new_child->name, id_s, old_pos, new_pos, new_parent->name);
     pcmk__mark_xml_node_dirty(new_parent);
     pcmk__set_xml_flags(nodepriv, pcmk__xf_moved);
 
-    if (p_old > p_new) {
+    /* @TODO Figure out and document why we skip the old child in future
+     * position calculations if the old position is higher, and skip the new
+     * child in future position calculations if the new position is higher. This
+     * goes back to d028b52, and there's no explanation in the commit message.
+     */
+    if (old_pos > new_pos) {
         nodepriv = old_child->_private;
-    } else {
-        nodepriv = new_child->_private;
     }
     pcmk__set_xml_flags(nodepriv, pcmk__xf_skip);
 }
 
-// Given original and new XML, mark new XML portions that have changed
-static void
-mark_xml_changes(xmlNode *old_xml, xmlNode *new_xml, bool check_top)
+/*!
+ * \internal
+ * \brief Check whether a new XML child comment matches an old XML child comment
+ *
+ * Two comments match if they have the same position among their siblings and
+ * the same contents.
+ *
+ * If \p new_comment has the \c pcmk__xf_skip flag set, then it is automatically
+ * considered not to match.
+ *
+ * \param[in] old_comment  Old XML child element
+ * \param[in] new_comment  New XML child element
+ *
+ * \retval \c true   if \p new_comment matches \p old_comment
+ * \retval \c false  otherwise
+ */
+static bool
+new_comment_matches(const xmlNode *old_comment, const xmlNode *new_comment)
 {
-    xmlNode *old_child = NULL;
-    xmlNode *new_child = NULL;
-    xml_node_private_t *nodepriv = NULL;
+    xml_node_private_t *nodepriv = new_comment->_private;
 
-    CRM_CHECK(new_xml != NULL, return);
-    if (old_xml == NULL) {
-        mark_xml_tree_dirty_created(new_xml);
-        pcmk__apply_creation_acl(new_xml, check_top);
-        return;
+    if (pcmk_is_set(nodepriv->flags, pcmk__xf_skip)) {
+        /* @TODO Should we also return false if old_comment has pcmk__xf_skip
+         * set? This preserves existing behavior at time of writing.
+         */
+        return false;
+    }
+    if (pcmk__xml_position(old_comment, pcmk__xf_skip)
+        != pcmk__xml_position(new_comment, pcmk__xf_skip)) {
+        return false;
+    }
+    return pcmk__xc_matches(old_comment, new_comment);
+}
+
+/*!
+ * \internal
+ * \brief Check whether a new XML child element matches an old XML child element
+ *
+ * Two elements match if they have the same name and the same ID. (Both IDs can
+ * be \c NULL.)
+ *
+ * For XML attributes other than \c PCMK_XA_ID, we can treat a value change as
+ * an in-place modification. However, when Pacemaker applies a patchset, it uses
+ * the \c PCMK_XA_ID attribute to find the node to update (modify, delete, or
+ * move). If we treat two nodes with different \c PCMK_XA_ID attributes as
+ * matching and then mark that attribute as changed, it can cause this lookup to
+ * fail.
+ *
+ * There's unlikely to ever be much practical reason to treat elements with
+ * different IDs as a change. Unless that changes, we'll treat them as a
+ * mismatch.
+ *
+ * \param[in] old_element  Old XML child element
+ * \param[in] new_element  New XML child element
+ *
+ * \retval \c true   if \p new_element matches \p old_element
+ * \retval \c false  otherwise
+ */
+static bool
+new_element_matches(const xmlNode *old_element, const xmlNode *new_element)
+{
+    return pcmk__xe_is(new_element, (const char *) old_element->name)
+           && pcmk__str_eq(pcmk__xe_id(old_element), pcmk__xe_id(new_element),
+                           pcmk__str_none);
+}
+
+/*!
+ * \internal
+ * \brief Check whether a new XML child node matches an old XML child node
+ *
+ * Node types must be the same in order to match.
+ *
+ * For comments, a match is a comment at the same position with the same
+ * content.
+ *
+ * For elements, a match is an element with the same name and the same ID. (Both
+ * IDs can be \c NULL.)
+ *
+ * For other node types, there is no match.
+ *
+ * \param[in] old_child  Child of old XML
+ * \param[in] new_child  Child of new XML
+ *
+ * \retval \c true   if \p new_child matches \p old_child
+ * \retval \c false  otherwise
+ */
+static bool
+new_child_matches(const xmlNode *old_child, const xmlNode *new_child)
+{
+    if (old_child->type != new_child->type) {
+        return false;
     }
 
-    nodepriv = new_xml->_private;
-    CRM_CHECK(nodepriv != NULL, return);
-
-    if(nodepriv->flags & pcmk__xf_processed) {
-        /* Avoid re-comparing nodes */
-        return;
+    switch (old_child->type) {
+        case XML_COMMENT_NODE:
+            return new_comment_matches(old_child, new_child);
+        case XML_ELEMENT_NODE:
+            return new_element_matches(old_child, new_child);
+        default:
+            return false;
     }
-    pcmk__set_xml_flags(nodepriv, pcmk__xf_processed);
+}
 
-    xml_diff_attrs(old_xml, new_xml);
-
-    // Check for differences in the original children
-    for (old_child = pcmk__xml_first_child(old_xml); old_child != NULL;
+/*!
+ * \internal
+ * \brief Find matching XML node pairs between old and new XML's children
+ *
+ * A node that is part of a matching pair gets its <tt>_private:match</tt>
+ * member set to the matching node.
+ *
+ * \param[in,out] old_xml       Old XML
+ * \param[in,out] new_xml       New XML
+ */
+static void
+find_matching_children(xmlNode *old_xml, xmlNode *new_xml)
+{
+    for (xmlNode *old_child = pcmk__xml_first_child(old_xml); old_child != NULL;
          old_child = pcmk__xml_next(old_child)) {
 
-        new_child = pcmk__xml_match(new_xml, old_child, true);
+        xml_node_private_t *old_nodepriv = old_child->_private;
 
-        if (new_child != NULL) {
-            mark_xml_changes(old_child, new_child, true);
-
-        } else {
-            mark_child_deleted(old_child, new_xml);
+        if ((old_nodepriv == NULL) || (old_nodepriv->match != NULL)) {
+            // Can't process, or we already found a match for this old child
+            continue;
         }
-    }
 
-    // Check for moved or created children
-    new_child = pcmk__xml_first_child(new_xml);
-    while (new_child != NULL) {
-        xmlNode *next = pcmk__xml_next(new_child);
+        for (xmlNode *new_child = pcmk__xml_first_child(new_xml);
+             new_child != NULL; new_child = pcmk__xml_next(new_child)) {
 
-        old_child = pcmk__xml_match(old_xml, new_child, true);
+            xml_node_private_t *new_nodepriv = new_child->_private;
 
-        if (old_child == NULL) {
-            // This is a newly created child
-            nodepriv = new_child->_private;
-            pcmk__set_xml_flags(nodepriv, pcmk__xf_skip);
+            if ((new_nodepriv == NULL) || (new_nodepriv->match != NULL)) {
+                /* Can't process, or this new child already matched some old
+                 * child
+                 */
+                continue;
+            }
 
-            // May free new_child
-            mark_xml_changes(old_child, new_child, true);
-
-        } else {
-            /* Check for movement, we already checked for differences */
-            int p_new = pcmk__xml_position(new_child, pcmk__xf_skip);
-            int p_old = pcmk__xml_position(old_child, pcmk__xf_skip);
-
-            if(p_old != p_new) {
-                mark_child_moved(old_child, new_xml, new_child, p_old, p_new);
+            if (new_child_matches(old_child, new_child)) {
+                old_nodepriv->match = new_child;
+                new_nodepriv->match = old_child;
+                break;
             }
         }
-
-        new_child = next;
     }
 }
 
-void
-xml_calculate_significant_changes(xmlNode *old_xml, xmlNode *new_xml)
-{
-    pcmk__set_xml_doc_flag(new_xml, pcmk__xf_lazy);
-    xml_calculate_changes(old_xml, new_xml);
-}
-
-// Called functions may set the \p pcmk__xf_skip flag on parts of \p old_xml
-void
-xml_calculate_changes(xmlNode *old_xml, xmlNode *new_xml)
-{
-    CRM_CHECK((old_xml != NULL) && (new_xml != NULL)
-              && pcmk__xe_is(old_xml, (const char *) new_xml->name)
-              && pcmk__str_eq(pcmk__xe_id(old_xml), pcmk__xe_id(new_xml),
-                              pcmk__str_none),
-              return);
-
-    if(xml_tracking_changes(new_xml) == FALSE) {
-        xml_track_changes(new_xml, NULL, NULL, FALSE);
-    }
-
-    mark_xml_changes(old_xml, new_xml, FALSE);
-}
-
 /*!
  * \internal
- * \brief Initialize the Pacemaker XML environment
+ * \brief Mark changes between two XML trees
  *
- * Set an XML buffer allocation scheme, set XML node create and destroy
- * callbacks, and load schemas into the cache.
+ * Set flags in a new XML tree to indicate changes relative to an old XML tree.
+ *
+ * \param[in,out] old_xml  XML before changes
+ * \param[in,out] new_xml  XML after changes
+ *
+ * \note This may set \c pcmk__xf_skip on parts of \p old_xml.
  */
 void
-pcmk__xml_init(void)
+pcmk__xml_mark_changes(xmlNode *old_xml, xmlNode *new_xml)
 {
-    // @TODO Try to find a better caller than crm_log_preinit()
-    static bool initialized = false;
+    /* This function may set the xml_node_private_t:match member on children of
+     * old_xml and new_xml, but it clears that member before returning.
+     *
+     * @TODO Ensure we handle (for example, by copying) or reject user-created
+     * XML that is missing xml_node_private_t at top level or in any children.
+     * Similarly, check handling of node types for which we don't create private
+     * data. For now, we'll skip them in the loops below.
+     */
+    CRM_CHECK((old_xml != NULL) && (new_xml != NULL), return);
+    if ((old_xml->_private == NULL) || (new_xml->_private == NULL)) {
+        return;
+    }
 
-    if (!initialized) {
-        initialized = true;
+    pcmk__xml_doc_set_flags(new_xml->doc, pcmk__xf_tracking);
+    xml_diff_attrs(old_xml, new_xml);
 
-        /* Double the buffer size when the buffer needs to grow. The default
-         * allocator XML_BUFFER_ALLOC_EXACT was found to cause poor performance
-         * due to the number of reallocs.
+    find_matching_children(old_xml, new_xml);
+
+    // Process matches (changed children) and deletions
+    for (xmlNode *old_child = pcmk__xml_first_child(old_xml); old_child != NULL;
+         old_child = pcmk__xml_next(old_child)) {
+
+        xml_node_private_t *nodepriv = old_child->_private;
+        xmlNode *new_child = NULL;
+
+        if (nodepriv == NULL) {
+            continue;
+        }
+
+        if (nodepriv->match == NULL) {
+            // No match in new XML means the old child was deleted
+            mark_child_deleted(old_child, new_xml);
+            continue;
+        }
+
+        /* Fetch the match and clear old_child->_private's match member.
+         * new_child->_private's match member is handled in the new_xml loop.
          */
-        xmlSetBufferAllocationScheme(XML_BUFFER_ALLOC_DOUBLEIT);
+        new_child = nodepriv->match;
+        nodepriv->match = NULL;
 
-        // Load schemas into the cache
-        pcmk__schema_init();
+        pcmk__assert(old_child->type == new_child->type);
+
+        if (old_child->type == XML_COMMENT_NODE) {
+            // Comments match only if their positions and contents match
+            continue;
+        }
+
+        pcmk__xml_mark_changes(old_child, new_child);
     }
-}
 
-/*!
- * \internal
- * \brief Tear down the Pacemaker XML environment
- *
- * Destroy schema cache and clean up memory allocated by libxml2.
- */
-void
-pcmk__xml_cleanup(void)
-{
-    pcmk__schema_cleanup();
-    xmlCleanupParser();
+    /* Mark unmatched new children as created, and mark matched new children as
+     * moved if their positions changed. Grab the next new child in advance,
+     * since new_child may get freed in the loop body.
+     */
+    for (xmlNode *new_child = pcmk__xml_first_child(new_xml),
+                 *next = pcmk__xml_next(new_child);
+         new_child != NULL;
+         new_child = next, next = pcmk__xml_next(new_child)) {
+
+        xml_node_private_t *nodepriv = new_child->_private;
+
+        if (nodepriv == NULL) {
+            continue;
+        }
+
+        if (nodepriv->match != NULL) {
+            /* Fetch the match and clear new_child->_private's match member. Any
+             * changes were marked in the old_xml loop. Mark the move.
+             *
+             * We might be able to mark the move earlier, when we mark changes
+             * for matches in the old_xml loop, consolidating both actions. We'd
+             * have to think about whether the timing of setting the
+             * pcmk__xf_skip flag makes any difference.
+             */
+            xmlNode *old_child = nodepriv->match;
+            int old_pos = pcmk__xml_position(old_child, pcmk__xf_skip);
+            int new_pos = pcmk__xml_position(new_child, pcmk__xf_skip);
+
+            if (old_pos != new_pos) {
+                mark_child_moved(old_child, new_child, old_pos, new_pos);
+            }
+            nodepriv->match = NULL;
+            continue;
+        }
+
+        // No match in old XML means the new child is newly created
+        pcmk__set_xml_flags(nodepriv, pcmk__xf_skip);
+        mark_xml_tree_dirty_created(new_child);
+
+        // Check whether creation was allowed (may free new_child)
+        pcmk__apply_creation_acl(new_child, true);
+    }
 }
 
 char *
@@ -1539,6 +1711,8 @@ pcmk__xml_artefact_path(enum pcmk__xml_artefact_ns ns, const char *filespec)
 // Deprecated functions kept only for backward API compatibility
 // LCOV_EXCL_START
 
+#include <libxml/parser.h>              // xmlCleanupParser()
+
 #include <crm/common/xml_compat.h>
 
 xmlNode *
@@ -1558,13 +1732,14 @@ copy_xml(xmlNode *src)
 void
 crm_xml_init(void)
 {
-    pcmk__xml_init();
+    pcmk__schema_init();
 }
 
 void
 crm_xml_cleanup(void)
 {
-    pcmk__xml_cleanup();
+    pcmk__schema_cleanup();
+    xmlCleanupParser();
 }
 
 void
@@ -1593,5 +1768,91 @@ crm_xml_sanitize_id(char *id)
     }
 }
 
+bool
+xml_tracking_changes(xmlNode *xml)
+{
+    return (xml != NULL)
+           && pcmk__xml_doc_all_flags_set(xml->doc, pcmk__xf_tracking);
+}
+
+bool
+xml_document_dirty(xmlNode *xml)
+{
+    return (xml != NULL)
+           && pcmk__xml_doc_all_flags_set(xml->doc, pcmk__xf_dirty);
+}
+
+void
+xml_accept_changes(xmlNode *xml)
+{
+    if (xml != NULL) {
+        pcmk__xml_commit_changes(xml->doc);
+    }
+}
+
+void
+xml_track_changes(xmlNode *xml, const char *user, xmlNode *acl_source,
+                  bool enforce_acls)
+{
+    if (xml == NULL) {
+        return;
+    }
+
+    pcmk__xml_commit_changes(xml->doc);
+    crm_trace("Tracking changes%s to %p",
+              (enforce_acls? " with ACLs" : ""), xml);
+    pcmk__xml_doc_set_flags(xml->doc, pcmk__xf_tracking);
+    if (enforce_acls) {
+        if (acl_source == NULL) {
+            acl_source = xml;
+        }
+        pcmk__xml_doc_set_flags(xml->doc, pcmk__xf_acl_enabled);
+        pcmk__unpack_acl(acl_source, xml, user);
+        pcmk__apply_acl(xml);
+    }
+}
+
+void
+xml_calculate_changes(xmlNode *old_xml, xmlNode *new_xml)
+{
+    CRM_CHECK((old_xml != NULL) && (new_xml != NULL)
+              && pcmk__xe_is(old_xml, (const char *) new_xml->name)
+              && pcmk__str_eq(pcmk__xe_id(old_xml), pcmk__xe_id(new_xml),
+                              pcmk__str_none),
+              return);
+
+    if (!pcmk__xml_doc_all_flags_set(new_xml->doc, pcmk__xf_tracking)) {
+        // Ensure tracking has a clean start (pcmk__xml_mark_changes() enables)
+        pcmk__xml_commit_changes(new_xml->doc);
+    }
+
+    pcmk__xml_mark_changes(old_xml, new_xml);
+}
+
+void
+xml_calculate_significant_changes(xmlNode *old_xml, xmlNode *new_xml)
+{
+    CRM_CHECK((old_xml != NULL) && (new_xml != NULL)
+              && pcmk__xe_is(old_xml, (const char *) new_xml->name)
+              && pcmk__str_eq(pcmk__xe_id(old_xml), pcmk__xe_id(new_xml),
+                              pcmk__str_none),
+              return);
+
+    /* BUG: If pcmk__xf_tracking is not set for new_xml when this function is
+     * called, then we unset pcmk__xf_ignore_attr_pos via
+     * pcmk__xml_commit_changes(). Since this function is about to be
+     * deprecated, it's not worth fixing this and changing the user-facing
+     * behavior.
+     */
+    pcmk__xml_doc_set_flags(new_xml->doc, pcmk__xf_ignore_attr_pos);
+
+    if (!pcmk__xml_doc_all_flags_set(new_xml->doc, pcmk__xf_tracking)) {
+        // Ensure tracking has a clean start (pcmk__xml_mark_changes() enables)
+        pcmk__xml_commit_changes(new_xml->doc);
+    }
+
+    pcmk__xml_mark_changes(old_xml, new_xml);
+}
+
 // LCOV_EXCL_STOP
 // End deprecated API
