@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,6 +18,7 @@
 
 #include <libxml/tree.h>                // xmlNode, etc.
 #include <libxml/valid.h>               // xmlValidateNameValue()
+#include <libxml/xmlstring.h>           // xmlChar
 
 #include <crm/crm.h>
 #include <crm/common/nvpair.h>          // crm_xml_add(), etc.
@@ -43,17 +44,16 @@ pcmk__xe_first_child(const xmlNode *parent, const char *node_name,
                      const char *attr_n, const char *attr_v)
 {
     xmlNode *child = NULL;
-    const char *parent_name = "<null>";
 
     CRM_CHECK((attr_v == NULL) || (attr_n != NULL), return NULL);
 
-    if (parent != NULL) {
-        child = parent->children;
-        while ((child != NULL) && (child->type != XML_ELEMENT_NODE)) {
-            child = child->next;
-        }
+    if (parent == NULL) {
+        return NULL;
+    }
 
-        parent_name = (const char *) parent->name;
+    child = parent->children;
+    while ((child != NULL) && (child->type != XML_ELEMENT_NODE)) {
+        child = child->next;
     }
 
     for (; child != NULL; child = pcmk__xe_next(child, NULL)) {
@@ -80,15 +80,13 @@ pcmk__xe_first_child(const xmlNode *parent, const char *node_name,
         }
     }
 
-    if (node_name == NULL) {
-        node_name = "(any)";    // For logging
-    }
-    if (attr_n != NULL) {
-        crm_trace("XML child node <%s %s=%s> not found in %s",
-                  node_name, attr_n, attr_v, parent_name);
+    if (attr_n == NULL) {
+        crm_trace("%s XML has no child element of %s type",
+                  (const char *) parent->name, pcmk__s(node_name, "any"));
     } else {
-        crm_trace("XML child node <%s> not found in %s",
-                  node_name, parent_name);
+        crm_trace("%s XML has no child element of %s type with %s='%s'",
+                  (const char *) parent->name, pcmk__s(node_name, "any"),
+                  attr_n, attr_v);
     }
     return NULL;
 }
@@ -339,7 +337,7 @@ void
 pcmk__xe_remove_attr(xmlNode *element, const char *name)
 {
     if (name != NULL) {
-        pcmk__xa_remove(xmlHasProp(element, (pcmkXmlStr) name), false);
+        pcmk__xa_remove(xmlHasProp(element, (const xmlChar *) name), false);
     }
 }
 
@@ -371,12 +369,14 @@ pcmk__xe_remove_attr_cb(xmlNode *xml, void *user_data)
  * \brief Remove an XML element's attributes that match some criteria
  *
  * \param[in,out] element    XML element to modify
+ * \param[in]     force      If \c true, remove matching attributes immediately,
+ *                           ignoring ACLs and change tracking
  * \param[in]     match      If not NULL, only remove attributes for which
  *                           this function returns true
  * \param[in,out] user_data  Data to pass to \p match
  */
 void
-pcmk__xe_remove_matching_attrs(xmlNode *element,
+pcmk__xe_remove_matching_attrs(xmlNode *element, bool force,
                                bool (*match)(xmlAttrPtr, void *),
                                void *user_data)
 {
@@ -385,7 +385,7 @@ pcmk__xe_remove_matching_attrs(xmlNode *element,
     for (xmlAttrPtr a = pcmk__xe_first_attr(element); a != NULL; a = next) {
         next = a->next; // Grab now because attribute might get removed
         if ((match == NULL) || match(a, user_data)) {
-            if (pcmk__xa_remove(a, false) != pcmk_rc_ok) {
+            if (pcmk__xa_remove(a, force) != pcmk_rc_ok) {
                 return;
             }
         }
@@ -413,13 +413,13 @@ pcmk__xe_create(xmlNode *parent, const char *name)
     if (parent == NULL) {
         xmlDoc *doc = pcmk__xml_new_doc();
 
-        node = xmlNewDocRawNode(doc, NULL, (pcmkXmlStr) name, NULL);
+        node = xmlNewDocRawNode(doc, NULL, (const xmlChar *) name, NULL);
         pcmk__mem_assert(node);
 
         xmlDocSetRootElement(doc, node);
 
     } else {
-        node = xmlNewChild(parent, NULL, (pcmkXmlStr) name, NULL);
+        node = xmlNewChild(parent, NULL, (const xmlChar *) name, NULL);
         pcmk__mem_assert(node);
     }
 
@@ -474,7 +474,7 @@ pcmk__xe_set_content(xmlNode *node, const char *format, ...)
             va_end(ap);
         }
 
-        xmlNodeSetContent(node, (pcmkXmlStr) content);
+        xmlNodeSetContent(node, (const xmlChar *) content);
         free(buf);
     }
 }
@@ -507,7 +507,7 @@ pcmk__xe_set_id(xmlNode *node, const char *format, ...)
     pcmk__assert(vasprintf(&id, format, ap) >= 0);
     va_end(ap);
 
-    if (!xmlValidateNameValue((pcmkXmlStr) id)) {
+    if (!xmlValidateNameValue((const xmlChar *) id)) {
         pcmk__xml_sanitize_id(id);
     }
     crm_xml_add(node, PCMK_XA_ID, id);
@@ -756,11 +756,11 @@ replace_node(xmlNode *old, xmlNode *new)
     // May be unnecessary but avoids slight changes to some test outputs
     pcmk__xml_tree_foreach(new, pcmk__xml_reset_node_flags, NULL);
 
-    if (xml_tracking_changes(new)) {
+    if (pcmk__xml_doc_all_flags_set(new->doc, pcmk__xf_tracking)) {
         // Replaced sections may have included relevant ACLs
         pcmk__apply_acl(new);
     }
-    xml_calculate_changes(old, new);
+    pcmk__xml_mark_changes(old, new);
     pcmk__xml_free_node(old);
 }
 
@@ -1025,7 +1025,7 @@ crm_xml_add(xmlNode *node, const char *name, const char *value)
         return NULL;
     }
 
-    if (pcmk__tracking_xml_changes(node, FALSE)) {
+    if (pcmk__xml_doc_all_flags_set(node->doc, pcmk__xf_tracking)) {
         const char *old = crm_element_value(node, name);
 
         if (old == NULL || value == NULL || strcmp(old, value) != 0) {
@@ -1038,7 +1038,7 @@ crm_xml_add(xmlNode *node, const char *name, const char *value)
         return NULL;
     }
 
-    attr = xmlSetProp(node, (pcmkXmlStr) name, (pcmkXmlStr) value);
+    attr = xmlSetProp(node, (const xmlChar *) name, (const xmlChar *) value);
 
     /* If the attribute already exists, this does nothing. Attribute values
      * don't get private data.
@@ -1179,7 +1179,7 @@ crm_element_value(const xmlNode *data, const char *name)
         return NULL;
     }
 
-    attr = xmlHasProp(data, (pcmkXmlStr) name);
+    attr = xmlHasProp(data, (const xmlChar *) name);
     if (!attr || !attr->children) {
         return NULL;
     }
