@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -16,7 +16,9 @@
 #include <libgen.h>
 #include <inttypes.h>
 #include <sys/types.h>
+
 #include <glib.h>
+#include <libxml/tree.h>            // xmlNode
 
 #include <crm/crm.h>
 #include <crm/stonith-ng.h>
@@ -63,8 +65,7 @@ static void log_action(stonith_action_t *action, pid_t pid);
 static void
 set_result_from_svc_action(stonith_action_t *action, svc_action_t *svc_action)
 {
-    pcmk__set_result(&(action->result), svc_action->rc, svc_action->status,
-                     services__exit_reason(svc_action));
+    services__copy_result(svc_action, &(action->result));
     pcmk__set_result_output(&(action->result),
                             services__grab_stdout(svc_action),
                             services__grab_stderr(svc_action));
@@ -109,20 +110,19 @@ append_config_arg(gpointer key, gpointer value, gpointer user_data)
  * \internal
  * \brief Create a table of arguments for a fencing action
  *
- * \param[in] agent          Fencing agent name
- * \param[in] action         Name of fencing action
- * \param[in] target         Name of target node for fencing action
- * \param[in] target_nodeid  Node ID of target node for fencing action
- * \param[in] device_args    Fence device parameters
- * \param[in] port_map       Target node-to-port mapping for fence device
- * \param[in] host_arg       Argument name for passing target
+ * \param[in] agent             Fencing agent name
+ * \param[in] action            Name of fencing action
+ * \param[in] target            Name of target node for fencing action
+ * \param[in] device_args       Fence device parameters
+ * \param[in] port_map          Target node-to-port mapping for fence device
+ * \param[in] default_host_arg  Default agent parameter for passing target
  *
  * \return Newly created hash table of arguments for fencing action
  */
 static GHashTable *
 make_args(const char *agent, const char *action, const char *target,
-          uint32_t target_nodeid, GHashTable *device_args,
-          GHashTable *port_map, const char *host_arg)
+          GHashTable *device_args, GHashTable *port_map,
+          const char *default_host_arg)
 {
     GHashTable *arg_list = NULL;
     const char *value = NULL;
@@ -158,21 +158,11 @@ make_args(const char *agent, const char *action, const char *target,
          */
         pcmk__insert_dup(arg_list, "nodename", target);
 
-        // If the target's node ID was specified, pass it, too
-        if (target_nodeid != 0) {
-            char *nodeid = crm_strdup_printf("%" PRIu32, target_nodeid);
-
-            // cts-fencing looks for this log message
-            crm_info("Passing '%s' as nodeid with fence action '%s' targeting %s",
-                     nodeid, action, pcmk__s(target, "no node"));
-            g_hash_table_insert(arg_list, strdup("nodeid"), nodeid);
-        }
-
         // Check whether target should be specified as some other argument
         param = g_hash_table_lookup(device_args, PCMK_STONITH_HOST_ARGUMENT);
         if (param == NULL) {
             // Use caller's default (likely from agent metadata)
-            param = host_arg;
+            param = default_host_arg;
         }
         if ((param != NULL)
             && !pcmk__str_eq(agent, "fence_legacy", pcmk__str_none)
@@ -249,27 +239,26 @@ stonith__action_result(stonith_action_t *action)
  * \internal
  * \brief Create a new fencing action to be executed
  *
- * \param[in] agent          Fence agent to use
- * \param[in] action_name    Fencing action to be executed
- * \param[in] target         Name of target of fencing action (if known)
- * \param[in] target_nodeid  Node ID of target of fencing action (if known)
- * \param[in] timeout_sec    Timeout to be used when executing action
- * \param[in] device_args    Parameters to pass to fence agent
- * \param[in] port_map       Mapping of target names to device ports
- * \param[in] host_arg       Agent parameter used to pass target name
+ * \param[in] agent             Fence agent to use
+ * \param[in] action_name       Fencing action to be executed
+ * \param[in] target            Name of target of fencing action (if known)
+ * \param[in] timeout_sec       Timeout to be used when executing action
+ * \param[in] device_args       Parameters to pass to fence agent
+ * \param[in] port_map          Mapping of target names to device ports
+ * \param[in] default_host_arg  Default agent parameter for passing target
  *
  * \return Newly created fencing action (asserts on error, never NULL)
  */
 stonith_action_t *
 stonith__action_create(const char *agent, const char *action_name,
-                       const char *target, uint32_t target_nodeid,
-                       int timeout_sec, GHashTable *device_args,
-                       GHashTable *port_map, const char *host_arg)
+                       const char *target, int timeout_sec,
+                       GHashTable *device_args, GHashTable *port_map,
+                       const char *default_host_arg)
 {
     stonith_action_t *action = pcmk__assert_alloc(1, sizeof(stonith_action_t));
 
-    action->args = make_args(agent, action_name, target, target_nodeid,
-                             device_args, port_map, host_arg);
+    action->args = make_args(agent, action_name, target, device_args, port_map,
+                             default_host_arg);
     crm_debug("Preparing '%s' action targeting %s using agent %s",
               action_name, pcmk__s(target, "no node"), agent);
     action->agent = strdup(agent);
@@ -469,13 +458,16 @@ stonith__xe_set_result(xmlNode *xml, const pcmk__action_result_t *result)
 xmlNode *
 stonith__find_xe_with_result(xmlNode *xml)
 {
-    xmlNode *match = get_xpath_object("//@" PCMK__XA_RC_CODE, xml, LOG_NEVER);
+    xmlNode *match = pcmk__xpath_find_one(xml->doc,
+                                          "//*[@" PCMK__XA_RC_CODE "]",
+                                          LOG_NEVER);
 
     if (match == NULL) {
         /* @COMPAT Peers <=2.1.2 in a rolling upgrade provide only a legacy
          * return code, not a full result, so check for that.
          */
-        match = get_xpath_object("//@" PCMK__XA_ST_RC, xml, LOG_ERR);
+        match = pcmk__xpath_find_one(xml->doc, "//*[@" PCMK__XA_ST_RC "]",
+                                     LOG_ERR);
     }
     return match;
 }
@@ -572,14 +564,54 @@ stonith_action_async_forked(svc_action_t *svc_action)
               action->pid, action->action);
 }
 
+/*!
+ * \internal
+ * \brief Convert a fencing library action to a services library action
+ *
+ * \param[in,out] action  Fencing library action to convert
+ *
+ * \return Services library action equivalent to \p action on success; on error,
+ *         NULL will be returned and \p action's result will be set
+ */
+static svc_action_t *
+stonith_action_to_svc(stonith_action_t *action)
+{
+    static int stonith_sequence = 0;
+
+    char *path = crm_strdup_printf(PCMK__FENCE_BINDIR "/%s", action->agent);
+    svc_action_t *svc_action = services_action_create_generic(path, NULL);
+
+    free(path);
+    if (svc_action->rc != PCMK_OCF_UNKNOWN) {
+        set_result_from_svc_action(action, svc_action);
+        services_action_free(svc_action);
+        return NULL;
+    }
+
+    svc_action->timeout = action->remaining_timeout * 1000;
+    svc_action->standard = pcmk__str_copy(PCMK_RESOURCE_CLASS_STONITH);
+    svc_action->id = crm_strdup_printf("%s_%s_%dof%d", action->agent,
+                                       action->action, action->tries,
+                                       action->max_retries);
+    svc_action->agent = pcmk__str_copy(action->agent);
+    svc_action->sequence = stonith_sequence++;
+    svc_action->params = action->args;
+    svc_action->cb_data = (void *) action;
+    svc_action->flags = pcmk__set_flags_as(__func__, __LINE__,
+                                           LOG_TRACE, "Action",
+                                           svc_action->id, svc_action->flags,
+                                           SVC_ACTION_NON_BLOCKED,
+                                           "SVC_ACTION_NON_BLOCKED");
+
+    return svc_action;
+}
+
 static int
 internal_stonith_action_execute(stonith_action_t * action)
 {
-    int rc = -EPROTO;
+    int rc = pcmk_ok;
     int is_retry = 0;
     svc_action_t *svc_action = NULL;
-    static int stonith_sequence = 0;
-    char *buffer = NULL;
 
     CRM_CHECK(action != NULL, return -EINVAL);
 
@@ -590,46 +622,28 @@ internal_stonith_action_execute(stonith_action_t * action)
         return -EINVAL;
     }
 
-    if (!action->tries) {
+    if (action->tries++ == 0) {
+        // First attempt of the desired action
         action->initial_start_time = time(NULL);
-    }
-    action->tries++;
-
-    if (action->tries > 1) {
-        crm_info("Attempt %d to execute %s (%s). remaining timeout is %d",
-                 action->tries, action->agent, action->action, action->remaining_timeout);
+    } else {
+        // Later attempt after earlier failure
+        crm_info("Attempt %d to execute '%s' action of agent %s "
+                 "(%ds timeout remaining)",
+                 action->tries, action->action, action->agent,
+                 action->remaining_timeout);
         is_retry = 1;
     }
 
-    buffer = crm_strdup_printf(PCMK__FENCE_BINDIR "/%s",
-                               basename(action->agent));
-    svc_action = services_action_create_generic(buffer, NULL);
-    free(buffer);
-
-    if (svc_action->rc != PCMK_OCF_UNKNOWN) {
-        set_result_from_svc_action(action, svc_action);
-        services_action_free(svc_action);
+    svc_action = stonith_action_to_svc(action);
+    if (svc_action == NULL) {
+        // The only possible errors are out-of-memory and too many arguments
         return -E2BIG;
     }
 
-    svc_action->timeout = 1000 * action->remaining_timeout;
-    svc_action->standard = strdup(PCMK_RESOURCE_CLASS_STONITH);
-    svc_action->id = crm_strdup_printf("%s_%s_%dof%d", basename(action->agent),
-                                       action->action, action->tries,
-                                       action->max_retries);
-    svc_action->agent = strdup(action->agent);
-    svc_action->sequence = stonith_sequence++;
-    svc_action->params = action->args;
-    svc_action->cb_data = (void *) action;
-    svc_action->flags = pcmk__set_flags_as(__func__, __LINE__,
-                                           LOG_TRACE, "Action",
-                                           svc_action->id, svc_action->flags,
-                                           SVC_ACTION_NON_BLOCKED,
-                                           "SVC_ACTION_NON_BLOCKED");
-
     /* keep retries from executing out of control and free previous results */
     if (is_retry) {
         pcmk__reset_result(&(action->result));
+        // @TODO This should be nonblocking via timer if mainloop is used
         sleep(1);
     }
 
@@ -640,11 +654,8 @@ internal_stonith_action_execute(stonith_action_t * action)
                                               &stonith_action_async_forked));
         return pcmk_ok;
 
-    } else if (services_action_sync(svc_action)) { // sync success
-        rc = pcmk_ok;
-
-    } else { // sync failure
-        rc = -ECONNABORTED;
+    } else if (!services_action_sync(svc_action)) {
+        rc = -ECONNABORTED; // @TODO Update API to return more useful error
     }
 
     set_result_from_svc_action(action, svc_action);
