@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2024 the Pacemaker project contributors
+ * Copyright 2019-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -48,6 +48,26 @@ timespec_string(time_t sec, long nsec, bool show_usec) {
                               |(show_usec? crm_time_usecs : 0));
 }
 
+/*!
+ * \internal
+ * \brief Return a readable string equivalent of a fencing history item's action
+ *
+ * \param[in] history  Fencing history entry
+ *
+ * \return Readable string equivalent of action belonging to \p history
+ */
+static const char *
+history_action_text(const stonith_history_t *history)
+{
+    if (pcmk__str_eq(history->action, PCMK_ACTION_ON, pcmk__str_none)) {
+        return "unfencing";
+    }
+    if (pcmk__str_eq(history->action, PCMK_ACTION_OFF, pcmk__str_none)) {
+        return "turning off";
+    }
+    return pcmk__s(history->action, "fencing");
+}
+
 /*!
  * \internal
  * \brief Return a status-friendly description of fence history entry state
@@ -55,7 +75,7 @@ timespec_string(time_t sec, long nsec, bool show_usec) {
  * \param[in] history  Fence history entry to describe
  *
  * \return One-word description of history entry state
- * \note This is similar to stonith_op_state_str() except user-oriented (i.e.
+ * \note This is similar to stonith__op_state_text() except user-oriented (i.e.,
  *       for cluster status) instead of developer-oriented (for debug logs).
  */
 static const char *
@@ -98,8 +118,7 @@ stonith__history_description(const stonith_history_t *history,
                                            history->completed_nsec, true);
     }
 
-    pcmk__g_strcat(str,
-                   stonith_action_str(history->action), " of ", history->target,
+    pcmk__g_strcat(str, history_action_text(history), " of ", history->target,
                    NULL);
 
     if (!pcmk_is_set(show_opts, pcmk_show_failed_detail)) {
