@@ -63,9 +63,9 @@ is_op_dup(const pcmk_resource_t *rsc, const char *name, guint interval_ms)
 {
     const char *id = NULL;
 
-    for (xmlNode *op = pcmk__xe_first_child(rsc->ops_xml, PCMK_XE_OP, NULL,
-                                            NULL);
-         op != NULL; op = pcmk__xe_next_same(op)) {
+    for (xmlNode *op = pcmk__xe_first_child(rsc->priv->ops_xml, PCMK_XE_OP,
+                                            NULL, NULL);
+         op != NULL; op = pcmk__xe_next(op, PCMK_XE_OP)) {
 
         // Check whether action name and interval match
         if (!pcmk__str_eq(crm_element_value(op, PCMK_XA_NAME), name,
@@ -208,14 +208,14 @@ active_recurring_should_be_optional(const pcmk_resource_t *rsc,
         return false;
     }
 
-    if (!pcmk_is_set(rsc->cmds->action_flags(start, NULL),
-                     pcmk_action_optional)) {
+    if (!pcmk_is_set(rsc->priv->cmds->action_flags(start, NULL),
+                     pcmk__action_optional)) {
         pcmk__rsc_trace(rsc, "%s will be mandatory because %s is",
                         key, start->uuid);
         return false;
     }
 
-    possible_matches = find_actions_exact(rsc->actions, key, node);
+    possible_matches = find_actions_exact(rsc->priv->actions, key, node);
     if (possible_matches == NULL) {
         pcmk__rsc_trace(rsc,
                         "%s will be mandatory because it is not active on %s",
@@ -228,7 +228,7 @@ active_recurring_should_be_optional(const pcmk_resource_t *rsc,
 
         const pcmk_action_t *op = (const pcmk_action_t *) iter->data;
 
-        if (pcmk_is_set(op->flags, pcmk_action_reschedule)) {
+        if (pcmk_is_set(op->flags, pcmk__action_reschedule)) {
             pcmk__rsc_trace(rsc,
                             "%s will be mandatory because "
                             "it needs to be rescheduled", key);
@@ -270,9 +270,9 @@ recurring_op_for_active(pcmk_resource_t *rsc, pcmk_action_t *start,
     // Check whether monitor's role matches role resource will have
     if (monitor_role == pcmk_role_unknown) {
         monitor_role = pcmk_role_unpromoted;
-        role_match = (rsc->next_role != pcmk_role_promoted);
+        role_match = (rsc->priv->next_role != pcmk_role_promoted);
     } else {
-        role_match = (rsc->next_role == monitor_role);
+        role_match = (rsc->priv->next_role == monitor_role);
     }
 
     if (!role_match) {
@@ -282,13 +282,13 @@ recurring_op_for_active(pcmk_resource_t *rsc, pcmk_action_t *start,
                                                                op->interval_ms,
                                                                node);
 
-            switch (rsc->role) {
+            switch (rsc->priv->orig_role) {
                 case pcmk_role_unpromoted:
                 case pcmk_role_started:
-                    if (rsc->next_role == pcmk_role_promoted) {
+                    if (rsc->priv->next_role == pcmk_role_promoted) {
                         after_key = promote_key(rsc);
 
-                    } else if (rsc->next_role == pcmk_role_stopped) {
+                    } else if (rsc->priv->next_role == pcmk_role_stopped) {
                         after_key = stop_key(rsc);
                     }
 
@@ -303,7 +303,7 @@ recurring_op_for_active(pcmk_resource_t *rsc, pcmk_action_t *start,
             if (after_key) {
                 pcmk__new_ordering(rsc, NULL, cancel_op, rsc, after_key, NULL,
                                    pcmk__ar_unrunnable_first_blocks,
-                                   rsc->cluster);
+                                   rsc->priv->scheduler);
             }
         }
 
@@ -312,66 +312,66 @@ recurring_op_for_active(pcmk_resource_t *rsc, pcmk_action_t *start,
                    "(not %s)",
                    (is_optional? "Cancelling" : "Ignoring"), op->key, op->id,
                    pcmk_role_text(monitor_role),
-                   pcmk_role_text(rsc->next_role));
+                   pcmk_role_text(rsc->priv->next_role));
         return;
     }
 
     pcmk__rsc_trace(rsc,
                     "Creating %s recurring action %s for %s (%s %s on %s)",
                     (is_optional? "optional" : "mandatory"), op->key,
-                    op->id, rsc->id, pcmk_role_text(rsc->next_role),
+                    op->id, rsc->id, pcmk_role_text(rsc->priv->next_role),
                     pcmk__node_name(node));
 
     mon = custom_action(rsc, strdup(op->key), op->name, node, is_optional,
-                        rsc->cluster);
+                        rsc->priv->scheduler);
 
-    if (!pcmk_is_set(start->flags, pcmk_action_runnable)) {
+    if (!pcmk_is_set(start->flags, pcmk__action_runnable)) {
         pcmk__rsc_trace(rsc, "%s is unrunnable because start is", mon->uuid);
-        pcmk__clear_action_flags(mon, pcmk_action_runnable);
+        pcmk__clear_action_flags(mon, pcmk__action_runnable);
 
     } else if ((node == NULL) || !node->details->online
                || node->details->unclean) {
         pcmk__rsc_trace(rsc, "%s is unrunnable because no node is available",
                         mon->uuid);
-        pcmk__clear_action_flags(mon, pcmk_action_runnable);
+        pcmk__clear_action_flags(mon, pcmk__action_runnable);
 
-    } else if (!pcmk_is_set(mon->flags, pcmk_action_optional)) {
+    } else if (!pcmk_is_set(mon->flags, pcmk__action_optional)) {
         pcmk__rsc_info(rsc, "Start %s-interval %s for %s on %s",
                        pcmk__readable_interval(op->interval_ms), mon->task,
                        rsc->id, pcmk__node_name(node));
     }
 
-    if (rsc->next_role == pcmk_role_promoted) {
+    if (rsc->priv->next_role == pcmk_role_promoted) {
         pe__add_action_expected_result(mon, CRM_EX_PROMOTED);
     }
 
     // Order monitor relative to other actions
-    if ((node == NULL) || pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+    if ((node == NULL) || pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
         pcmk__new_ordering(rsc, start_key(rsc), NULL,
                            NULL, strdup(mon->uuid), mon,
                            pcmk__ar_first_implies_then
                            |pcmk__ar_unrunnable_first_blocks,
-                           rsc->cluster);
+                           rsc->priv->scheduler);
 
         pcmk__new_ordering(rsc, reload_key(rsc), NULL,
                            NULL, strdup(mon->uuid), mon,
                            pcmk__ar_first_implies_then
                            |pcmk__ar_unrunnable_first_blocks,
-                           rsc->cluster);
+                           rsc->priv->scheduler);
 
-        if (rsc->next_role == pcmk_role_promoted) {
+        if (rsc->priv->next_role == pcmk_role_promoted) {
             pcmk__new_ordering(rsc, promote_key(rsc), NULL,
                                rsc, NULL, mon,
                                pcmk__ar_ordered
                                |pcmk__ar_unrunnable_first_blocks,
-                               rsc->cluster);
+                               rsc->priv->scheduler);
 
-        } else if (rsc->role == pcmk_role_promoted) {
+        } else if (rsc->priv->orig_role == pcmk_role_promoted) {
             pcmk__new_ordering(rsc, demote_key(rsc), NULL,
                                rsc, NULL, mon,
                                pcmk__ar_ordered
                                |pcmk__ar_unrunnable_first_blocks,
-                               rsc->cluster);
+                               rsc->priv->scheduler);
         }
     }
 }
@@ -390,7 +390,8 @@ static void
 cancel_if_running(pcmk_resource_t *rsc, const pcmk_node_t *node,
                   const char *key, const char *name, guint interval_ms)
 {
-    GList *possible_matches = find_actions_exact(rsc->actions, key, node);
+    GList *possible_matches = find_actions_exact(rsc->priv->actions, key,
+                                                 node);
     pcmk_action_t *cancel_op = NULL;
 
     if (possible_matches == NULL) {
@@ -400,7 +401,7 @@ cancel_if_running(pcmk_resource_t *rsc, const pcmk_node_t *node,
 
     cancel_op = pcmk__new_cancel_action(rsc, name, interval_ms, node);
 
-    switch (rsc->next_role) {
+    switch (rsc->priv->next_role) {
         case pcmk_role_started:
         case pcmk_role_unpromoted:
             /* Order starts after cancel. If the current role is
@@ -410,7 +411,8 @@ cancel_if_running(pcmk_resource_t *rsc, const pcmk_node_t *node,
              */
             pcmk__new_ordering(rsc, NULL, cancel_op,
                                rsc, start_key(rsc), NULL,
-                               pcmk__ar_unrunnable_first_blocks, rsc->cluster);
+                               pcmk__ar_unrunnable_first_blocks,
+                               rsc->priv->scheduler);
             break;
         default:
             break;
@@ -419,7 +421,8 @@ cancel_if_running(pcmk_resource_t *rsc, const pcmk_node_t *node,
                    "Cancelling %s-interval %s action for %s on %s because "
                    "configured for " PCMK_ROLE_STOPPED " role (not %s)",
                    pcmk__readable_interval(interval_ms), name, rsc->id,
-                   pcmk__node_name(node), pcmk_role_text(rsc->next_role));
+                   pcmk__node_name(node),
+                   pcmk_role_text(rsc->priv->next_role));
 }
 
 /*!
@@ -460,26 +463,26 @@ order_after_stops(pcmk_resource_t *rsc, const pcmk_node_t *node,
     for (GList *iter = stop_ops; iter != NULL; iter = iter->next) {
         pcmk_action_t *stop = (pcmk_action_t *) iter->data;
 
-        if (!pcmk_is_set(stop->flags, pcmk_action_optional)
-            && !pcmk_is_set(action->flags, pcmk_action_optional)
-            && !pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+        if (!pcmk_is_set(stop->flags, pcmk__action_optional)
+            && !pcmk_is_set(action->flags, pcmk__action_optional)
+            && !pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
             pcmk__rsc_trace(rsc, "%s optional on %s: unmanaged",
                             action->uuid, pcmk__node_name(node));
-            pcmk__set_action_flags(action, pcmk_action_optional);
+            pcmk__set_action_flags(action, pcmk__action_optional);
         }
 
-        if (!pcmk_is_set(stop->flags, pcmk_action_runnable)) {
+        if (!pcmk_is_set(stop->flags, pcmk__action_runnable)) {
             crm_debug("%s unrunnable on %s: stop is unrunnable",
                       action->uuid, pcmk__node_name(node));
-            pcmk__clear_action_flags(action, pcmk_action_runnable);
+            pcmk__clear_action_flags(action, pcmk__action_runnable);
         }
 
-        if (pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+        if (pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
             pcmk__new_ordering(rsc, stop_key(rsc), stop,
                                NULL, NULL, action,
                                pcmk__ar_first_implies_then
                                |pcmk__ar_unrunnable_first_blocks,
-                               rsc->cluster);
+                               rsc->priv->scheduler);
         }
     }
     g_list_free(stop_ops);
@@ -504,7 +507,7 @@ recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
         return;
     }
 
-    if (!pcmk_is_set(rsc->flags, pcmk_rsc_unique)) {
+    if (!pcmk_is_set(rsc->flags, pcmk__rsc_unique)) {
         crm_notice("Ignoring %s (recurring monitors for " PCMK_ROLE_STOPPED
                    " role are not supported for anonymous clones)", op->id);
         return; // @TODO add support
@@ -514,7 +517,9 @@ recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
                     "Creating recurring action %s for %s on nodes "
                     "where it should not be running", op->id, rsc->id);
 
-    for (GList *iter = rsc->cluster->nodes; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc->priv->scheduler->nodes;
+         iter != NULL; iter = iter->next) {
+
         pcmk_node_t *stop_node = (pcmk_node_t *) iter->data;
 
         bool is_optional = true;
@@ -522,14 +527,15 @@ recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
 
         // Cancel action on node where resource will be active
         if ((node != NULL)
-            && pcmk__str_eq(stop_node->details->uname, node->details->uname,
+            && pcmk__str_eq(stop_node->priv->name, node->priv->name,
                             pcmk__str_casei)) {
             cancel_if_running(rsc, node, op->key, op->name, op->interval_ms);
             continue;
         }
 
         // Recurring action on this node is optional if it's already active here
-        possible_matches = find_actions_exact(rsc->actions, op->key, stop_node);
+        possible_matches = find_actions_exact(rsc->priv->actions, op->key,
+                                              stop_node);
         is_optional = (possible_matches != NULL);
         g_list_free(possible_matches);
 
@@ -540,11 +546,11 @@ recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
                         op->key, op->id, rsc->id, pcmk__node_name(stop_node));
 
         stopped_mon = custom_action(rsc, strdup(op->key), op->name, stop_node,
-                                    is_optional, rsc->cluster);
+                                    is_optional, rsc->priv->scheduler);
 
         pe__add_action_expected_result(stopped_mon, CRM_EX_NOT_RUNNING);
 
-        if (pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+        if (pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
             order_after_probes(rsc, stop_node, stopped_mon);
         }
 
@@ -556,11 +562,11 @@ recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
         if (!stop_node->details->online || stop_node->details->unclean) {
             pcmk__rsc_debug(rsc, "%s unrunnable on %s: node unavailable)",
                             stopped_mon->uuid, pcmk__node_name(stop_node));
-            pcmk__clear_action_flags(stopped_mon, pcmk_action_runnable);
+            pcmk__clear_action_flags(stopped_mon, pcmk__action_runnable);
         }
 
-        if (pcmk_is_set(stopped_mon->flags, pcmk_action_runnable)
-            && !pcmk_is_set(stopped_mon->flags, pcmk_action_optional)) {
+        if (pcmk_is_set(stopped_mon->flags, pcmk__action_runnable)
+            && !pcmk_is_set(stopped_mon->flags, pcmk__action_optional)) {
             crm_notice("Start recurring %s-interval %s for "
                        PCMK_ROLE_STOPPED " %s on %s",
                        pcmk__readable_interval(op->interval_ms),
@@ -580,41 +586,41 @@ pcmk__create_recurring_actions(pcmk_resource_t *rsc)
 {
     pcmk_action_t *start = NULL;
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_blocked)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_blocked)) {
         pcmk__rsc_trace(rsc,
                         "Skipping recurring actions for blocked resource %s",
                         rsc->id);
         return;
     }
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_maintenance)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_maintenance)) {
         pcmk__rsc_trace(rsc,
                         "Skipping recurring actions for %s "
                         "in maintenance mode", rsc->id);
         return;
     }
 
-    if (rsc->allocated_to == NULL) {
+    if (rsc->priv->assigned_node == NULL) {
         // Recurring actions for active roles not needed
 
-    } else if (rsc->allocated_to->details->maintenance) {
+    } else if (rsc->priv->assigned_node->details->maintenance) {
         pcmk__rsc_trace(rsc,
                         "Skipping recurring actions for %s on %s "
                         "in maintenance mode",
-                        rsc->id, pcmk__node_name(rsc->allocated_to));
+                        rsc->id, pcmk__node_name(rsc->priv->assigned_node));
 
-    } else if ((rsc->next_role != pcmk_role_stopped)
-        || !pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+    } else if ((rsc->priv->next_role != pcmk_role_stopped)
+               || !pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
         // Recurring actions for active roles needed
-        start = start_action(rsc, rsc->allocated_to, TRUE);
+        start = start_action(rsc, rsc->priv->assigned_node, TRUE);
     }
 
     pcmk__rsc_trace(rsc, "Creating any recurring actions needed for %s",
                     rsc->id);
 
-    for (xmlNode *op = pcmk__xe_first_child(rsc->ops_xml, PCMK_XE_OP, NULL,
-                                            NULL);
-         op != NULL; op = pcmk__xe_next_same(op)) {
+    for (xmlNode *op = pcmk__xe_first_child(rsc->priv->ops_xml, PCMK_XE_OP,
+                                            NULL, NULL);
+         op != NULL; op = pcmk__xe_next(op, PCMK_XE_OP)) {
 
         struct op_history op_history = { NULL, };
 
@@ -623,9 +629,11 @@ pcmk__create_recurring_actions(pcmk_resource_t *rsc)
         }
 
         if (start != NULL) {
-            recurring_op_for_active(rsc, start, rsc->allocated_to, &op_history);
+            recurring_op_for_active(rsc, start, rsc->priv->assigned_node,
+                                    &op_history);
         }
-        recurring_op_for_inactive(rsc, rsc->allocated_to, &op_history);
+        recurring_op_for_inactive(rsc, rsc->priv->assigned_node,
+                                  &op_history);
 
         free(op_history.key);
     }
@@ -650,7 +658,7 @@ pcmk__new_cancel_action(pcmk_resource_t *rsc, const char *task,
     char *key = NULL;
     char *interval_ms_s = NULL;
 
-    CRM_ASSERT((rsc != NULL) && (task != NULL) && (node != NULL));
+    pcmk__assert((rsc != NULL) && (task != NULL) && (node != NULL));
 
     key = pcmk__op_key(rsc->id, task, interval_ms);
 
@@ -658,7 +666,7 @@ pcmk__new_cancel_action(pcmk_resource_t *rsc, const char *task,
      * cancel_op->task.
      */
     cancel_op = custom_action(rsc, key, PCMK_ACTION_CANCEL, node, FALSE,
-                              rsc->cluster);
+                              rsc->priv->scheduler);
 
     pcmk__str_update(&(cancel_op->task), PCMK_ACTION_CANCEL);
     pcmk__str_update(&(cancel_op->cancel_task), task);
@@ -701,7 +709,7 @@ pcmk__schedule_cancel(pcmk_resource_t *rsc, const char *call_id,
 
     // Cancellations happen after stops
     pcmk__new_ordering(rsc, stop_key(rsc), NULL, rsc, NULL, cancel,
-                       pcmk__ar_ordered, rsc->cluster);
+                       pcmk__ar_ordered, rsc->priv->scheduler);
 }
 
 /*!
@@ -720,10 +728,10 @@ pcmk__reschedule_recurring(pcmk_resource_t *rsc, const char *task,
     pcmk_action_t *op = NULL;
 
     trigger_unfencing(rsc, node, "Device parameters changed (reschedule)",
-                      NULL, rsc->cluster);
+                      NULL, rsc->priv->scheduler);
     op = custom_action(rsc, pcmk__op_key(rsc->id, task, interval_ms),
-                       task, node, TRUE, rsc->cluster);
-    pcmk__set_action_flags(op, pcmk_action_reschedule);
+                       task, node, TRUE, rsc->priv->scheduler);
+    pcmk__set_action_flags(op, pcmk__action_reschedule);
 }
 
 /*!
