@@ -62,12 +62,13 @@ order_start_then_action(pcmk_resource_t *first_rsc, pcmk_action_t *then_action,
                         uint32_t extra)
 {
     if ((first_rsc != NULL) && (then_action != NULL)) {
+
         pcmk__new_ordering(first_rsc, start_key(first_rsc), NULL,
                            then_action->rsc, NULL, then_action,
                            pcmk__ar_guest_allowed
                            |pcmk__ar_unrunnable_first_blocks
                            |extra,
-                           first_rsc->cluster);
+                           first_rsc->priv->scheduler);
     }
 }
 
@@ -78,7 +79,8 @@ order_action_then_stop(pcmk_action_t *first_action, pcmk_resource_t *then_rsc,
     if ((first_action != NULL) && (then_rsc != NULL)) {
         pcmk__new_ordering(first_action->rsc, NULL, first_action,
                            then_rsc, stop_key(then_rsc), NULL,
-                           pcmk__ar_guest_allowed|extra, then_rsc->cluster);
+                           pcmk__ar_guest_allowed|extra,
+                           then_rsc->priv->scheduler);
     }
 }
 
@@ -88,10 +90,10 @@ get_remote_node_state(const pcmk_node_t *node)
     const pcmk_resource_t *remote_rsc = NULL;
     const pcmk_node_t *cluster_node = NULL;
 
-    CRM_ASSERT(node != NULL);
+    pcmk__assert(node != NULL);
 
-    remote_rsc = node->details->remote_rsc;
-    CRM_ASSERT(remote_rsc != NULL);
+    remote_rsc = node->priv->remote;
+    pcmk__assert(remote_rsc != NULL);
 
     cluster_node = pcmk__current_node(remote_rsc);
 
@@ -99,8 +101,8 @@ get_remote_node_state(const pcmk_node_t *node)
      * is unclean or went offline, we can't process any operations
      * on that remote node until after it starts elsewhere.
      */
-    if ((remote_rsc->next_role == pcmk_role_stopped)
-        || (remote_rsc->allocated_to == NULL)) {
+    if ((remote_rsc->priv->next_role == pcmk_role_stopped)
+        || (remote_rsc->priv->assigned_node == NULL)) {
 
         // The connection resource is not going to run anywhere
 
@@ -111,16 +113,16 @@ get_remote_node_state(const pcmk_node_t *node)
             return remote_state_failed;
         }
 
-        if (!pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)) {
+        if (!pcmk_is_set(remote_rsc->flags, pcmk__rsc_failed)) {
             /* Connection resource is cleanly stopped */
             return remote_state_stopped;
         }
 
         /* Connection resource is failed */
 
-        if ((remote_rsc->next_role == pcmk_role_stopped)
-            && remote_rsc->remote_reconnect_ms
-            && node->details->remote_was_fenced
+        if ((remote_rsc->priv->next_role == pcmk_role_stopped)
+            && (remote_rsc->priv->remote_reconnect_ms > 0U)
+            && pcmk_is_set(node->priv->flags, pcmk__node_remote_fenced)
             && !pe__shutdown_requested(node)) {
 
             /* We won't know whether the connection is recoverable until the
@@ -146,9 +148,9 @@ get_remote_node_state(const pcmk_node_t *node)
         // Connection is running on a dead node, see if we can recover it first
         return remote_state_resting;
 
-    } else if (pcmk__list_of_multiple(remote_rsc->running_on)
-               && (remote_rsc->partial_migration_source != NULL)
-               && (remote_rsc->partial_migration_target != NULL)) {
+    } else if (pcmk__list_of_multiple(remote_rsc->priv->active_nodes)
+               && (remote_rsc->priv->partial_migration_source != NULL)
+               && (remote_rsc->priv->partial_migration_target != NULL)) {
         /* We're in the middle of migrating a connection resource, so wait until
          * after the migration completes before performing any actions.
          */
@@ -168,7 +170,7 @@ static void
 apply_remote_ordering(pcmk_action_t *action)
 {
     pcmk_resource_t *remote_rsc = NULL;
-    enum action_tasks task = pcmk_parse_action(action->task);
+    enum pcmk__action_type task = pcmk__parse_action(action->task);
     enum remote_connection_state state = get_remote_node_state(action->node);
 
     uint32_t order_opts = pcmk__ar_none;
@@ -177,27 +179,27 @@ apply_remote_ordering(pcmk_action_t *action)
         return;
     }
 
-    CRM_ASSERT(pcmk__is_pacemaker_remote_node(action->node));
+    pcmk__assert(pcmk__is_pacemaker_remote_node(action->node));
 
-    remote_rsc = action->node->details->remote_rsc;
-    CRM_ASSERT(remote_rsc != NULL);
+    remote_rsc = action->node->priv->remote;
+    pcmk__assert(remote_rsc != NULL);
 
     crm_trace("Order %s action %s relative to %s%s (state: %s)",
               action->task, action->uuid,
-              pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)? "failed " : "",
+              pcmk_is_set(remote_rsc->flags, pcmk__rsc_failed)? "failed " : "",
               remote_rsc->id, state2text(state));
 
     if (pcmk__strcase_any_of(action->task, PCMK_ACTION_MIGRATE_TO,
                              PCMK_ACTION_MIGRATE_FROM, NULL)) {
-        /* Migration ops map to pcmk_action_unspecified, but we need to apply
+        /* Migration ops map to pcmk__action_unspecified, but we need to apply
          * the same ordering as for stop or demote (see get_router_node()).
          */
-        task = pcmk_action_stop;
+        task = pcmk__action_stop;
     }
 
     switch (task) {
-        case pcmk_action_start:
-        case pcmk_action_promote:
+        case pcmk__action_start:
+        case pcmk__action_promote:
             order_opts = pcmk__ar_none;
 
             if (state == remote_state_failed) {
@@ -210,7 +212,7 @@ apply_remote_ordering(pcmk_action_t *action)
             order_start_then_action(remote_rsc, action, order_opts);
             break;
 
-        case pcmk_action_stop:
+        case pcmk__action_stop:
             if (state == remote_state_alive) {
                 order_action_then_stop(action, remote_rsc,
                                        pcmk__ar_then_implies_first);
@@ -222,12 +224,12 @@ apply_remote_ordering(pcmk_action_t *action)
                  * to the remote connection, since the stop will become implied
                  * by the fencing.
                  */
-                pe_fence_node(remote_rsc->cluster, action->node,
+                pe_fence_node(remote_rsc->priv->scheduler, action->node,
                               "resources are active but "
                               "connection is unrecoverable",
                               FALSE);
 
-            } else if (remote_rsc->next_role == pcmk_role_stopped) {
+            } else if (remote_rsc->priv->next_role == pcmk_role_stopped) {
                 /* State must be remote_state_unknown or remote_state_stopped.
                  * Since the connection is not coming back up in this
                  * transition, stop this resource first.
@@ -243,7 +245,7 @@ apply_remote_ordering(pcmk_action_t *action)
             }
             break;
 
-        case pcmk_action_demote:
+        case pcmk__action_demote:
             /* Only order this demote relative to the connection start if the
              * connection isn't being torn down. Otherwise, the demote would be
              * blocked because the connection start would not be allowed.
@@ -268,12 +270,13 @@ apply_remote_ordering(pcmk_action_t *action)
             } else {
                 pcmk_node_t *cluster_node = pcmk__current_node(remote_rsc);
 
-                if ((task == pcmk_action_monitor) && (state == remote_state_failed)) {
+                if ((task == pcmk__action_monitor)
+                    && (state == remote_state_failed)) {
                     /* We would only be here if we do not know the state of the
                      * resource on the remote node. Since we have no way to find
                      * out, it is necessary to fence the node.
                      */
-                    pe_fence_node(remote_rsc->cluster, action->node,
+                    pe_fence_node(remote_rsc->priv->scheduler, action->node,
                                   "resources are in unknown state "
                                   "and connection is unrecoverable", FALSE);
                 }
@@ -295,66 +298,58 @@ apply_remote_ordering(pcmk_action_t *action)
 }
 
 static void
-apply_container_ordering(pcmk_action_t *action)
+apply_launcher_ordering(pcmk_action_t *action)
 {
-    /* VMs are also classified as containers for these purposes... in
-     * that they both involve a 'thing' running on a real or remote
-     * cluster node.
-     *
-     * This allows us to be smarter about the type and extent of
-     * recovery actions required in various scenarios
-     */
     pcmk_resource_t *remote_rsc = NULL;
-    pcmk_resource_t *container = NULL;
-    enum action_tasks task = pcmk_parse_action(action->task);
+    pcmk_resource_t *launcher = NULL;
+    enum pcmk__action_type task = pcmk__parse_action(action->task);
 
-    CRM_ASSERT(action->rsc != NULL);
-    CRM_ASSERT(action->node != NULL);
-    CRM_ASSERT(pcmk__is_pacemaker_remote_node(action->node));
+    pcmk__assert(action->rsc != NULL);
+    pcmk__assert(pcmk__is_pacemaker_remote_node(action->node));
 
-    remote_rsc = action->node->details->remote_rsc;
-    CRM_ASSERT(remote_rsc != NULL);
+    remote_rsc = action->node->priv->remote;
+    pcmk__assert(remote_rsc != NULL);
 
-    container = remote_rsc->container;
-    CRM_ASSERT(container != NULL);
+    launcher = remote_rsc->priv->launcher;
+    pcmk__assert(launcher != NULL);
 
-    if (pcmk_is_set(container->flags, pcmk_rsc_failed)) {
-        pe_fence_node(action->rsc->cluster, action->node, "container failed",
-                      FALSE);
+    if (pcmk_is_set(launcher->flags, pcmk__rsc_failed)) {
+        pe_fence_node(action->rsc->priv->scheduler, action->node,
+                      "container failed", FALSE);
     }
 
     crm_trace("Order %s action %s relative to %s%s for %s%s",
               action->task, action->uuid,
-              pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)? "failed " : "",
+              pcmk_is_set(remote_rsc->flags, pcmk__rsc_failed)? "failed " : "",
               remote_rsc->id,
-              pcmk_is_set(container->flags, pcmk_rsc_failed)? "failed " : "",
-              container->id);
+              pcmk_is_set(launcher->flags, pcmk__rsc_failed)? "failed " : "",
+              launcher->id);
 
     if (pcmk__strcase_any_of(action->task, PCMK_ACTION_MIGRATE_TO,
                              PCMK_ACTION_MIGRATE_FROM, NULL)) {
-        /* Migration ops map to pcmk_action_unspecified, but we need to apply
+        /* Migration ops map to pcmk__action_unspecified, but we need to apply
          * the same ordering as for stop or demote (see get_router_node()).
          */
-        task = pcmk_action_stop;
+        task = pcmk__action_stop;
     }
 
     switch (task) {
-        case pcmk_action_start:
-        case pcmk_action_promote:
-            // Force resource recovery if the container is recovered
-            order_start_then_action(container, action,
+        case pcmk__action_start:
+        case pcmk__action_promote:
+            // Force resource recovery if the launcher is recovered
+            order_start_then_action(launcher, action,
                                     pcmk__ar_first_implies_then);
 
             // Wait for the connection resource to be up, too
             order_start_then_action(remote_rsc, action, pcmk__ar_none);
             break;
 
-        case pcmk_action_stop:
-        case pcmk_action_demote:
-            if (pcmk_is_set(container->flags, pcmk_rsc_failed)) {
-                /* When the container representing a guest node fails, any stop
+        case pcmk__action_stop:
+        case pcmk__action_demote:
+            if (pcmk_is_set(launcher->flags, pcmk__rsc_failed)) {
+                /* When the launcher representing a guest node fails, any stop
                  * or demote actions for resources running on the guest node
-                 * are implied by the container stopping. This is similar to
+                 * are implied by the launcher stopping. This is similar to
                  * how fencing operations work for cluster nodes and remote
                  * nodes.
                  */
@@ -363,7 +358,7 @@ apply_container_ordering(pcmk_action_t *action)
                  * down.
                  *
                  * If we really wanted to, we could order these after the
-                 * connection start, IFF the container's current role was
+                 * connection start, IFF the launcher's current role was
                  * stopped (otherwise we re-introduce an ordering loop when the
                  * connection is restarting).
                  */
@@ -378,7 +373,7 @@ apply_container_ordering(pcmk_action_t *action)
                  * recurring monitors to be restarted, even if just
                  * the connection was re-established
                  */
-                if (task != pcmk_action_unspecified) {
+                if (task != pcmk__action_unspecified) {
                     order_start_then_action(remote_rsc, action,
                                             pcmk__ar_first_implies_then);
                 }
@@ -398,13 +393,14 @@ apply_container_ordering(pcmk_action_t *action)
 void
 pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
 {
-    if (!pcmk_is_set(scheduler->flags, pcmk_sched_have_remote_nodes)) {
+    if (!pcmk_is_set(scheduler->flags, pcmk__sched_have_remote_nodes)) {
         return;
     }
 
     crm_trace("Creating remote connection orderings");
 
-    for (GList *iter = scheduler->actions; iter != NULL; iter = iter->next) {
+    for (GList *iter = scheduler->priv->actions;
+         iter != NULL; iter = iter->next) {
         pcmk_action_t *action = iter->data;
         pcmk_resource_t *remote = NULL;
 
@@ -417,9 +413,9 @@ pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
          * remote connection resource, then make sure this happens before
          * any start of the resource in this transition.
          */
-        if (action->rsc->is_remote_node &&
-            pcmk__str_eq(action->task, PCMK_ACTION_CLEAR_FAILCOUNT,
-                         pcmk__str_none)) {
+        if (pcmk_is_set(action->rsc->flags, pcmk__rsc_is_remote_connection)
+            && pcmk__str_eq(action->task, PCMK_ACTION_CLEAR_FAILCOUNT,
+                            pcmk__str_none)) {
 
             pcmk__new_ordering(action->rsc, NULL, action, action->rsc,
                                pcmk__op_key(action->rsc->id, PCMK_ACTION_START,
@@ -444,11 +440,11 @@ pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
          * real actions and vice versa later in update_actions() at the end of
          * pcmk__apply_orderings().
          */
-        if (pcmk_is_set(action->flags, pcmk_action_pseudo)) {
+        if (pcmk_is_set(action->flags, pcmk__action_pseudo)) {
             continue;
         }
 
-        remote = action->node->details->remote_rsc;
+        remote = action->node->priv->remote;
         if (remote == NULL) {
             // Orphaned
             continue;
@@ -460,7 +456,7 @@ pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
          * start, we leave the resource running on the original node.
          */
         if (pcmk__str_eq(action->task, PCMK_ACTION_START, pcmk__str_none)) {
-            for (GList *item = action->rsc->actions; item != NULL;
+            for (GList *item = action->rsc->priv->actions; item != NULL;
                  item = item->next) {
                 pcmk_action_t *rsc_action = item->data;
 
@@ -483,9 +479,9 @@ pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
          * It would probably be better to add PCMK__XA_ROUTER_NODE as part of
          * this logic rather than create_graph_action().
          */
-        if (remote->container) {
+        if (remote->priv->launcher != NULL) {
             crm_trace("Container ordering for %s", action->uuid);
-            apply_container_ordering(action);
+            apply_launcher_ordering(action);
 
         } else {
             crm_trace("Remote ordering for %s", action->uuid);
@@ -505,7 +501,7 @@ pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
 bool
 pcmk__is_failed_remote_node(const pcmk_node_t *node)
 {
-    return pcmk__is_remote_node(node) && (node->details->remote_rsc != NULL)
+    return pcmk__is_remote_node(node) && (node->priv->remote != NULL)
            && (get_remote_node_state(node) == remote_state_failed);
 }
 
@@ -523,9 +519,9 @@ bool
 pcmk__rsc_corresponds_to_guest(const pcmk_resource_t *rsc,
                                const pcmk_node_t *node)
 {
-    return (rsc != NULL) && (rsc->fillers != NULL) && (node != NULL)
-            && (node->details->remote_rsc != NULL)
-            && (node->details->remote_rsc->container == rsc);
+    return (rsc != NULL) && (rsc->priv->launched != NULL) && (node != NULL)
+            && (node->priv->remote != NULL)
+            && (node->priv->remote->priv->launcher == rsc);
 }
 
 /*!
@@ -549,19 +545,20 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
     pcmk_node_t *ended_on = NULL;
     bool partial_migration = false;
     const char *task = action->task;
+    pcmk_resource_t *remote = NULL;
 
     if (pcmk__str_eq(task, PCMK_ACTION_STONITH, pcmk__str_none)
         || !pcmk__is_pacemaker_remote_node(action->node)) {
         return NULL;
     }
 
-    CRM_ASSERT(action->node->details->remote_rsc != NULL);
+    remote = action->node->priv->remote;
+    pcmk__assert(remote != NULL);
 
-    began_on = pcmk__current_node(action->node->details->remote_rsc);
-    ended_on = action->node->details->remote_rsc->allocated_to;
-    if (action->node->details->remote_rsc
-        && (action->node->details->remote_rsc->container == NULL)
-        && action->node->details->remote_rsc->partial_migration_target) {
+    began_on = pcmk__current_node(remote);
+    ended_on = remote->priv->assigned_node;
+    if ((remote->priv->launcher == NULL)
+        && (remote->priv->partial_migration_target != NULL)) {
         partial_migration = true;
     }
 
@@ -569,7 +566,7 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
         crm_trace("Routing %s for %s through remote connection's "
                   "next node %s (starting)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
-                  (ended_on? ended_on->details->uname : "none"),
+                  (ended_on? ended_on->priv->name : "none"),
                   partial_migration? " (partial migration)" : "");
         return ended_on;
     }
@@ -578,7 +575,7 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
         crm_trace("Routing %s for %s through remote connection's "
                   "current node %s (stopping)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
-                  (began_on? began_on->details->uname : "none"),
+                  (began_on? began_on->priv->name : "none"),
                   partial_migration? " (partial migration)" : "");
         return began_on;
     }
@@ -587,7 +584,7 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
         crm_trace("Routing %s for %s through remote connection's "
                   "current node %s (not moving)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
-                  (began_on? began_on->details->uname : "none"),
+                  (began_on? began_on->priv->name : "none"),
                   partial_migration? " (partial migration)" : "");
         return began_on;
     }
@@ -622,7 +619,7 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
         crm_trace("Routing %s for %s through remote connection's "
                   "current node %s (moving)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
-                  (began_on? began_on->details->uname : "none"),
+                  (began_on? began_on->priv->name : "none"),
                   partial_migration? " (partial migration)" : "");
         return began_on;
     }
@@ -634,7 +631,7 @@ pcmk__connection_host_for_action(const pcmk_action_t *action)
     crm_trace("Routing %s for %s through remote connection's "
               "next node %s (moving)%s",
               action->task, (action->rsc? action->rsc->id : "no resource"),
-              (ended_on? ended_on->details->uname : "none"),
+              (ended_on? ended_on->priv->name : "none"),
               partial_migration? " (partial migration)" : "");
     return ended_on;
 }
@@ -657,7 +654,7 @@ pcmk__substitute_remote_addr(pcmk_resource_t *rsc, GHashTable *params)
     const char *remote_addr = g_hash_table_lookup(params, PCMK_REMOTE_RA_ADDR);
 
     if (pcmk__str_eq(remote_addr, "#uname", pcmk__str_none)) {
-        GHashTable *base = pe_rsc_params(rsc, NULL, rsc->cluster);
+        GHashTable *base = pe_rsc_params(rsc, NULL, rsc->priv->scheduler);
 
         remote_addr = g_hash_table_lookup(base, PCMK_REMOTE_RA_ADDR);
         if (remote_addr != NULL) {
@@ -686,34 +683,36 @@ pcmk__add_guest_meta_to_xml(xmlNode *args_xml, const pcmk_action_t *action)
 {
     const pcmk_node_t *guest = action->node;
     const pcmk_node_t *host = NULL;
-    enum action_tasks task;
+    const pcmk_resource_t *launcher = NULL;
+    enum pcmk__action_type task;
 
     if (!pcmk__is_guest_or_bundle_node(guest)) {
         return;
     }
+    launcher = guest->priv->remote->priv->launcher;
 
-    task = pcmk_parse_action(action->task);
-    if ((task == pcmk_action_notify) || (task == pcmk_action_notified)) {
-        task = pcmk_parse_action(g_hash_table_lookup(action->meta,
-                                                     "notify_operation"));
+    task = pcmk__parse_action(action->task);
+    if ((task == pcmk__action_notify) || (task == pcmk__action_notified)) {
+        task = pcmk__parse_action(g_hash_table_lookup(action->meta,
+                                                      "notify_operation"));
     }
 
     switch (task) {
-        case pcmk_action_stop:
-        case pcmk_action_stopped:
-        case pcmk_action_demote:
-        case pcmk_action_demoted:
+        case pcmk__action_stop:
+        case pcmk__action_stopped:
+        case pcmk__action_demote:
+        case pcmk__action_demoted:
             // "Down" actions take place on guest's current host
-            host = pcmk__current_node(guest->details->remote_rsc->container);
+            host = pcmk__current_node(launcher);
             break;
 
-        case pcmk_action_start:
-        case pcmk_action_started:
-        case pcmk_action_monitor:
-        case pcmk_action_promote:
-        case pcmk_action_promoted:
+        case pcmk__action_start:
+        case pcmk__action_started:
+        case pcmk__action_monitor:
+        case pcmk__action_promote:
+        case pcmk__action_promoted:
             // "Up" actions take place on guest's next host
-            host = guest->details->remote_rsc->container->allocated_to;
+            host = launcher->priv->assigned_node;
             break;
 
         default:
@@ -722,14 +721,14 @@ pcmk__add_guest_meta_to_xml(xmlNode *args_xml, const pcmk_action_t *action)
 
     if (host != NULL) {
         gpointer target =
-            g_hash_table_lookup(action->rsc->meta,
+            g_hash_table_lookup(action->rsc->priv->meta,
                                 PCMK_META_CONTAINER_ATTRIBUTE_TARGET);
 
         hash2metafield((gpointer) PCMK_META_CONTAINER_ATTRIBUTE_TARGET,
                        target,
                        (gpointer) args_xml);
         hash2metafield((gpointer) PCMK__META_PHYSICAL_HOST,
-                       (gpointer) host->details->uname,
+                       (gpointer) host->priv->name,
                        (gpointer) args_xml);
     }
 }
