@@ -13,6 +13,7 @@
 #include <glib.h>
 
 #include <crm/crm.h>
+#include <crm/common/scheduler.h>
 #include <crm/common/scheduler_internal.h>
 #include <crm/pengine/status.h>
 #include <pacemaker-internal.h>
@@ -29,7 +30,7 @@ enum loss_ticket_policy {
 typedef struct {
     const char *id;
     pcmk_resource_t *rsc;
-    pcmk_ticket_t *ticket;
+    pcmk__ticket_t *ticket;
     enum loss_ticket_policy loss_policy;
     int role;
 } rsc_ticket_t;
@@ -47,7 +48,7 @@ static bool
 ticket_role_matches(const pcmk_resource_t *rsc, const rsc_ticket_t *rsc_ticket)
 {
     if ((rsc_ticket->role == pcmk_role_unknown)
-        || (rsc_ticket->role == rsc->role)) {
+        || (rsc_ticket->role == rsc->priv->orig_role)) {
         return true;
     }
     pcmk__rsc_trace(rsc, "Skipping constraint: \"%s\" state filter",
@@ -68,13 +69,14 @@ constraints_for_ticket(pcmk_resource_t *rsc, const rsc_ticket_t *rsc_ticket)
 
     CRM_CHECK((rsc != NULL) && (rsc_ticket != NULL), return);
 
-    if (rsc_ticket->ticket->granted && !rsc_ticket->ticket->standby) {
+    if (pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_granted)
+        && !pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_standby)) {
         return;
     }
 
-    if (rsc->children) {
+    if (rsc->priv->children != NULL) {
         pcmk__rsc_trace(rsc, "Processing ticket dependencies from %s", rsc->id);
-        for (iter = rsc->children; iter != NULL; iter = iter->next) {
+        for (iter = rsc->priv->children; iter != NULL; iter = iter->next) {
             constraints_for_ticket((pcmk_resource_t *) iter->data, rsc_ticket);
         }
         return;
@@ -84,19 +86,22 @@ constraints_for_ticket(pcmk_resource_t *rsc, const rsc_ticket_t *rsc_ticket)
                     rsc->id, rsc_ticket->ticket->id, rsc_ticket->id,
                     pcmk_role_text(rsc_ticket->role));
 
-    if (!rsc_ticket->ticket->granted && (rsc->running_on != NULL)) {
+    if (!pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_granted)
+        && (rsc->priv->active_nodes != NULL)) {
 
         switch (rsc_ticket->loss_policy) {
             case loss_ticket_stop:
                 resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                                  "__loss_of_ticket__", rsc->cluster);
+                                  "__loss_of_ticket__",
+                                  rsc->priv->scheduler);
                 break;
 
             case loss_ticket_demote:
                 // Promotion score will be set to -INFINITY in promotion_order()
                 if (rsc_ticket->role != pcmk_role_promoted) {
                     resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                                      "__loss_of_ticket__", rsc->cluster);
+                                      "__loss_of_ticket__",
+                                      rsc->priv->scheduler);
                 }
                 break;
 
@@ -106,10 +111,14 @@ constraints_for_ticket(pcmk_resource_t *rsc, const rsc_ticket_t *rsc_ticket)
                 }
 
                 resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                                  "__loss_of_ticket__", rsc->cluster);
+                                  "__loss_of_ticket__",
+                                  rsc->priv->scheduler);
 
-                for (iter = rsc->running_on; iter != NULL; iter = iter->next) {
-                    pe_fence_node(rsc->cluster, (pcmk_node_t *) iter->data,
+                for (iter = rsc->priv->active_nodes;
+                     iter != NULL; iter = iter->next) {
+
+                    pe_fence_node(rsc->priv->scheduler,
+                                  (pcmk_node_t *) iter->data,
                                   "deadman ticket was lost", FALSE);
                 }
                 break;
@@ -118,60 +127,57 @@ constraints_for_ticket(pcmk_resource_t *rsc, const rsc_ticket_t *rsc_ticket)
                 if (!ticket_role_matches(rsc, rsc_ticket)) {
                     return;
                 }
-                if (rsc->running_on != NULL) {
-                    pcmk__clear_rsc_flags(rsc, pcmk_rsc_managed);
-                    pcmk__set_rsc_flags(rsc, pcmk_rsc_blocked);
+                if (rsc->priv->active_nodes != NULL) {
+                    pcmk__clear_rsc_flags(rsc, pcmk__rsc_managed);
+                    pcmk__set_rsc_flags(rsc, pcmk__rsc_blocked);
                 }
                 break;
         }
 
-    } else if (!rsc_ticket->ticket->granted) {
+    } else if (!pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_granted)) {
 
         if ((rsc_ticket->role != pcmk_role_promoted)
             || (rsc_ticket->loss_policy == loss_ticket_stop)) {
             resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                              "__no_ticket__", rsc->cluster);
+                              "__no_ticket__", rsc->priv->scheduler);
         }
 
-    } else if (rsc_ticket->ticket->standby) {
+    } else if (pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_standby)) {
 
         if ((rsc_ticket->role != pcmk_role_promoted)
             || (rsc_ticket->loss_policy == loss_ticket_stop)) {
             resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                              "__ticket_standby__", rsc->cluster);
+                              "__ticket_standby__", rsc->priv->scheduler);
         }
     }
 }
 
 static void
-rsc_ticket_new(const char *id, pcmk_resource_t *rsc, pcmk_ticket_t *ticket,
-               const char *state, const char *loss_policy)
+rsc_ticket_new(const char *id, pcmk_resource_t *rsc, pcmk__ticket_t *ticket,
+               const char *role_spec, const char *loss_policy)
 {
     rsc_ticket_t *new_rsc_ticket = NULL;
+    enum rsc_role_e role = pcmk_role_unknown;
 
     if (rsc == NULL) {
         pcmk__config_err("Ignoring ticket '%s' because resource "
                          "does not exist", id);
         return;
     }
-
-    new_rsc_ticket = calloc(1, sizeof(rsc_ticket_t));
-    if (new_rsc_ticket == NULL) {
+    if (pcmk__parse_constraint_role(id, role_spec, &role) != pcmk_rc_ok) {
+        // Not possible with schema validation enabled (error already logged)
         return;
     }
 
-    if (pcmk__str_eq(state, PCMK_ROLE_STARTED,
-                     pcmk__str_null_matches|pcmk__str_casei)) {
-        state = PCMK__ROLE_UNKNOWN;
-    }
-
+    new_rsc_ticket = pcmk__assert_alloc(1, sizeof(rsc_ticket_t));
     new_rsc_ticket->id = id;
     new_rsc_ticket->ticket = ticket;
     new_rsc_ticket->rsc = rsc;
-    new_rsc_ticket->role = pcmk_parse_role(state);
+    new_rsc_ticket->role = role;
 
     if (pcmk__str_eq(loss_policy, PCMK_VALUE_FENCE, pcmk__str_casei)) {
-        if (pcmk_is_set(rsc->cluster->flags, pcmk_sched_fencing_enabled)) {
+        if (pcmk_is_set(rsc->priv->scheduler->flags,
+                        pcmk__sched_fencing_enabled)) {
             new_rsc_ticket->loss_policy = loss_ticket_fence;
         } else {
             pcmk__config_err("Resetting '" PCMK_XA_LOSS_POLICY "' "
@@ -222,19 +228,18 @@ rsc_ticket_new(const char *id, pcmk_resource_t *rsc, pcmk_ticket_t *ticket,
     pcmk__rsc_trace(rsc, "%s (%s) ==> %s",
                     rsc->id, pcmk_role_text(new_rsc_ticket->role), ticket->id);
 
-    rsc->rsc_tickets = g_list_append(rsc->rsc_tickets, new_rsc_ticket);
+    rsc->priv->ticket_constraints =
+        g_list_append(rsc->priv->ticket_constraints, new_rsc_ticket);
 
-    rsc->cluster->ticket_constraints = g_list_append(
-        rsc->cluster->ticket_constraints, new_rsc_ticket);
-
-    if (!(new_rsc_ticket->ticket->granted) || new_rsc_ticket->ticket->standby) {
+    if (!pcmk_is_set(new_rsc_ticket->ticket->flags, pcmk__ticket_granted)
+        || pcmk_is_set(new_rsc_ticket->ticket->flags, pcmk__ticket_standby)) {
         constraints_for_ticket(rsc, new_rsc_ticket);
     }
 }
 
 // \return Standard Pacemaker return code
 static int
-unpack_rsc_ticket_set(xmlNode *set, pcmk_ticket_t *ticket,
+unpack_rsc_ticket_set(xmlNode *set, pcmk__ticket_t *ticket,
                       const char *loss_policy, pcmk_scheduler_t *scheduler)
 {
     const char *set_id = NULL;
@@ -254,11 +259,12 @@ unpack_rsc_ticket_set(xmlNode *set, pcmk_ticket_t *ticket,
 
     for (xmlNode *xml_rsc = pcmk__xe_first_child(set, PCMK_XE_RESOURCE_REF,
                                                  NULL, NULL);
-         xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+         xml_rsc != NULL;
+         xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
         pcmk_resource_t *resource = NULL;
 
-        resource = pcmk__find_constraint_resource(scheduler->resources,
+        resource = pcmk__find_constraint_resource(scheduler->priv->resources,
                                                   pcmk__xe_id(xml_rsc));
         if (resource == NULL) {
             pcmk__config_err("%s: No resource found for %s",
@@ -280,22 +286,13 @@ unpack_simple_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     const char *ticket_str = crm_element_value(xml_obj, PCMK_XA_TICKET);
     const char *loss_policy = crm_element_value(xml_obj, PCMK_XA_LOSS_POLICY);
 
-    pcmk_ticket_t *ticket = NULL;
+    pcmk__ticket_t *ticket = NULL;
 
     const char *rsc_id = crm_element_value(xml_obj, PCMK_XA_RSC);
     const char *state = crm_element_value(xml_obj, PCMK_XA_RSC_ROLE);
 
-    // @COMPAT: Deprecated since 2.1.5
-    const char *instance = crm_element_value(xml_obj, PCMK__XA_RSC_INSTANCE);
-
     pcmk_resource_t *rsc = NULL;
 
-    if (instance != NULL) {
-        pcmk__warn_once(pcmk__wo_coloc_inst,
-                        "Support for " PCMK__XA_RSC_INSTANCE " is deprecated "
-                        "and will be removed in a future release");
-    }
-
     CRM_CHECK(xml_obj != NULL, return);
 
     id = pcmk__xe_id(xml_obj);
@@ -310,7 +307,8 @@ unpack_simple_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
                          id);
         return;
     } else {
-        ticket = g_hash_table_lookup(scheduler->tickets, ticket_str);
+        ticket = g_hash_table_lookup(scheduler->priv->ticket_constraints,
+                                     ticket_str);
     }
 
     if (ticket == NULL) {
@@ -323,29 +321,14 @@ unpack_simple_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
         pcmk__config_err("Ignoring constraint '%s' without resource", id);
         return;
     } else {
-        rsc = pcmk__find_constraint_resource(scheduler->resources, rsc_id);
+        rsc = pcmk__find_constraint_resource(scheduler->priv->resources,
+                                             rsc_id);
     }
 
     if (rsc == NULL) {
         pcmk__config_err("Ignoring constraint '%s' because resource '%s' "
                          "does not exist", id, rsc_id);
         return;
-
-    } else if ((instance != NULL) && !pcmk__is_clone(rsc)) {
-        pcmk__config_err("Ignoring constraint '%s' because resource '%s' "
-                         "is not a clone but instance '%s' was requested",
-                         id, rsc_id, instance);
-        return;
-    }
-
-    if (instance != NULL) {
-        rsc = find_clone_instance(rsc, instance);
-        if (rsc == NULL) {
-            pcmk__config_warn("Ignoring constraint '%s' because resource '%s' "
-                              "does not have an instance '%s'",
-                              id, rsc_id, instance);
-            return;
-        }
     }
 
     rsc_ticket_new(id, rsc, ticket, state, loss_policy);
@@ -361,7 +344,7 @@ unpack_rsc_ticket_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
     const char *state = NULL;
 
     pcmk_resource_t *rsc = NULL;
-    pcmk_tag_t *tag = NULL;
+    pcmk__idref_t *tag = NULL;
 
     xmlNode *rsc_set = NULL;
 
@@ -407,7 +390,7 @@ unpack_rsc_ticket_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
      */
     if (!pcmk__tag_to_set(*expanded_xml, &rsc_set, PCMK_XA_RSC, false,
                           scheduler)) {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
         return pcmk_rc_unpack_error;
     }
@@ -422,7 +405,7 @@ unpack_rsc_ticket_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
         }
 
     } else {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
     }
 
@@ -438,7 +421,7 @@ pcmk__unpack_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     const char *id = NULL;
     const char *ticket_str = NULL;
 
-    pcmk_ticket_t *ticket = NULL;
+    pcmk__ticket_t *ticket = NULL;
 
     xmlNode *orig_xml = NULL;
     xmlNode *expanded_xml = NULL;
@@ -452,8 +435,9 @@ pcmk__unpack_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
         return;
     }
 
-    if (scheduler->tickets == NULL) {
-        scheduler->tickets = pcmk__strkey_table(free, destroy_ticket);
+    if (scheduler->priv->ticket_constraints == NULL) {
+        scheduler->priv->ticket_constraints =
+            pcmk__strkey_table(free, destroy_ticket);
     }
 
     ticket_str = crm_element_value(xml_obj, PCMK_XA_TICKET);
@@ -461,7 +445,8 @@ pcmk__unpack_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
         pcmk__config_err("Ignoring constraint '%s' without ticket", id);
         return;
     } else {
-        ticket = g_hash_table_lookup(scheduler->tickets, ticket_str);
+        ticket = g_hash_table_lookup(scheduler->priv->ticket_constraints,
+                                     ticket_str);
     }
 
     if (ticket == NULL) {
@@ -481,26 +466,26 @@ pcmk__unpack_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     }
 
     for (set = pcmk__xe_first_child(xml_obj, PCMK_XE_RESOURCE_SET, NULL, NULL);
-         set != NULL; set = pcmk__xe_next_same(set)) {
+         set != NULL; set = pcmk__xe_next(set, PCMK_XE_RESOURCE_SET)) {
 
         const char *loss_policy = NULL;
 
         any_sets = true;
-        set = expand_idref(set, scheduler->input);
+        set = pcmk__xe_resolve_idref(set, scheduler->input);
         loss_policy = crm_element_value(xml_obj, PCMK_XA_LOSS_POLICY);
 
         if ((set == NULL) // Configuration error, message already logged
             || (unpack_rsc_ticket_set(set, ticket, loss_policy,
                                       scheduler) != pcmk_rc_ok)) {
             if (expanded_xml != NULL) {
-                free_xml(expanded_xml);
+                pcmk__xml_free(expanded_xml);
             }
             return;
         }
     }
 
     if (expanded_xml) {
-        free_xml(expanded_xml);
+        pcmk__xml_free(expanded_xml);
         xml_obj = orig_xml;
     }
 
@@ -521,13 +506,18 @@ pcmk__unpack_rsc_ticket(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
 void
 pcmk__require_promotion_tickets(pcmk_resource_t *rsc)
 {
-    for (GList *item = rsc->rsc_tickets; item != NULL; item = item->next) {
+    for (GList *item = rsc->priv->ticket_constraints;
+         item != NULL; item = item->next) {
+
         rsc_ticket_t *rsc_ticket = (rsc_ticket_t *) item->data;
 
         if ((rsc_ticket->role == pcmk_role_promoted)
-            && (!rsc_ticket->ticket->granted || rsc_ticket->ticket->standby)) {
+            && (!pcmk_is_set(rsc_ticket->ticket->flags, pcmk__ticket_granted)
+                || pcmk_is_set(rsc_ticket->ticket->flags,
+                               pcmk__ticket_standby))) {
             resource_location(rsc, NULL, -PCMK_SCORE_INFINITY,
-                              "__stateful_without_ticket__", rsc->cluster);
+                              "__stateful_without_ticket__",
+                              rsc->priv->scheduler);
         }
     }
 }
