@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 the Pacemaker project contributors
+ * Copyright 2024-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -103,7 +103,7 @@ pcmk__setup_output_fencing(pcmk__output_t **out, stonith_t **st, xmlNode **xml)
         return rc;
     }
 
-    *st = stonith_api_new();
+    *st = stonith__api_new();
     if (*st == NULL) {
         return ENOMEM;
     }
@@ -111,7 +111,7 @@ pcmk__setup_output_fencing(pcmk__output_t **out, stonith_t **st, xmlNode **xml)
     rc = (*st)->cmds->connect(*st, crm_system_name, NULL);
     if (rc < 0) {
         rc = pcmk_legacy2rc(rc);
-        stonith_api_delete(*st);
+        stonith__api_free(*st);
         return rc;
     }
 
