@@ -82,6 +82,8 @@ pcmk__get_ticket_state(cib_t *cib, const char *ticket_id, xmlNode **state)
     char *xpath = NULL;
 
     pcmk__assert((cib != NULL) && (state != NULL));
+    // cppcheck doesn't understand the above pcmk__assert line
+    // cppcheck-suppress ctunullpointer
     *state = NULL;
 
     if (ticket_id != NULL) {
@@ -253,7 +255,7 @@ done:
     }
 
     pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
-    pe_free_working_set(scheduler);
+    pcmk_free_scheduler(scheduler);
     return rc;
 }
 
@@ -307,7 +309,7 @@ pcmk_ticket_get_attr(xmlNodePtr *xml, const char *ticket_id,
 
 done:
     pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
-    pe_free_working_set(scheduler);
+    pcmk_free_scheduler(scheduler);
     return rc;
 }
 
@@ -367,7 +369,7 @@ pcmk_ticket_info(xmlNodePtr *xml, const char *ticket_id)
 
 done:
     pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
-    pe_free_working_set(scheduler);
+    pcmk_free_scheduler(scheduler);
     return rc;
 }
 
@@ -439,7 +441,7 @@ done:
     }
 
     pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
-    pe_free_working_set(scheduler);
+    pcmk_free_scheduler(scheduler);
     return rc;
 }
 
@@ -508,7 +510,7 @@ done:
     }
 
     pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
-    pe_free_working_set(scheduler);
+    pcmk_free_scheduler(scheduler);
     return rc;
 }
 
