@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 the Pacemaker project contributors
+ * Copyright 2024-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,8 @@
 #include <crm/common/xml.h>
 #include <pacemaker.h>
 
+#include <libxml/xpath.h>                   // xmlXPathObject, etc.
+
 #include <pacemaker-internal.h>
 
 static char *cib_path = NULL;
@@ -94,10 +96,12 @@ ticket_exists(void **state)
     /* Verify that the XML result has only one <ticket>, and that its ID is
      * what we asked for.
      */
-    xpath_obj = xpath_search(xml, "//" PCMK__XE_TICKET_STATE "[@" PCMK_XA_ID "=\"ticketA\"]");
-    assert_int_equal(numXpathResults(xpath_obj), 1);
+    xpath_obj = pcmk__xpath_search(xml->doc,
+                                   "//" PCMK__XE_TICKET_STATE
+                                   "[@" PCMK_XA_ID "=\"ticketA\"]");
+    assert_int_equal(pcmk__xpath_num_results(xpath_obj), 1);
 
-    freeXpathObject(xpath_obj);
+    xmlXPathFreeObject(xpath_obj);
     pcmk__xml_free(xml);
     cib__clean_up_connection(&cib);
 }
@@ -117,23 +121,27 @@ multiple_tickets(void **state)
     /* Verify that the XML result has four <ticket> elements, and that their
      * IDs are as expected.
      */
-    xpath_obj = xpath_search(xml, "//" PCMK__XE_TICKET_STATE);
+    xpath_obj = pcmk__xpath_search(xml->doc, "//" PCMK__XE_TICKET_STATE);
 
-    assert_int_equal(numXpathResults(xpath_obj), 4);
+    assert_int_equal(pcmk__xpath_num_results(xpath_obj), 4);
 
-    ticket_node = getXpathResult(xpath_obj, 0);
+    ticket_node = pcmk__xpath_result(xpath_obj, 0);
+    assert_non_null(ticket_node);
     assert_string_equal(crm_element_value(ticket_node, PCMK_XA_ID), "ticketA");
 
-    ticket_node = getXpathResult(xpath_obj, 1);
+    ticket_node = pcmk__xpath_result(xpath_obj, 1);
+    assert_non_null(ticket_node);
     assert_string_equal(crm_element_value(ticket_node, PCMK_XA_ID), "ticketB");
 
-    ticket_node = getXpathResult(xpath_obj, 2);
+    ticket_node = pcmk__xpath_result(xpath_obj, 2);
+    assert_non_null(ticket_node);
     assert_string_equal(crm_element_value(ticket_node, PCMK_XA_ID), "ticketC");
 
-    ticket_node = getXpathResult(xpath_obj, 3);
+    ticket_node = pcmk__xpath_result(xpath_obj, 3);
+    assert_non_null(ticket_node);
     assert_string_equal(crm_element_value(ticket_node, PCMK_XA_ID), "ticketC");
 
-    freeXpathObject(xpath_obj);
+    xmlXPathFreeObject(xpath_obj);
     pcmk__xml_free(xml);
     cib__clean_up_connection(&cib);
 }
@@ -152,10 +160,12 @@ duplicate_tickets(void **state)
     /* Verify that the XML result has two <ticket> elements, and that their
      * IDs are as expected.
      */
-    xpath_obj = xpath_search(xml, "//" PCMK__XE_TICKET_STATE "[@" PCMK_XA_ID "=\"ticketC\"]");
+    xpath_obj = pcmk__xpath_search(xml->doc,
+                                   "//" PCMK__XE_TICKET_STATE
+                                   "[@" PCMK_XA_ID "=\"ticketC\"]");
 
-    assert_int_equal(numXpathResults(xpath_obj), 2);
-    freeXpathObject(xpath_obj);
+    assert_int_equal(pcmk__xpath_num_results(xpath_obj), 2);
+    xmlXPathFreeObject(xpath_obj);
     pcmk__xml_free(xml);
     cib__clean_up_connection(&cib);
 }
