@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2024 the Pacemaker project contributors
+ * Copyright 2004-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,7 +9,8 @@
 
 #include <crm_internal.h>
 
-#include <crm/pengine/rules.h>
+#include <libxml/tree.h>                    // xmlNode
+
 #include <crm/pengine/internal.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
@@ -27,7 +28,6 @@ static pcmk__rsc_methods_t resource_class_functions[] = {
     {
          native_unpack,
          native_find_rsc,
-         native_parameter,
          native_active,
          native_resource_state,
          native_location,
@@ -40,7 +40,6 @@ static pcmk__rsc_methods_t resource_class_functions[] = {
     {
          group_unpack,
          native_find_rsc,
-         native_parameter,
          group_active,
          group_resource_state,
          native_location,
@@ -53,7 +52,6 @@ static pcmk__rsc_methods_t resource_class_functions[] = {
     {
          clone_unpack,
          native_find_rsc,
-         native_parameter,
          clone_active,
          clone_resource_state,
          native_location,
@@ -66,7 +64,6 @@ static pcmk__rsc_methods_t resource_class_functions[] = {
     {
          pe__unpack_bundle,
          native_find_rsc,
-         native_parameter,
          pe__bundle_active,
          pe__bundle_resource_state,
          native_location,
@@ -127,7 +124,7 @@ dup_attr(gpointer key, gpointer value, gpointer user_data)
 
 static void
 expand_parents_fixed_nvpairs(pcmk_resource_t *rsc,
-                             pe_rule_eval_data_t *rule_data,
+                             const pcmk_rule_input_t *rule_input,
                              GHashTable *meta_hash, pcmk_scheduler_t *scheduler)
 {
     GHashTable *parent_orig_meta = pcmk__strkey_table(free, free);
@@ -145,7 +142,7 @@ expand_parents_fixed_nvpairs(pcmk_resource_t *rsc,
     while(p != NULL) {
         /* A hash table for comparison is generated, including the id-ref. */
         pe__unpack_dataset_nvpairs(p->priv->xml, PCMK_XE_META_ATTRIBUTES,
-                                   rule_data, parent_orig_meta, NULL,
+                                   rule_input, parent_orig_meta, NULL,
                                    scheduler);
         p = p->priv->parent;
     }
@@ -175,18 +172,11 @@ void
 get_meta_attributes(GHashTable * meta_hash, pcmk_resource_t * rsc,
                     pcmk_node_t *node, pcmk_scheduler_t *scheduler)
 {
-    pe_rsc_eval_data_t rsc_rule_data = {
-        .standard = crm_element_value(rsc->priv->xml, PCMK_XA_CLASS),
-        .provider = crm_element_value(rsc->priv->xml, PCMK_XA_PROVIDER),
-        .agent = crm_element_value(rsc->priv->xml, PCMK_XA_TYPE)
-    };
-
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
+    const pcmk_rule_input_t rule_input = {
         .now = scheduler->priv->now,
-        .match_data = NULL,
-        .rsc_data = &rsc_rule_data,
-        .op_data = NULL
+        .rsc_standard = crm_element_value(rsc->priv->xml, PCMK_XA_CLASS),
+        .rsc_provider = crm_element_value(rsc->priv->xml, PCMK_XA_PROVIDER),
+        .rsc_agent = crm_element_value(rsc->priv->xml, PCMK_XA_TYPE)
     };
 
     for (xmlAttrPtr a = pcmk__xe_first_attr(rsc->priv->xml);
@@ -199,19 +189,19 @@ get_meta_attributes(GHashTable * meta_hash, pcmk_resource_t * rsc,
     }
 
     pe__unpack_dataset_nvpairs(rsc->priv->xml, PCMK_XE_META_ATTRIBUTES,
-                               &rule_data, meta_hash, NULL, scheduler);
+                               &rule_input, meta_hash, NULL, scheduler);
 
     /* Set the PCMK_XE_META_ATTRIBUTES explicitly set in the parent resource to
      * the hash table of the child resource. If it is already explicitly set as
      * a child, it will not be overwritten.
      */
     if (rsc->priv->parent != NULL) {
-        expand_parents_fixed_nvpairs(rsc, &rule_data, meta_hash, scheduler);
+        expand_parents_fixed_nvpairs(rsc, &rule_input, meta_hash, scheduler);
     }
 
     /* check the defaults */
     pe__unpack_dataset_nvpairs(scheduler->priv->rsc_defaults,
-                               PCMK_XE_META_ATTRIBUTES, &rule_data, meta_hash,
+                               PCMK_XE_META_ATTRIBUTES, &rule_input, meta_hash,
                                NULL, scheduler);
 
     /* If there is PCMK_XE_META_ATTRIBUTES that the parent resource has not
@@ -236,25 +226,21 @@ void
 get_rsc_attributes(GHashTable *instance_attrs, const pcmk_resource_t *rsc,
                    const pcmk_node_t *node, pcmk_scheduler_t *scheduler)
 {
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
+    pcmk_rule_input_t rule_input = {
         .now = NULL,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
     };
 
     CRM_CHECK((instance_attrs != NULL) && (rsc != NULL) && (scheduler != NULL),
               return);
 
-    rule_data.now = scheduler->priv->now;
+    rule_input.now = scheduler->priv->now;
     if (node != NULL) {
-        rule_data.node_hash = node->priv->attrs;
+        rule_input.node_attrs = node->priv->attrs;
     }
 
     // Evaluate resource's own values, then its ancestors' values
     pe__unpack_dataset_nvpairs(rsc->priv->xml, PCMK_XE_INSTANCE_ATTRIBUTES,
-                               &rule_data, instance_attrs, NULL, scheduler);
+                               &rule_input, instance_attrs, NULL, scheduler);
     if (rsc->priv->parent != NULL) {
         get_rsc_attributes(instance_attrs, rsc->priv->parent, node, scheduler);
     }
@@ -312,8 +298,8 @@ unpack_template(xmlNode *xml_obj, xmlNode **expanded_xml,
         return FALSE;
     }
 
-    cib_resources = get_xpath_object("//" PCMK_XE_RESOURCES, scheduler->input,
-                                     LOG_TRACE);
+    cib_resources = pcmk__xpath_find_one(scheduler->input->doc,
+                                         "//" PCMK_XE_RESOURCES, LOG_TRACE);
     if (cib_resources == NULL) {
         pcmk__config_err("No resources configured");
         return FALSE;
@@ -708,12 +694,8 @@ pe__unpack_resource(xmlNode *xml_obj, pcmk_resource_t **rsc,
     bool remote_node = false;
     pcmk__resource_private_t *rsc_private = NULL;
 
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
+    pcmk_rule_input_t rule_input = {
         .now = NULL,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
     };
 
     CRM_CHECK(rsc != NULL, return EINVAL);
@@ -721,7 +703,7 @@ pe__unpack_resource(xmlNode *xml_obj, pcmk_resource_t **rsc,
               *rsc = NULL;
               return EINVAL);
 
-    rule_data.now = scheduler->priv->now;
+    rule_input.now = scheduler->priv->now;
 
     crm_log_xml_trace(xml_obj, "[raw XML]");
 
@@ -962,8 +944,8 @@ pe__unpack_resource(xmlNode *xml_obj, pcmk_resource_t **rsc,
                     (rsc_private->next_role == pcmk_role_unknown)?
                         "default" : pcmk_role_text(rsc_private->next_role));
 
-    if (rsc_private->fns->unpack(*rsc, scheduler) == FALSE) {
-        rsc_private->fns->free(*rsc);
+    if (!rsc_private->fns->unpack(*rsc)) {
+        pcmk__free_resource(*rsc);
         *rsc = NULL;
         return pcmk_rc_unpack_error;
     }
@@ -983,12 +965,12 @@ pe__unpack_resource(xmlNode *xml_obj, pcmk_resource_t **rsc,
                     pcmk_is_set((*rsc)->flags, pcmk__rsc_notify)? "required" : "not required");
 
     pe__unpack_dataset_nvpairs(rsc_private->xml, PCMK_XE_UTILIZATION,
-                               &rule_data, rsc_private->utilization, NULL,
+                               &rule_input, rsc_private->utilization, NULL,
                                scheduler);
 
     if (expanded_xml) {
         if (add_template_rsc(xml_obj, scheduler) == FALSE) {
-            rsc_private->fns->free(*rsc);
+            pcmk__free_resource(*rsc);
             *rsc = NULL;
             return pcmk_rc_unpack_error;
         }
