@@ -1,25 +1,31 @@
 /*
- * Copyright (C) 2014-2016 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2014-2024 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK_DBUS__H
-#  define PCMK_DBUS__H
+#ifndef PCMK__SERVICES_PCMK_DBUS__H
+#define PCMK__SERVICES_PCMK_DBUS__H
 
-#  include <dbus/dbus.h>
+#include <stdbool.h>    // bool
 
-#  ifndef DBUS_TIMEOUT_USE_DEFAULT
-#    define DBUS_TIMEOUT_USE_DEFAULT -1
-#  endif
+#include <dbus/dbus.h>  // DBUS_TIMEOUT_USE_DEFAULT, DBusConnection, etc.
+#include <glib.h>       // G_GNUC_INTERNAL, gchar
+
+#ifndef DBUS_TIMEOUT_USE_DEFAULT
+#define DBUS_TIMEOUT_USE_DEFAULT -1
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 G_GNUC_INTERNAL
 DBusConnection *pcmk_dbus_connect(void);
 
-G_GNUC_INTERNAL
-void pcmk_dbus_connection_setup_with_select(DBusConnection *c);
-
 G_GNUC_INTERNAL
 void pcmk_dbus_disconnect(DBusConnection *connection);
 
@@ -40,7 +46,11 @@ char *pcmk_dbus_get_property(
     DBusPendingCall **pending, int timeout);
 
 G_GNUC_INTERNAL
-bool pcmk_dbus_find_error(DBusPendingCall *pending, DBusMessage *reply,
+bool pcmk_dbus_find_error(const DBusPendingCall *pending, DBusMessage *reply,
                           DBusError *error);
 
-#endif  /* PCMK_DBUS__H */
+#ifdef __cplusplus
+}
+#endif
+
+#endif // PCMK__SERVICES_PCMK_DBUS__H
