@@ -1,6 +1,6 @@
 /*
  * Copyright 2010-2011 Red Hat, Inc.
- * Later changes copyright 2012-2024 the Pacemaker project contributors
+ * Later changes copyright 2012-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -49,6 +49,10 @@ struct svc_action_private_s {
     DBusPendingCall* pending;
     unsigned timerid;
 #endif
+
+#if SUPPORT_SYSTEMD
+    char *job_path;         // D-Bus object path for enqueued start/stop job
+#endif  // SUPPORT_SYSTEMD
 };
 
 G_GNUC_INTERNAL
