@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2024 the Pacemaker project contributors
+ * Copyright 2012-2025 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,10 @@
 #include <crm/services_internal.h>
 #include <crm/common/mainloop.h>
 
+#include <inttypes.h>               // PRIu32
+#include <stdbool.h>
+#include <stdint.h>                 // uint32_t
+#include <stdio.h>                  // fopen(), NULL, etc.
 #include <sys/stat.h>
 #include <gio/gio.h>
 #include <services_private.h>
@@ -23,6 +27,9 @@
 
 static void invoke_unit_by_path(svc_action_t *op, const char *unit);
 
+/* Systemd D-Bus interface
+ * https://www.freedesktop.org/software/systemd/man/latest/org.freedesktop.systemd1.html
+ */
 #define BUS_NAME         "org.freedesktop.systemd1"
 #define BUS_NAME_MANAGER BUS_NAME ".Manager"
 #define BUS_NAME_UNIT    BUS_NAME ".Unit"
@@ -103,13 +110,15 @@ systemd_send_recv(DBusMessage *msg, DBusError *error, int timeout)
 static DBusMessage *
 systemd_call_simple_method(const char *method)
 {
-    DBusMessage *msg = systemd_new_method(method);
+    DBusMessage *msg = NULL;
     DBusMessage *reply = NULL;
     DBusError error;
 
     /* Don't call systemd_init() here, because that calls this */
     CRM_CHECK(systemd_proxy, return NULL);
 
+    msg = systemd_new_method(method);
+
     if (msg == NULL) {
         crm_err("Could not create message to send %s to systemd", method);
         return NULL;
@@ -133,7 +142,51 @@ systemd_call_simple_method(const char *method)
     return reply;
 }
 
-static gboolean
+/*!
+ * \internal
+ * \brief Subscribe to D-Bus signals from systemd
+ *
+ * Systemd does not broadcast signal messages unless at least one client has
+ * called the \c Subscribe() method. Also, a D-Bus client ignores broadcast
+ * messages unless an appropriate match rule is set, so we set one here.
+ *
+ * \return Standard Pacemaker return code
+ */
+static int
+subscribe_to_signals(void)
+{
+    const char *match_rule = "type='signal',"
+                             "sender='" BUS_NAME "',"
+                             "interface='" BUS_NAME_MANAGER "',"
+                             "path='" BUS_PATH "'";
+    DBusMessage *reply = NULL;
+    DBusError error;
+
+    /* Tell D-Bus to accept signal messages from systemd.
+     * https://dbus.freedesktop.org/doc/dbus-specification.html#message-bus-routing-match-rules
+     */
+    dbus_error_init(&error);
+    dbus_bus_add_match(systemd_proxy, match_rule, &error);
+
+    if (dbus_error_is_set(&error)) {
+        crm_err("Could not listen for systemd DBus signals: %s " QB_XS " (%s)",
+                error.message, error.name);
+        dbus_error_free(&error);
+        return ECOMM;
+    }
+
+    // Tell systemd to broadcast signals
+    reply = systemd_call_simple_method("Subscribe");
+    if (reply == NULL) {
+        dbus_bus_remove_match(systemd_proxy, match_rule, &error);
+        return ECOMM;
+    }
+
+    dbus_message_unref(reply);
+    return pcmk_rc_ok;
+}
+
+static bool
 systemd_init(void)
 {
     static int need_init = 1;
@@ -150,11 +203,14 @@ systemd_init(void)
     if (need_init) {
         need_init = 0;
         systemd_proxy = pcmk_dbus_connect();
+
+        if (subscribe_to_signals() != pcmk_rc_ok) {
+            pcmk_dbus_disconnect(systemd_proxy);
+            systemd_proxy = NULL;
+        }
     }
-    if (systemd_proxy == NULL) {
-        return FALSE;
-    }
-    return TRUE;
+
+    return (systemd_proxy != NULL);
 }
 
 static inline char *
@@ -207,7 +263,7 @@ systemd_unit_extension(const char *name)
 }
 
 static char *
-systemd_service_name(const char *name, bool add_instance_name)
+systemd_unit_name(const char *name, bool add_instance_name)
 {
     const char *dot = NULL;
 
@@ -228,16 +284,10 @@ systemd_service_name(const char *name, bool add_instance_name)
 
     if (dot) {
         if (dot != name && *(dot-1) == '@') {
-            char *s = NULL;
-
-            if (asprintf(&s, "%.*spacemaker%s", (int) (dot-name), name, dot) == -1) {
-                /* If asprintf fails, just return name. */
-                return strdup(name);
-            }
-
-            return s;
+            return crm_strdup_printf("%.*spacemaker%s",
+                                     (int) (dot - name), name, dot);
         } else {
-            return strdup(name);
+            return pcmk__str_copy(name);
         }
 
     } else if (add_instance_name && *(name+strlen(name)-1) == '@') {
@@ -320,6 +370,19 @@ set_result_from_method_error(svc_action_t *op, const DBusError *error)
         services__format_result(op, PCMK_OCF_NOT_INSTALLED,
                                PCMK_EXEC_NOT_INSTALLED,
                                "systemd unit %s not found", op->agent);
+
+    /* If systemd happens to be re-executing by `systemctl daemon-reexec` at the
+     * same time, dbus gives an error with the name
+     * `org.freedesktop.DBus.Error.NoReply` and the message "Message recipient
+     * disconnected from message bus without replying".
+     * Consider the monitor pending rather than return an error yet, so that it
+     * can retry with another iteration.
+     */
+    } else if (pcmk__str_any_of(op->action, PCMK_ACTION_MONITOR,
+                                PCMK_ACTION_STATUS, NULL)
+               && strstr(error->name, DBUS_ERROR_NO_REPLY)
+               && strstr(error->message, "disconnected")) {
+        services__set_result(op, PCMK_OCF_UNKNOWN, PCMK_EXEC_PENDING, NULL);
     }
 
     crm_info("DBus request for %s of systemd unit %s%s%s failed: %s",
@@ -376,9 +439,13 @@ execute_after_loadunit(DBusMessage *reply, svc_action_t *op)
             invoke_unit_by_path(op, path);
 
         } else if (!(op->synchronous)) {
-            services__format_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
-                                    "No DBus object found for systemd unit %s",
-                                    op->agent);
+            if (!pcmk__str_any_of(op->action, PCMK_ACTION_MONITOR,
+                                  PCMK_ACTION_STATUS, NULL)
+                || op->status != PCMK_EXEC_PENDING) {
+                services__format_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
+                                        "No DBus object found for systemd unit %s",
+                                        op->agent);
+            }
             services__finalize_async_op(op);
         }
     }
@@ -440,6 +507,10 @@ invoke_unit_by_name(const char *arg_name, svc_action_t *op, char **path)
     DBusPendingCall *pending = NULL;
     char *name = NULL;
 
+    if (pcmk__str_empty(arg_name)) {
+        return EINVAL;
+    }
+
     if (!systemd_init()) {
         if (op != NULL) {
             services__set_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
@@ -460,11 +531,10 @@ invoke_unit_by_name(const char *arg_name, svc_action_t *op, char **path)
     pcmk__assert(msg != NULL);
 
     // Add the (expanded) unit name as the argument
-    name = systemd_service_name(arg_name,
-                                (op == NULL)
-                                || pcmk__str_eq(op->action,
-                                                PCMK_ACTION_META_DATA,
-                                                pcmk__str_none));
+    name = systemd_unit_name(arg_name,
+                             (op == NULL)
+                             || pcmk__str_eq(op->action, PCMK_ACTION_META_DATA,
+                                             pcmk__str_none));
     CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &name,
                                             DBUS_TYPE_INVALID));
     free(name);
@@ -548,7 +618,7 @@ systemd_unit_listall(void)
     DBusMessageIter elem;
     DBusMessage *reply = NULL;
 
-    if (systemd_init() == FALSE) {
+    if (!systemd_init()) {
         return NULL;
     }
 
@@ -635,18 +705,19 @@ systemd_unit_listall(void)
     return units;
 }
 
-gboolean
+bool
 systemd_unit_exists(const char *name)
 {
     char *path = NULL;
     char *state = NULL;
+    int rc = false;
 
     /* Note: Makes a blocking dbus calls
      * Used by resources_find_service_class() when resource class=service
      */
     if ((invoke_unit_by_name(name, NULL, &path) != pcmk_rc_ok)
         || (path == NULL)) {
-        return FALSE;
+        goto done;
     }
 
     /* A successful LoadUnit is not sufficient to determine the unit's
@@ -655,13 +726,12 @@ systemd_unit_exists(const char *name)
      */
     state = systemd_get_property(path, "LoadState", NULL, NULL, NULL,
                                  DBUS_TIMEOUT_USE_DEFAULT);
+    rc = pcmk__str_any_of(state, "loaded", "masked", NULL);
+
+done:
     free(path);
-    if (pcmk__str_any_of(state, "loaded", "masked", NULL)) {
-        free(state);
-        return TRUE;
-    }
     free(state);
-    return FALSE;
+    return rc;
 }
 
 // @TODO Use XML string constants and maybe a real XML object
@@ -734,6 +804,8 @@ systemd_unit_metadata(const char *name, int timeout)
 static void
 process_unit_method_reply(DBusMessage *reply, svc_action_t *op)
 {
+    bool start_stop = pcmk__strcase_any_of(op->action, PCMK_ACTION_START,
+                                           PCMK_ACTION_STOP, NULL);
     DBusError error;
 
     dbus_error_init(&error);
@@ -747,11 +819,25 @@ process_unit_method_reply(DBusMessage *reply, svc_action_t *op)
 
     } else if (!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH,
                                      __func__, __LINE__)) {
+        const char *reason = "systemd D-Bus method had unexpected reply";
+
         crm_info("DBus request for %s of %s succeeded but "
                  "return type was unexpected",
                  op->action, pcmk__s(op->rsc, "unknown resource"));
-        services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE,
-                             "systemd DBus method had unexpected reply");
+
+        if (!op->synchronous && start_stop) {
+            /* The start or stop job is enqueued but is not complete. We need a
+             * job path to detect completion in job_removed_filter().
+             */
+            services__set_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
+                                 reason);
+
+        } else {
+            /* Something weird happened, but the action is finished and there
+             * was no D-Bus error. So call it a success.
+             */
+            services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, reason);
+        }
 
     } else {
         const char *path = NULL;
@@ -759,12 +845,119 @@ process_unit_method_reply(DBusMessage *reply, svc_action_t *op)
         dbus_message_get_args(reply, NULL,
                               DBUS_TYPE_OBJECT_PATH, &path,
                               DBUS_TYPE_INVALID);
+
         crm_debug("DBus request for %s of %s using %s succeeded",
                   op->action, pcmk__s(op->rsc, "unknown resource"), path);
-        services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
+
+        if (!op->synchronous && start_stop) {
+            // Should be set to unknown/pending already
+            services__set_result(op, PCMK_OCF_UNKNOWN, PCMK_EXEC_PENDING, NULL);
+            pcmk__str_update(&(op->opaque->job_path), path);
+
+        } else {
+            services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
+        }
     }
 }
 
+/*!
+ * \internal
+ * \brief Process a systemd \c JobRemoved signal for a given service action
+ *
+ * This filter is expected to be added with \c finalize_async_action_dbus() as
+ * the \c free_data_function. Then if \p message is a \c JobRemoved signal for
+ * the action specified by \p user_data, the action's result is set, the filter
+ * is removed, and the action is finalized.
+ *
+ * \param[in,out] connection  D-Bus connection
+ * \param[in]     message     D-Bus message
+ * \param[in,out] user_data   Service action (\c svc_action_t)
+ *
+ * \retval \c DBUS_HANDLER_RESULT_HANDLED if \p message is a \c JobRemoved
+ *         signal for \p user_data
+ * \retval \c DBUS_HANDLER_RESULT_NOT_YET_HANDLED otherwise (on error, if
+ *         \p message is not a \c JobRemoved signal, or if the signal is for
+ *         some other action's job)
+ */
+static DBusHandlerResult
+job_removed_filter(DBusConnection *connection, DBusMessage *message,
+                   void *user_data)
+{
+    svc_action_t *action = user_data;
+    const char *action_name = NULL;
+    uint32_t job_id = 0;
+    const char *bus_path = NULL;
+    const char *unit_name = NULL;
+    const char *result = NULL;
+    DBusError error;
+
+    CRM_CHECK((connection != NULL) && (message != NULL),
+              return DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+
+    // action should always be set when the filter is added
+    if ((action == NULL)
+        || !dbus_message_is_signal(message, BUS_NAME_MANAGER, "JobRemoved")) {
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    dbus_error_init(&error);
+    if (!dbus_message_get_args(message, &error,
+                               DBUS_TYPE_UINT32, &job_id,
+                               DBUS_TYPE_OBJECT_PATH, &bus_path,
+                               DBUS_TYPE_STRING, &unit_name,
+                               DBUS_TYPE_STRING, &result,
+                               DBUS_TYPE_INVALID)) {
+        crm_err("Could not interpret systemd DBus signal: %s " QB_XS " (%s)",
+                error.message, error.name);
+        dbus_error_free(&error);
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    if (!pcmk__str_eq(bus_path, action->opaque->job_path, pcmk__str_none)) {
+        // This filter is not for this job
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    action_name = pcmk__s(action->action, "(unknown)");
+
+    crm_trace("Setting %s result for %s (JobRemoved id=%" PRIu32 ", result=%s",
+              action_name, unit_name, job_id, result);
+
+    if (pcmk__str_eq(result, "done", pcmk__str_none)) {
+        services__set_result(action, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
+
+    } else if (pcmk__str_eq(result, "timeout", pcmk__str_none)) {
+        services__format_result(action, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_TIMEOUT,
+                                "systemd %s job for %s timed out",
+                                action_name, unit_name);
+
+    } else {
+        services__format_result(action, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
+                                "systemd %s job for %s failed with result '%s'",
+                                action_name, unit_name, result);
+    }
+
+    /* This instance of the filter was specifically for the given action.
+     *
+     * The action gets finalized by services__finalize_async_op() via the
+     * filter's free_data_function.
+     */
+    dbus_connection_remove_filter(systemd_proxy, job_removed_filter, action);
+    return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+/*!
+ * \internal
+ * \brief \c DBusFreeFunction wrapper for \c services__finalize_async_op()
+ *
+ * \param[in,out] action  Asynchronous service action to finalize
+ */
+static void
+finalize_async_action_dbus(void *action)
+{
+    services__finalize_async_op((svc_action_t *) action);
+}
+
 /*!
  * \internal
  * \brief Process the completion of an asynchronous unit start, stop, or restart
@@ -789,15 +982,35 @@ unit_method_complete(DBusPendingCall *pending, void *user_data)
     CRM_LOG_ASSERT(pending == op->opaque->pending);
     services_set_op_pending(op, NULL);
 
-    // Determine result and finalize action
     process_unit_method_reply(reply, op);
-    services__finalize_async_op(op);
+
     if (reply != NULL) {
         dbus_message_unref(reply);
     }
-}
 
-#define SYSTEMD_OVERRIDE_ROOT "/run/systemd/system/"
+    if ((op->status == PCMK_EXEC_PENDING)
+        && pcmk__strcase_any_of(op->action, PCMK_ACTION_START, PCMK_ACTION_STOP,
+                                NULL)) {
+        /* Start and stop method calls return when the job is enqueued, not when
+         * it's complete. Start and stop actions must be finalized after the job
+         * is complete, because the action callback function may use it. We add
+         * a message filter to process the JobRemoved signal, which indicates
+         * completion.
+         *
+         * The filter takes ownership of op, which will be finalized when the
+         * filter is later removed.
+         */
+        if (dbus_connection_add_filter(systemd_proxy, job_removed_filter, op,
+                                       finalize_async_action_dbus)) {
+            return;
+        }
+        crm_err("Could not add D-Bus filter for systemd JobRemoved signals");
+        services__set_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_ERROR,
+                             "Failed to add D-Bus filter for systemd "
+                             "JobRemoved signal");
+    }
+    services__finalize_async_op(op);
+}
 
 /* When the cluster manages a systemd resource, we create a unit file override
  * to order the service "before" pacemaker. The "before" relationship won't
@@ -808,93 +1021,167 @@ unit_method_complete(DBusPendingCall *pending, void *user_data)
  *
  * @TODO Add start timeout
  */
-#define SYSTEMD_OVERRIDE_TEMPLATE                           \
+#define SYSTEMD_UNIT_OVERRIDE_TEMPLATE                      \
     "[Unit]\n"                                              \
     "Description=Cluster Controlled %s\n"                   \
-    "Before=pacemaker.service pacemaker_remote.service\n"   \
+    "Before=pacemaker.service pacemaker_remote.service\n"
+
+#define SYSTEMD_SERVICE_OVERRIDE                            \
     "\n"                                                    \
     "[Service]\n"                                           \
     "Restart=no\n"
 
-// Temporarily use rwxr-xr-x umask when opening a file for writing
-static FILE *
-create_world_readable(const char *filename)
+/*!
+ * \internal
+ * \brief Get runtime drop-in directory path for a systemd unit
+ *
+ * \param[in] unit_name  Systemd unit (with extension)
+ *
+ * \return Drop-in directory path
+ */
+static GString *
+get_override_dir(const char *unit_name)
 {
-    mode_t orig_umask = umask(S_IWGRP | S_IWOTH);
-    FILE *fp = fopen(filename, "w");
+    GString *buf = g_string_sized_new(128);
 
-    umask(orig_umask);
-    return fp;
+    pcmk__g_strcat(buf, "/run/systemd/system/", unit_name, ".d", NULL);
+    return buf;
 }
 
-static void
-create_override_dir(const char *agent)
+/*!
+ * \internal
+ * \brief Append systemd override filename to a directory path
+ *
+ * \param[in,out] buf  Buffer containing directory path to append to
+ */
+static inline void
+append_override_basename(GString *buf)
 {
-    char *override_dir = crm_strdup_printf(SYSTEMD_OVERRIDE_ROOT
-                                           "/%s.service.d", agent);
-    int rc = pcmk__build_path(override_dir, 0755);
-
-    if (rc != pcmk_rc_ok) {
-        crm_warn("Could not create systemd override directory %s: %s",
-                 override_dir, pcmk_rc_str(rc));
-    }
-    free(override_dir);
+    g_string_append(buf, "/50-pacemaker.conf");
 }
 
-static char *
-get_override_filename(const char *agent)
-{
-    return crm_strdup_printf(SYSTEMD_OVERRIDE_ROOT
-                             "/%s.service.d/50-pacemaker.conf", agent);
-}
-
-static void
+/*!
+ * \internal
+ * \brief Create a runtime override file for a systemd unit
+ *
+ * The systemd daemon is then reloaded. This file does not survive a reboot.
+ *
+ * \param[in] agent    Systemd resource agent
+ * \param[in] timeout  Timeout for systemd daemon reload
+ *
+ * \return Standard Pacemaker return code
+ *
+ * \note Any configuration in \c /etc takes precedence over our drop-in.
+ * \todo Document this in Pacemaker Explained or Administration?
+ */
+static int
 systemd_create_override(const char *agent, int timeout)
 {
-    FILE *file_strm = NULL;
-    char *override_file = get_override_filename(agent);
+    char *unit_name = NULL;
+    GString *filename = NULL;
+    GString *override = NULL;
+    FILE *fp = NULL;
+    int fd = 0;
+    int rc = pcmk_rc_ok;
 
-    create_override_dir(agent);
+    unit_name = systemd_unit_name(agent, false);
+    CRM_CHECK(!pcmk__str_empty(unit_name),
+              rc = EINVAL; goto done);
 
-    /* Ensure the override file is world-readable. This is not strictly
-     * necessary, but it avoids a systemd warning in the logs.
-     */
-    file_strm = create_world_readable(override_file);
-    if (file_strm == NULL) {
-        crm_err("Cannot open systemd override file %s for writing",
-                override_file);
-    } else {
-        char *override = crm_strdup_printf(SYSTEMD_OVERRIDE_TEMPLATE, agent);
-
-        int rc = fprintf(file_strm, "%s\n", override);
-
-        free(override);
-        if (rc < 0) {
-            crm_perror(LOG_WARNING, "Cannot write to systemd override file %s",
-                       override_file);
-        }
-        fflush(file_strm);
-        fclose(file_strm);
-        systemd_daemon_reload(timeout);
+    filename = get_override_dir(unit_name);
+    rc = pcmk__build_path(filename->str, 0755);
+    if (rc != pcmk_rc_ok) {
+        crm_err("Could not create systemd override directory %s: %s",
+                filename->str, pcmk_rc_str(rc));
+        goto done;
     }
 
-    free(override_file);
+    append_override_basename(filename);
+    fp = fopen(filename->str, "w");
+    if (fp == NULL) {
+        rc = errno;
+        crm_err("Cannot open systemd override file %s for writing: %s",
+                filename->str, pcmk_rc_str(rc));
+        goto done;
+    }
+
+    // Ensure the override file is world-readable (avoid systemd warning in log)
+    fd = fileno(fp);
+    if ((fd < 0) || (fchmod(fd, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH) < 0)) {
+        rc = errno;
+        crm_err("Failed to set permissions on systemd override file %s: %s",
+                filename->str, pcmk_rc_str(rc));
+        goto done;
+    }
+
+    override = g_string_sized_new(2 * sizeof(SYSTEMD_UNIT_OVERRIDE_TEMPLATE));
+    g_string_printf(override, SYSTEMD_UNIT_OVERRIDE_TEMPLATE, unit_name);
+    if (pcmk__ends_with_ext(unit_name, ".service")) {
+        g_string_append(override, SYSTEMD_SERVICE_OVERRIDE);
+    }
+
+    if (fputs(override->str, fp) == EOF) {
+        rc = EIO;
+        crm_err("Cannot write to systemd override file %s", filename->str);
+    }
+
+done:
+    if (fp != NULL) {
+        fclose(fp);
+    }
+
+    if (rc == pcmk_rc_ok) {
+        // @TODO Make sure the reload succeeds
+        systemd_daemon_reload(timeout);
+
+    } else if (fp != NULL) {
+        // File was created, so remove it
+        unlink(filename->str);
+    }
+
+    free(unit_name);
+
+    // coverity[check_after_deref : FALSE]
+    if (filename != NULL) {
+        g_string_free(filename, TRUE);
+    }
+    if (override != NULL) {
+        g_string_free(override, TRUE);
+    }
+    return rc;
 }
 
 static void
 systemd_remove_override(const char *agent, int timeout)
 {
-    char *override_file = get_override_filename(agent);
-    int rc = unlink(override_file);
+    char *unit_name = systemd_unit_name(agent, false);
+    GString *filename = NULL;
+
+    CRM_CHECK(!pcmk__str_empty(unit_name), goto done);
+
+    filename = get_override_dir(unit_name);
+    append_override_basename(filename);
+
+    if (unlink(filename->str) < 0) {
+        int rc = errno;
+
+        if (rc != ENOENT) {
+            // Stop may be called when already stopped, which is fine
+            crm_warn("Cannot remove systemd override file %s: %s",
+                     filename->str, pcmk_rc_str(rc));
+        }
 
-    if (rc < 0) {
-        // Stop may be called when already stopped, which is fine
-        crm_perror(LOG_DEBUG, "Cannot remove systemd override file %s",
-                   override_file);
     } else {
         systemd_daemon_reload(timeout);
     }
-    free(override_file);
+
+done:
+    free(unit_name);
+
+    // coverity[check_after_deref : FALSE]
+    if (filename != NULL) {
+        g_string_free(filename, TRUE);
+    }
 }
 
 /*!
@@ -978,8 +1265,20 @@ invoke_unit_by_path(svc_action_t *op, const char *unit)
         return;
 
     } else if (pcmk__str_eq(op->action, PCMK_ACTION_START, pcmk__str_none)) {
+        int rc = pcmk_rc_ok;
+
         method = "StartUnit";
-        systemd_create_override(op->agent, op->timeout);
+        rc = systemd_create_override(op->agent, op->timeout);
+        if (rc != pcmk_rc_ok) {
+            services__format_result(op, pcmk_rc2ocf(rc), PCMK_EXEC_ERROR,
+                                    "Failed to create systemd override file "
+                                    "for %s",
+                                    pcmk__s(op->agent, "(unspecified)"));
+            if (!(op->synchronous)) {
+                services__finalize_async_op(op);
+            }
+            return;
+        }
 
     } else if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_none)) {
         method = "StopUnit";
@@ -1010,10 +1309,10 @@ invoke_unit_by_path(svc_action_t *op, const char *unit)
     /* (ss) */
     {
         const char *replace_s = "replace";
-        char *name = systemd_service_name(op->agent,
-                                          pcmk__str_eq(op->action,
-                                                       PCMK_ACTION_META_DATA,
-                                                       pcmk__str_none));
+        char *name = systemd_unit_name(op->agent,
+                                       pcmk__str_eq(op->action,
+                                                    PCMK_ACTION_META_DATA,
+                                                    pcmk__str_none));
 
         CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &name, DBUS_TYPE_INVALID));
         CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &replace_s, DBUS_TYPE_INVALID));
@@ -1056,7 +1355,14 @@ systemd_timeout_callback(gpointer p)
     services__format_result(op, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_TIMEOUT,
                             "%s action for systemd unit %s "
                             "did not complete in time", op->action, op->agent);
-    services__finalize_async_op(op);
+
+    if (op->opaque->job_path != NULL) {
+        // A filter owns this op
+        dbus_connection_remove_filter(systemd_proxy, job_removed_filter, op);
+
+    } else {
+        services__finalize_async_op(op);
+    }
     return FALSE;
 }
 
@@ -1081,7 +1387,7 @@ services__execute_systemd(svc_action_t *op)
 {
     pcmk__assert(op != NULL);
 
-    if ((op->action == NULL) || (op->agent == NULL)) {
+    if (pcmk__str_empty(op->action) || pcmk__str_empty(op->agent)) {
         services__set_result(op, PCMK_OCF_NOT_CONFIGURED, PCMK_EXEC_ERROR_FATAL,
                              "Bug in action caller");
         goto done;
@@ -1110,6 +1416,7 @@ services__execute_systemd(svc_action_t *op)
                          "Bug in service library");
 
     if (invoke_unit_by_name(op->agent, op, NULL) == pcmk_rc_ok) {
+        // @TODO Why plus 5000? No explanation in fccd046.
         op->opaque->timerid = pcmk__create_timer(op->timeout + 5000,
                                                  systemd_timeout_callback, op);
         services_add_inflight_op(op);
