@@ -295,9 +295,8 @@ static GOptionEntry addl_entries[] = {
       INDENT "   existing value and X are parsed and added as scores.\n\n"
 
       INDENT "Scores are integer values capped at INFINITY and -INFINITY.\n"
-      INDENT "Refer to Pacemaker Explained and to the char2score() function\n"
-      INDENT "for more details on scores, including how they're parsed and\n"
-      INDENT "added.",
+      INDENT "Refer to Pacemaker Explained for more details on scores,\n"
+      INDENT "including how they are parsed and added.",
       NULL },
 
     { "wait", 'W', 0, G_OPTION_ARG_CALLBACK, wait_cb,
@@ -328,6 +327,7 @@ static GOptionEntry deprecated_entries[] = {
       NULL, NULL
     },
 
+    // NOTE: resource-agents <4.2.0 (2018-10-24) uses this option
     { "attr-name", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_CALLBACK, attr_name_cb,
       NULL, NULL
     },
@@ -340,10 +340,12 @@ static GOptionEntry deprecated_entries[] = {
       NULL, NULL
     },
 
+    // NOTE: resource-agents <4.2.0 (2018-10-24) uses this option
     { "get-value", 0, G_OPTION_FLAG_HIDDEN|G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, value_cb,
       NULL, NULL
     },
 
+    // NOTE: resource-agents <4.2.0 (2018-10-24) uses this option
     { "node-uname", 'U', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &options.dest_uname,
       NULL, NULL
     },
@@ -471,7 +473,7 @@ command_delete(pcmk__output_t *out, cib_t *cib)
     }
 
 done_deleting:
-    free_xml(result);
+    pcmk__xml_free(result);
 
     if (rc == ENXIO) {
         /* Nothing to delete...
@@ -544,7 +546,7 @@ command_update(pcmk__output_t *out, cib_t *cib, int is_remote_node)
     }
 
 done_updating:
-    free_xml(result);
+    pcmk__xml_free(result);
     return rc;
 }
 
@@ -630,7 +632,7 @@ command_query(pcmk__output_t *out, cib_t *cib)
         output_one_attribute(result, &od);
     }
 
-    free_xml(result);
+    pcmk__xml_free(result);
     return rc;
 }
 
@@ -651,7 +653,7 @@ set_type(void)
             options.type = g_strdup(PCMK_XE_CRM_CONFIG);
         }
 
-    } else if (pcmk__str_eq(options.type, "reboot", pcmk__str_casei)) {
+    } else if (pcmk__str_eq(options.type, PCMK_VALUE_REBOOT, pcmk__str_casei)) {
         options.type = g_strdup(PCMK_XE_STATUS);
 
     } else if (pcmk__str_eq(options.type, "forever", pcmk__str_casei)) {
@@ -718,6 +720,7 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
           "Print only the value on stdout",
           NULL },
 
+        // NOTE: resource-agents <4.2.0 (2018-10-24) uses -Q
         { "quiet", 'Q', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &(args->quiet),
           NULL, NULL
         },
@@ -818,7 +821,7 @@ main(int argc, char **argv)
     }
 
     the_cib = cib_new();
-    rc = cib__signon_attempts(the_cib, crm_system_name, cib_command, 5);
+    rc = cib__signon_attempts(the_cib, cib_command, 5);
     rc = pcmk_legacy2rc(rc);
 
     if (rc != pcmk_rc_ok) {
@@ -937,7 +940,7 @@ main(int argc, char **argv)
         if (options.command == attr_cmd_delete) {
             update = "<none>";
         }
-        crm_info("Update %s=%s sent via pacemaker-attrd",
+        crm_info("Update %s=%s sent to the attribute manager",
                  options.attr_name, update);
 
     } else if (options.command == attr_cmd_delete) {
