@@ -553,7 +553,7 @@ remove_from_section(cib_t *cib, const char *element, const char *section,
     }
 
     rc = cib->cmds->remove(cib, section, xml, cib_transaction);
-    free_xml(xml);
+    pcmk__xml_free(xml);
     return (rc >= 0)? pcmk_rc_ok : pcmk_legacy2rc(rc);
 }
 
@@ -579,7 +579,7 @@ purge_node_from_cib(const char *node_name, long node_id)
     if (cib == NULL) {
         return ENOTCONN;
     }
-    rc = cib__signon_attempts(cib, crm_system_name, cib_command, 5);
+    rc = cib__signon_attempts(cib, cib_command, 5);
     if (rc == pcmk_ok) {
         rc = cib->cmds->init_transaction(cib);
     }
@@ -686,8 +686,8 @@ purge_node_from_fencer(const char *node_name, long node_id)
         return rc;
     }
 
-    cmd = create_request(CRM_OP_RM_NODE_CACHE, NULL, NULL, "stonith-ng",
-                         crm_system_name, NULL);
+    cmd = pcmk__new_request(pcmk_ipc_fenced, crm_system_name, NULL,
+                            PCMK__VALUE_STONITH_NG, CRM_OP_RM_NODE_CACHE, NULL);
     if (node_id > 0) {
         crm_xml_add_ll(cmd, PCMK_XA_ID, node_id);
     }
@@ -703,7 +703,7 @@ purge_node_from_fencer(const char *node_name, long node_id)
         fprintf(stderr, "Could not purge node %s from fencer: %s\n",
                 pcmk__s(node_name, "by ID"), pcmk_rc_str(rc));
     }
-    free_xml(cmd);
+    pcmk__xml_free(cmd);
     crm_ipc_close(conn);
     crm_ipc_destroy(conn);
     return rc;
